// dialogs.c
// LiVES (lives-exe)
// (c) G. Finch 2003
// Released under the GPL 2.0
// see file ../COPYING for licensing details

#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "interface.h"
#include "main.h"


// for ping-pong loops
extern void on_dirchange_activate            (GtkMenuItem     *menuitem,
					      gpointer         user_data);



gint
do_warning_dialog(const gchar *text) {
  return do_warning_dialog_with_check(text, 0);
}


gint
do_warning_dialog_with_check(const gchar *text, gint warn_mask_number) {
  // show OK/CANCEL, returns FALSE if cancelled
  GtkWidget *warning;
  gint response;

  if (prefs->warning_mask&warn_mask_number) {
    return TRUE;
  }

  warning=create_dialog2(warn_mask_number);
  gtk_label_set_text(GTK_LABEL(mainw->warning_label),text);

  response=gtk_dialog_run (GTK_DIALOG (warning));
  gtk_widget_destroy (warning);

  if (mainw->warn_mask==TRUE) {
    prefs->warning_mask|=warn_mask_number;
    set_int_pref("lives_warning_mask",prefs->warning_mask);
  }
  return (response==GTK_RESPONSE_OK);
}



void 
do_error_dialog(const gchar *text) {
  // show error/info box

  GtkWidget *err_box=create_dialog3(text,FALSE);
  gtk_widget_show(err_box);
}


void 
do_blocking_error_dialog(const gchar *text) {
  // show error/info box - blocks until OK is pressed
  GtkWidget *err_box=create_dialog3(text,TRUE);
  gtk_dialog_run(GTK_DIALOG (err_box));
  gtk_widget_destroy(err_box);

  if (mainw!=NULL&&mainw->is_ready==TRUE) {
    gtk_widget_queue_draw(mainw->LiVES);
  }
}



void 
do_memory_error_dialog (void) {
  do_error_dialog ("\n\nLiVES was unable to perform this operation due to unsufficient memory.\nPlease try closing some other applications first.\n");
}


gboolean do_progress_dialog(gboolean visible, gboolean cancellable, const gchar *text) {
  // monitor progress, return FALSE if the operation was cancelled
  gint oframeno=0;
  FILE *infofile;
  file *proc_file=cfile;

  mainw->startticks=0l;
  mainw->startsecs=0l;
  gdouble audio_start=mainw->play_start;
  gboolean accelerators_swapped=FALSE;
  gint first_frame,last_frame;
  gboolean finished=FALSE;

  mainw->currticks=0l;

  if (!visible) {
    if (!(cfile->events[EVENT_FPS]==NULL)) {
      // this is for audio, work out the apparent frame when we have variable fps
      audio_start=calc_variable_time(mainw->play_start)*cfile->fps;
    }
  }

  if (prefs->show_player_stats) {
    mainw->fps_measure=0.;
  }

  mainw->cancelled=0;   
  mainw->error=FALSE;
  clear_mainw_msg();

  if (visible) {
    mainw->is_processing=TRUE;
    desensitize();
    procw_desensitize();
    cfile->proc_ptr=create_processing (text);
    
    gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar),.01);
    gtk_widget_show(cfile->proc_ptr->processing);
    
    cfile->proc_ptr->frames_done=0;
    
    if (cancellable) {
      gtk_widget_show (cfile->proc_ptr->cancel_button);
    }
    if (cfile->opening==TRUE&&capable->has_sox&&mainw->playing_file==-1) {
      gtk_widget_show (cfile->proc_ptr->preview_button);
      // gtk_widget_remove_accelerator (mainw->playall, mainw->accel_group, GDK_p, 0);
      //gtk_widget_add_accelerator (cfile->proc_ptr->preview_button, "clicked", mainw->accel_group, GDK_p, 0, 0);
      accelerators_swapped=TRUE;
    }
  }
  
  if (prefs->video_player!=VID_PLAYER_MPLAYER&&!visible) {
      cfile->last_frameno=cfile->frameno=mainw->play_start;

      mainw->deltaticks=0;
      gettimeofday(tv, tz);

      mainw->startsecs=tv->tv_sec;
      // mainw->origticks is our base for quantising
      mainw->origticks=mainw->firstticks=mainw->startticks=1000000l*(tv->tv_sec-mainw->startsecs)+tv->tv_usec;
  }

  //try to open info file - or if internal_messaging is TRUE, we get mainw->msg
  // from the mainw->progress_fn function
  do {
    while (!((infofile=fopen(cfile->info_file,"r"))||(mainw->internal_messaging&&(*mainw->progress_fn)(FALSE)))) {
      if (prefs->video_player!=VID_PLAYER_MPLAYER&&!visible) {
	// INTERNAL PLAYER
	last_frame=mainw->playing_sel?cfile->end:mainw->play_end;
	first_frame=mainw->playing_sel?cfile->start:1;

	gettimeofday(tv, tz);
	mainw->currticks=1000000l*(tv->tv_sec-mainw->startsecs)+tv->tv_usec;
	if (cfile->play_paused==FALSE) {
	  oframeno=cfile->frameno;
	  // see if we reached the start (backwards...)
	  if ((cfile->frameno+=(gint)((mainw->currticks-mainw->startticks+mainw->deltaticks)/mainw->period))<first_frame) {
	    // loop backwards...
	    if (mainw->ping_pong==TRUE&&mainw->pb_fps<0.) {
	      on_dirchange_activate (NULL,NULL);
	      if ((cfile->frameno=2*first_frame-cfile->frameno)<first_frame) cfile->frameno=first_frame;
	      if ((oframeno=2*first_frame-oframeno)<first_frame) oframeno=first_frame;
	    }
	    else {
	      oframeno=(cfile->frameno=last_frame)+1;
	      mainw->startticks=mainw->currticks+mainw->deltaticks-(last_frame-first_frame)*mainw->period;
	    }
	  }

	  // check if we are done
	  if ((cfile->frameno>last_frame)&&last_frame>0) {
	    if (mainw->whentostop==STOP_ON_VID_END) {
	      mainw->cancelled=1;
	    }
	    else {
	      // loop forwards
	      if (mainw->playing_sel) mainw->play_start=cfile->start;
	      if (mainw->ping_pong==TRUE&&mainw->pb_fps>0.) {
		on_dirchange_activate (NULL,NULL);
		if ((cfile->frameno=2*last_frame-cfile->frameno)>last_frame) cfile->frameno=last_frame;
		if ((oframeno=2*(last_frame)-oframeno)>last_frame) oframeno=last_frame+1;
	      }
	      else {
		oframeno=(cfile->frameno=first_frame)-1;
		// loop performance
		if (-cfile->perf_counter>cfile->perf_end&&(mainw->loop_cont==TRUE||mainw->loop==TRUE)) {
		  cfile->perf_counter=-mainw->play_start;
		}
		if (cfile->perf_counter<0&&(mainw->loop==TRUE||mainw->loop_cont==TRUE)&&mainw->playing_sel==TRUE) {
		cfile->perf_counter=-mainw->play_start;
		}
	      }
	    }
	  }

	  // play next frame
	  if (!mainw->cancelled) {
	    // calculate the audio 'frame'
	    mainw->aframeno=(mainw->currticks-mainw->firstticks)*cfile->fps/1000000l+audio_start;
	    if (mainw->loop_cont==TRUE&&(mainw->aframeno>(mainw->audio_end?mainw->audio_end:cfile->laudio_time*cfile->fps))) {
	      mainw->firstticks=mainw->currticks;
	    }
	    if (cfile->frameno!=oframeno) {
	      load_frame_image(cfile->frameno,oframeno);
	      mainw->startticks+=(cfile->frameno-oframeno)*mainw->period;
	    }
	  }
	}
	// paused
	else {
	  mainw->startticks=mainw->currticks;
	  if (mainw->ext_playback) (*vid_playback_plugin->send_keycodes)();
	}
      }
      if (visible) {
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar));
      }
      if (visible||mainw->fs==FALSE||prefs->video_player!=VID_PLAYER_MPLAYER)
	while (g_main_context_iteration(NULL,FALSE));

      if (mainw->cancelled) {
	if (prefs->show_player_stats&&!visible&&mainw->fps_measure>0.) {
	    // statistics
	    gettimeofday(tv, tz);
	    mainw->fps_measure/=(gdouble)(1000000l*(tv->tv_sec-mainw->startsecs)+tv->tv_usec-mainw->origticks)/1000000.;
	}
	if (visible) {
	  if (accelerators_swapped) {
	    //gtk_widget_remove_accelerator (cfile->proc_ptr->preview_button, mainw->accel_group, GDK_p, 0);
	    //gtk_widget_add_accelerator (mainw->playall, "activate", mainw->accel_group,GDK_p, 0, GTK_ACCEL_VISIBLE);
	  }
	  if (proc_file->proc_ptr!=NULL) {
	    gtk_widget_destroy(GTK_WIDGET(proc_file->proc_ptr->processing));
	    g_free(proc_file->proc_ptr);
	    proc_file->proc_ptr=NULL;
	  }
	  mainw->is_processing=FALSE;
	  if (!(cfile->menuentry==NULL)) {
	      sensitize();
	  }
	}
	else mainw->is_processing=TRUE;
	unlink(cfile->info_file);
	return FALSE;
      }
      if (visible) {
	g_usleep(prefs->sleep_time);
      }
    }

    if (infofile) {
      // OK, now we might have some frames
      fgets(mainw->msg,512,infofile);
      fclose(infofile);
    }

#if defined DEBUG
    g_print("msg %s\n",mainw->msg);
#endif

    if (visible&&cancellable&&cfile->nopreview==FALSE) {
      gtk_widget_show (cfile->proc_ptr->pause_button);
      gtk_widget_show (cfile->proc_ptr->preview_button);
      //gtk_widget_remove_accelerator (mainw->playall, mainw->accel_group, GDK_p, 0);
      //gtk_widget_add_accelerator (cfile->proc_ptr->preview_button, "clicked", mainw->accel_group, GDK_p, 0, 0);
      accelerators_swapped=TRUE;
    }

    if (strncmp(mainw->msg,"completed",9)&&strncmp(mainw->msg,"error",5)&&(visible||((strncmp(mainw->msg,"video_ended",11)||mainw->whentostop!=STOP_ON_VID_END)&&(strncmp(mainw->msg,"audio_ended",11)||mainw->preview==TRUE||mainw->whentostop!=STOP_ON_AUD_END)))) {
      // processing not yet completed...

      if (visible) {
	// last frame processed ->> will go from cfile->start to cfile->end
	cfile->proc_ptr->frames_done=atoi(mainw->msg);
      }

      if (prefs->video_player!=VID_PLAYER_MPLAYER&&!visible) {
	// INTERNAL PLAYER
	last_frame=mainw->playing_sel?cfile->end:mainw->play_end;
	first_frame=mainw->playing_sel?cfile->start:1;

	gettimeofday(tv, tz);
	mainw->currticks=1000000l*(tv->tv_sec-mainw->startsecs)+tv->tv_usec;
	if (cfile->play_paused==FALSE) {
	  oframeno=cfile->frameno;
	  // see if we reached the start (backwards...)
	  if ((cfile->frameno+=(gint)((mainw->currticks-mainw->startticks+mainw->deltaticks)/mainw->period))<(mainw->playing_sel?cfile->start:1)) {
	    // loop backwards...
	    if (mainw->ping_pong==TRUE&&mainw->pb_fps<0.) {
	      on_dirchange_activate (NULL,NULL);
	      cfile->frameno=2*(mainw->playing_sel?cfile->start:1)-cfile->frameno;
	      oframeno=2*(mainw->playing_sel?cfile->start:1)-oframeno;
	    }
	    else {
	      oframeno=(cfile->frameno=mainw->playing_sel?cfile->end:mainw->play_end)+1;
	      mainw->startticks=mainw->currticks+mainw->deltaticks-(mainw->playing_sel==TRUE?cfile->end-cfile->start:mainw->play_end-1)*mainw->period;
	    }
	  }
	  // check if we are done
	  if (cfile->frameno>last_frame||(-cfile->perf_counter>cfile->perf_end&&cfile->perf_end>mainw->play_end)) {
	    if (mainw->whentostop!=STOP_ON_VID_END) {
	      if (mainw->playing_sel) mainw->play_start=cfile->start;
	      // loop forwards
	      if (mainw->ping_pong==TRUE&&mainw->pb_fps>0.) {
		on_dirchange_activate (NULL,NULL);
	      if ((cfile->frameno=2*first_frame-cfile->frameno)<first_frame) cfile->frameno=first_frame;
	      if ((oframeno=2*first_frame-oframeno)<first_frame) oframeno=first_frame;
	      }
	      else {
		oframeno=(cfile->frameno=mainw->play_start)-1;
		// loop performance
		if (mainw->loop_cont==TRUE&&-cfile->perf_counter>cfile->perf_end) {
		  cfile->perf_counter=-mainw->play_start;
		}
		if (cfile->perf_counter<0&&mainw->loop==TRUE&&mainw->playing_sel==TRUE) {
		cfile->perf_counter=-mainw->play_start;
		}
	      }
	    }
	    else {
	      mainw->cancelled=1;
	    }
	  }
	  // play next frame
	  if (!mainw->cancelled) {
	    mainw->aframeno=(mainw->currticks-mainw->firstticks)*cfile->fps/1000000l+audio_start;
	    if (mainw->loop_cont==TRUE&&(mainw->aframeno>(mainw->audio_end?mainw->audio_end:cfile->laudio_time*cfile->fps))) {
	      mainw->firstticks=mainw->currticks;
	    }
	    if (cfile->frameno!=oframeno) {
	      load_frame_image(cfile->frameno,oframeno);
	      mainw->startticks+=(cfile->frameno-oframeno)*mainw->period;
	    }
	  }
	      }
	// paused
	else {
	  mainw->startticks=mainw->currticks;
	  if (mainw->ext_playback) (*vid_playback_plugin->send_keycodes)();
	}
      }
	
      // set the progress bar %
      if (visible&&cfile->proc_ptr->frames_done>=cfile->progress_start&&cfile->proc_ptr->frames_done<=cfile->progress_end) gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar),(gdouble)(cfile->proc_ptr->frames_done-cfile->progress_start)/(gdouble)(cfile->progress_end-cfile->progress_start+1));
      if (visible||mainw->fs==FALSE||prefs->video_player!=VID_PLAYER_MPLAYER)
	while (g_main_context_iteration(NULL,FALSE));
      if (mainw->cancelled) {
	if (prefs->show_player_stats&&!visible&&mainw->fps_measure>0.) {
	  gettimeofday(tv, tz);
	  mainw->fps_measure/=(gdouble)(1000000l*(tv->tv_sec-mainw->startsecs)+tv->tv_usec-mainw->origticks)/1000000.;
	}
	if (visible) {
	  if (accelerators_swapped) {
	    //gtk_widget_remove_accelerator (cfile->proc_ptr->preview_button, mainw->accel_group, GDK_p, 0);
	    //gtk_widget_add_accelerator (mainw->playall, "activate", mainw->accel_group, GDK_p, 0, GTK_ACCEL_VISIBLE);
	  }
	  if (proc_file->proc_ptr!=NULL) {
	    gtk_widget_destroy(proc_file->proc_ptr->processing);
	    g_free(proc_file->proc_ptr);
	    proc_file->proc_ptr=NULL;
	  }
	  mainw->is_processing=FALSE;
	  if (!(cfile->menuentry==NULL)) {
	    sensitize();
	  }
	}
	else mainw->is_processing=TRUE;
	unlink(cfile->info_file);
	return FALSE;
      }
      if (visible) {
	g_usleep(prefs->sleep_time);
      }
    }
    else finished=TRUE;
  } while (!finished);

#if defined DEBUG
  g_print ("exit pt 3\n");
#endif

  //play/operation ended
    if (visible) {
      unlink(cfile->info_file);
      if (accelerators_swapped) {
	//gtk_widget_remove_accelerator (cfile->proc_ptr->preview_button, mainw->accel_group, GDK_p, 0);
	//gtk_widget_add_accelerator (mainw->playall, "activate", mainw->accel_group, GDK_p, 0, GTK_ACCEL_VISIBLE);
      }
      gtk_widget_destroy(proc_file->proc_ptr->processing);
      g_free(proc_file->proc_ptr);
      proc_file->proc_ptr=NULL;
      mainw->is_processing=FALSE;
      if (!(cfile->menuentry==NULL)) {
	// note - for operations to/from clipboard (file 0) we
	// should manually call sensitize() after operation
	sensitize();
      }
    }
    else {
      if (prefs->show_player_stats) {
	if (mainw->fps_measure>0.) {
	  gettimeofday(tv, tz);
	  mainw->fps_measure/=(gdouble)(1000000l*(tv->tv_sec-mainw->startsecs)+tv->tv_usec-mainw->origticks)/1000000.;
	}
      }
      mainw->is_processing=TRUE;
    }
    
    // get error message (if any)
    if (!strncmp(mainw->msg,"error",5)) {
      int i;
      gchar **array;
      gint numtok=get_token_count (mainw->msg,'|');
  
      array=g_strsplit(mainw->msg,"|",numtok);
      g_snprintf(mainw->msg,512,"\n\n");
      for (i=1;i<numtok;i++) {
	g_strappend(mainw->msg,512,array[i]);
	g_strappend(mainw->msg,512,"\n");
      }
      g_strfreev(array);
      mainw->error=TRUE;
    }
#if defined DEBUG
    g_print("exiting progress dialogue\n");
#endif
    return TRUE;
}



void 
do_auto_dialog (const gchar *text) {
  FILE *infofile;

  gint count=100000/prefs->sleep_time;  // don't want to flash too fast...

  cfile->proc_ptr=create_processing (text);
  gtk_widget_hide (cfile->proc_ptr->stop_button);
  gtk_window_set_modal (GTK_WINDOW (cfile->proc_ptr->processing), TRUE);
      
  gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar),.01);
  gtk_widget_show(cfile->proc_ptr->processing);
  
  clear_mainw_msg();

  while (!(infofile=fopen(cfile->info_file,"r"))) {
    gtk_progress_bar_pulse(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar));
    while (g_main_context_iteration(NULL,FALSE));
    g_usleep(prefs->sleep_time);
    count--;
  }
  
  fgets(mainw->msg,512,infofile);
  fclose(infofile);
  unlink(cfile->info_file);

  while (count>0) {
    gtk_progress_bar_pulse(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar));
    while (g_main_context_iteration(NULL,FALSE));
    g_usleep(prefs->sleep_time);
    count--;
  }

  gtk_widget_destroy(cfile->proc_ptr->processing);
  g_free(cfile->proc_ptr);
  cfile->proc_ptr=NULL;

  // get error message (if any)
  if (!strncmp(mainw->msg,"error",5)) {
    gchar **array=g_strsplit(mainw->msg,"|",5);
    g_snprintf(mainw->msg,512,"\n\n%s\n\n%s\n\n%s\n\n%s\n",array[1],array[2],array[3],array[4]);
    g_strfreev(array);
    mainw->error=TRUE;
  }
}


gboolean 
do_socket_dialog (lives_socket *v, const gchar *text, const char *host, int port) {
  gint count=1000000/prefs->sleep_time;  // minumum of 1 second - we don't want to flash too fast...
  gint timeout=SOCKET_TIMEOUT*1000000; // timeout

  cfile->proc_ptr=create_processing (text);
  gtk_window_set_modal (GTK_WINDOW (cfile->proc_ptr->processing), TRUE);
      
  gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar),.01);
  gtk_widget_show(cfile->proc_ptr->processing);
  
  while (socket_connect (v, host, port)==FALSE&&count<timeout) {
    gtk_progress_bar_pulse(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar));
    while (g_main_context_iteration(NULL,FALSE));
    g_usleep(prefs->sleep_time);
    count--;
  }
  // TODO - this is rubbish, fix it (but this code is not used yet)
  if (count>=timeout) {
    // network timed out
    return FALSE;
  }

  gtk_widget_destroy(cfile->proc_ptr->processing);
  g_free(cfile->proc_ptr);
  cfile->proc_ptr=NULL;

  return TRUE;
}







void
too_many_files(void) {
  gchar *warn=g_strdup_printf("\nSorry, LiVES can only open %d files at once.\nPlease close a file and then try again.",MAX_FILES);
  do_error_dialog(warn);
  g_free(warn);
}

void
tempdir_warning (void) {
    gchar *com=g_strdup_printf("LiVES was unable to write to its temporary directory.\n\nThe current temporary directory is:\n\n%s\n\nPlease make sure you can write to this directory.",prefs->tmpdir);
    if (mainw!=NULL&&mainw->is_ready) {
      do_error_dialog(com);
    }
    g_free(com);
}


void
do_encoder_fps_error(gdouble fps_restrict) {
  gchar *msg;

  if (fps_restrict==-1) {
    msg=g_strdup_printf ("%s can only save %s at integer fps values.\n\nPlease adjust the playback speed from Tools|Resample Video\nor switch to another save format in Preferences",prefs->encoder.name,prefs->encoder.of_name);
  }
  else {
    msg=g_strdup_printf ("%s can only save %s at \n%.3f frames per second.\n\nPlease adjust the playback speed from Tools|Resample Video\nor switch to another save format in Preferences.",prefs->encoder.name,prefs->encoder.of_name,fps_restrict);
  }
  do_error_dialog(msg);
  g_free(msg);
  return;
}


void
do_encoder_size_error(gint width, gint height) {
  gchar *msg;

  msg=g_strdup_printf ("%s can only save %s at a frame size of \n%dX%d.\n\nPlease adjust the frame size from Tools|Resize All\nor switch to another save format in Preferences.",prefs->encoder.name,prefs->encoder.of_name,width,height);
  do_error_dialog(msg);
  g_free(msg);
  return;
}


void
perf_mem_warning(void) {
  do_error_dialog("\n\nLiVES was unable to record a performance. There is currently insufficient memory available.\nTry recording for just a selection of the file.");
}

gint
do_clipboard_fps_warning(void) {
  if (prefs->warning_mask&WARN_MASK_FPS) {
    return TRUE;
  }
  return do_warning_dialog_with_check("The playback speed (fps), or the audio rate\n of the clipboard does not match\nthe playback speed or audio rate of the clip you are inserting into.\n\nThe insertion will be adjusted to fit into the clip.\n\nPlease press Cancel to abort the insert, or OK to continue.",WARN_MASK_FPS);
}



gboolean
do_comments_dialog (void) {
  gboolean response;

  commentsw=create_comments_dialog();
  if ((response=(gtk_dialog_run(GTK_DIALOG (commentsw->comments_dialog))==GTK_RESPONSE_OK))) {
    g_snprintf (cfile->title,256,"%s",gtk_entry_get_text (GTK_ENTRY (commentsw->title_entry)));
    g_snprintf (cfile->author,256,"%s",gtk_entry_get_text (GTK_ENTRY (commentsw->author_entry)));
    g_snprintf (cfile->comment,256,"%s",gtk_entry_get_text (GTK_ENTRY (commentsw->comment_entry)));
  }
  gtk_widget_destroy (commentsw->comments_dialog);
  g_free (commentsw);
  
  return response;
}


void 
do_keys_dialog (void) {
  do_text_window ("Show Keys","You can use the following keys during playback to control LiVES:-\n\nRecordable keys\n-----------------------\nctrl-left                     skip back\nctrl-right                   skip forwards\nctrl-up                      faster/increase effect\nctrl-down                 slower/decrease effect\nctrl-space                reverse direction\nctrl-backspace         freeze frame\nn                             nervous\n\n\nOther VJ keys\n--------------------\nctrl-page up            previous clip\nctrl-page down        next clip\n\nctrl-1                       toggle real-time effect 1\nctrl-2                       toggle real-time effect 2\n                                          ...etc...\nctrl-0                       real-time effects off\n\nk            grab keyboard for last grabable effect\nx         switch effect mode (when effect has keyboard grab)\n\nf1                           store/switch to clip mnemonic 1\nf2                           store/switch to clip mnemonic 2\n                                          ...etc...\nf12                          clear function keys\n\n\n Other playback keys\n-----------------------------\np                             play all\ny                             play selection\nq                             stop\nf                               fullscreen\ns                              separate window\nd                             double size\ng                             ping pong loops\n");

}


void 
do_text_window (const gchar *title, const gchar *text) {
  GtkWidget *text_window=create_text_window (title,text);
  gtk_widget_show (text_window);
}



void 
do_firstever_dialog (void) {
  gtk_init_add(startup_message_nonfatal, g_strdup ("\n\nWelcome to LiVES. If this is your first time using LiVES, please check all the\nPreferences using Control-P. In particular make sure the temporary directory is located somewhere with plenty of diskspace.\nHave fun !\n"));
}


void 
do_upgrade_error_dialog (void) {
  gtk_init_add(startup_message_nonfatal, g_strdup ("After upgrading/installing, you may need to adjust the <prefix_dir> setting in your ~/.lives file"));
}

