// mainwindow.h
// LiVES (lives-exe)
// (c) G. Finch <salsaman@xs4all.nl> 2003
// Released under the GPL 2.0
// see file ../COPYING for licensing details


#ifndef HAS_MAINWINDOW_H
#define HAS_MAINWINDOW_H

// hardware related prefs

// key rpt (millisecs) for playback plugin, needs to match backend (and should be << FPS_MAX)
// TODO - make pref
#define KEY_RPT 40

// parameters for resizing the image frames, and for capture
// TODO - make prefs
#define V_RESIZE_ADJUST 36
#define H_RESIZE_ADJUST 22

#define FN_KEYS 12

// defaults for insert_silence
# define DEFAULT_AUDIO_RATE 44100
# define DEFAULT_AUDIO_CHANS 2
# define DEFAULT_AUDIO_SAMPS 16




// prefs window
typedef struct __prefsw {
  gulong encoder_ofmt_fn;
  gulong encoder_name_fn;
  GtkWidget *prefs_dialog;
  GtkWidget *prefs_notebook;
  GtkWidget *bgcolour_entry;
  GtkWidget *stop_screensaver_check;
  GtkWidget *open_maximised_check;
  GtkWidget *show_tool;
  GtkWidget *fs_max_check;
  GtkWidget *recent_check;
  GtkWidget *encoder_command_entry;
  GtkWidget *effects_command_entry;
  GtkWidget *video_command_entry;
  GtkWidget *audio_command_entry;
  GtkWidget *vid_load_dir_entry;
  GtkWidget *vid_save_dir_entry;
  GtkWidget *audio_dir_entry;
  GtkWidget *image_dir_entry;
  GtkWidget *proj_dir_entry;
  GtkWidget *tmpdir_entry;
  GtkWidget *cdplay_entry;
  GtkWidget *spinbutton_def_fps;
  GtkWidget *ofmt_combo;
  GtkWidget *vidp_combo;
  GtkWidget *audp_combo;
  GtkWidget *encoder_combo;
  GtkWidget *debug_encoders;
  GtkWidget *checkbutton_warn_fps;
  GtkWidget *checkbutton_warn_set;
  GtkWidget *checkbutton_warn_save_quality;
  GtkWidget *checkbutton_show_stats;
  GtkWidget *checkbutton_no_key_rpt;
  GtkWidget *check_xmms_pause;
  GtkWidget *checkbutton_warn_fsize;
  GtkWidget *spinbutton_warn_fsize;
  GtkWidget *spinbutton_bwidth;
  GtkWidget *theme_combo;
  GtkWidget *check_midi;
  GtkWidget *rb_aq_high;
  GtkWidget *rb_aq_low;
  GtkWidget *ins_speed;

} _prefsw;



typedef struct __palette {
  // set in set_palette_colours()
  gint style;
#define STYLE_PLAIN 0 // no theme (theme 'none')
#define STYLE_1 1<<0 // turn on theming if set
#define STYLE_2 1<<1 // colour the spinbuttons on the front page if set
#define STYLE_3 1<<2 // style is lightish - allow themeing of widgets with dark text, otherwise use menu bg

  GdkColor white;
  GdkColor black;
  GdkColor light_blue;
  GdkColor light_yellow;
  GdkColor pink;
  GdkColor light_red;
  GdkColor dark_orange;
  GdkColor fade_colour;
  GdkColor normal_back;
  GdkColor normal_fore;

  GdkColor menu_and_bars;
  GdkColor banner_fade_text;
  GdkColor info_text;
  GdkColor info_base;

  GdkColor bm_opaque;
  GdkColor bm_trans;

} _palette;




// mainw->
typedef struct _mw {
  gchar msg[512];

  // files
  gint current_file;
  gint first_free_file;
  file *files[MAX_FILES+1]; // +1 for the clipboard
  gchar vid_load_dir[256];
  gchar vid_save_dir[256];
  gchar audio_dir[256];
  gchar image_dir[256];
  gchar proj_dir[256];
  gchar xmms_dir[256];
  gchar def_ext[8];
  gint untitled_number;
  gint cap_number;
  gint clips_available;

  // hash table of clips in menu order
  GList *cliplist;

  // sets
  gchar set_name[256];

  // playback
  gboolean faded;
  gboolean double_size;
  gboolean sep_win;
  gboolean fs;
  gboolean loop;
  gboolean loop_cont;
  gboolean ping_pong;
  gboolean mute;
  gboolean must_resize;
  gint audio_start;
  gint audio_end;

  gboolean ext_playback;
  guint ext_palette;
#define EXTP_UNDEFINED 0
#define EXTP_RGB24LE 1
#define EXTP_UYVY 2
  gint ptr_x;
  gint ptr_y;

  gdouble fps_measure;


  // flags
  gboolean save_with_sound;
  gboolean ccpd_with_sound;
  gboolean selwidth_locked;
  gboolean is_ready;
  gboolean opening_loc;  // opening location (streaming)
  gboolean dvgrab_preview;
  gboolean switch_during_pb;

  gboolean in_fs_preview;
  gint cancelled;
  gboolean error;

  gshort endian;

  gint pwidth;
  gint pheight;

  gshort whentostop;
  // which stream end should cause playback to finish ?
#define NEVER_STOP 0
#define STOP_ON_VID_END 1
#define STOP_ON_AUD_END 2

  gint play_start;
  gint play_end;
  gboolean playing_sel;
  gboolean preview;

  gboolean is_processing;

  gboolean foreign;
  gboolean record_foreign;
  gboolean t_hidden;

  // some VJ effects
  gboolean nervous;

  guint rte;
#define EFFECT_NONE 0
#define EFFECT_ALPHA 1<<0
#define EFFECT_NOISE 1<<1
#define EFFECT_NEGATE 1<<2
#define EFFECT_REMOVE_BG 1<<3
#define EFFECT_POSTER 1<<4
#define EFFECT_MIRRORX 1<<5
#define EFFECT_MIRRORY 1<<6
#define EFFECT_EDGE 1<<7
#define EFFECT_BLEND 1<<8
  guint last_grabable_effect;
  guint rte_keys;
  
  // TODO - make part of rte struct, so one per rte
  gshort rte_mode;
  gshort rte_mode_max;

  gint blend_file;
  guchar blend_factor;
  guchar *old_pixel_data;
  // actually avg luma data
  guint *av_luma_data;
  uyvy_frame *uyvy_frame;
  gint av_count;
  gint opd_hsize;
  gint opd_vsize;

  // which number file we are playing
  gint playing_file;
  
  gint scr_width;
  gint scr_height;
  gint toy_type;
#define TOY_NONE 0
#define TOY_RANDOM_FRAMES 1
#define TOY_TV 1
  gboolean toy_go_wild;

  // copy/paste
  gboolean insert_after;
  gboolean with_sound;

  // selection
  gint sel_start;
  gshort sel_move;

  // which bar should we move ?
#define SEL_MOVE_START 1
#define SEL_MOVE_END 2
#define SEL_MOVE_AUTO 3
#define SEL_MOVE_SINGLE 4

  // prefs (Save on exit)
  gboolean prefs_changed;

  // this is toggled when the user clicks 'do not show'
  gboolean warn_mask;

  // default sizes for when no file is loaded
  gint def_width;
  gint def_height;

  // for the framedraw preview
  gint framedraw_frame;
  guint framedraw_type;
#define FD_NONE 0
#define FD_TRIM_FRAMES 1<<0


  /////////////////////////////////////////////////

  // end of static-ish info
  gboolean save_all;
  gchar first_info_file[256];
  gboolean leave_files;
  gboolean was_set;

  // extra parameters for opening special files
  gchar *file_open_params;

  gint last_dprint_file;

  // actual frame being displayed
  gint actual_frame;

  // and the audio 'frame' for when we are looping
  gdouble aframeno;

  // for the internal player
  gdouble period;
  long double startticks;
  long double startsecs;
  long double currticks;
  long deltaticks;
  long double firstticks;
  long double origticks;
  gboolean size_warn;

  // TODO - remove, use cfile->pb_fps instead
  gdouble pb_fps;

  // internal fx
  gboolean internal_messaging;
  gint (*progress_fn) (gboolean reset);


  // fx controls
  gdouble fx1_val;
  gdouble fx2_val;
  gdouble fx3_val;
  gdouble fx4_val;
  gdouble fx5_val;
  gdouble fx6_val;

  gint fx1_start;
  gint fx2_start;
  gint fx3_start;
  gint fx4_start;

  gint fx1_step;
  gint fx2_step;
  gint fx3_step;
  gint fx4_step;

  gint fx1_end;
  gint fx2_end;
  gint fx3_end;
  gint fx4_end;

  gboolean fx1_bool;
  gboolean fx2_bool;
  gboolean fx3_bool;
  gboolean fx4_bool;
  gboolean fx5_bool;
  gboolean fx6_bool;

  gboolean effects_paused;

  //function pointers
  gulong config_func;
  gulong pb_fps_func;
  gulong spin_start_func;
  gulong spin_end_func;
  gulong record_perf_func;
  gulong vidbar_func;
  gulong laudbar_func;
  gulong raudbar_func;
  gulong hrule_func;
  gulong toy_func_none;
  gulong toy_func_random_frames;
  gulong hnd_id;

  // selection functions
  gulong mouse_fn1;

  // stored clips
  gint clipstore[FN_KEYS-1];

  // GdkGC (graphics context)
  GdkGC *gc;

  // Widgets
  GdkNativeWindow xwin;
  GtkTooltips *tooltips;
  GtkWidget *frame1;
  GtkWidget *frame2;
  GtkWidget *playframe;
  GdkPixbuf *imframe;
  GtkWidget *LiVES;
  GtkWidget *save;
  GtkWidget *open;
  GtkWidget *open_sel;
  GtkWidget *open_vcd_menu;
  GtkWidget *open_vcd_submenu;
  GtkWidget *open_vcd;
  GtkWidget *open_dvd;
  GtkWidget *open_loc;
  GtkWidget *open_device_menu;
  GtkWidget *open_device_submenu;
  GtkWidget *open_firewire;
  GtkWidget *recent_menu;
  GtkWidget *recent_submenu;
  GtkWidget *recent1;
  GtkWidget *recent2;
  GtkWidget *recent3;
  GtkWidget *recent4;
  GtkWidget *save_as;
  GtkWidget *backup;
  GtkWidget *restore;
  GtkWidget *save_selection;
  GtkWidget *close;
  GtkWidget *sw_sound;
  GtkWidget *ccpd_sound;
  GtkWidget *quit;
  GtkWidget *undo;
  GtkWidget *redo;
  GtkWidget *copy;
  GtkWidget *cut;
  GtkWidget *insert;
  GtkWidget *paste_as_new;
  GtkWidget *merge;
  GtkWidget *delete;
  GtkWidget *select_submenu;
  GtkWidget *select_all;
  GtkWidget *select_new;
  GtkWidget *select_to_end;
  GtkWidget *select_from_start;
  GtkWidget *select_start_only;
  GtkWidget *select_end_only;
  GtkWidget *select_last;
  GtkWidget *select_invert;
  GtkWidget *lock_selwidth;
  GtkWidget *record;
  GtkWidget *record_perf;
  GtkWidget *playall;
  GtkWidget *playsel;
  GtkWidget *playclip;
  GtkWidget *rev_clipboard;
  GtkWidget *stop;
  GtkWidget *rewind;
  GtkWidget *full_screen;
  GtkWidget *loop_video;
  GtkWidget *loop_continue;
  GtkWidget *loop_ping_pong;
  GtkWidget *sepwin;
  GtkWidget *mute_audio;
  GtkWidget *sticky;
  GtkWidget *showfct;
  GtkWidget *fade;
  GtkWidget *dsize;
  GtkWidget *blank_frames;
  GtkWidget *colour_cycle;
  GtkWidget *colour_filter;
  GtkWidget *colorize;
  GtkWidget *edge_detect;
  GtkWidget *enhance;
  GtkWidget *flip;
  GtkWidget *flop;
  GtkWidget *dream;
  GtkWidget *jumble;
  GtkWidget *normalize;
  GtkWidget *negate;
  GtkWidget *monochrome;
  GtkWidget *charcoal;
  GtkWidget *randomzoom;
  GtkWidget *rotate;
  GtkWidget *shrink;
  GtkWidget *solarize;
  GtkWidget *swirl;
  GtkWidget *transx;
  GtkWidget *transy;
  GtkWidget *tunnel;
  GtkWidget *wave;
  GtkWidget *realtime_fx;
  GtkWidget *change_speed;
  GtkWidget *trim_frames;
  GtkWidget *capture;
  GtkWidget *reorder;
  GtkWidget *load_audio;
  GtkWidget *load_cdtrack;
  GtkWidget *eject_cd;
  GtkWidget *export_submenu;
  GtkWidget *export_allaudio;
  GtkWidget *export_selaudio;
  GtkWidget *append_audio;
  GtkWidget *trim_submenu;
  GtkWidget *trim_audio;
  GtkWidget *trim_to_pstart;
  GtkWidget *delaudio_submenu;
  GtkWidget *delsel_audio;
  GtkWidget *delall_audio;
  GtkWidget *ins_silence;
  GtkWidget *resample_audio;
  GtkWidget *resample_video;
  GtkWidget *preferences;
  GtkWidget *xmms_play_audio;
  GtkWidget *xmms_random_audio;
  GtkWidget *xmms_stop_audio;
  GtkWidget *resize;
  GtkWidget *rotate_all;
  GtkWidget *rename;
  GtkWidget *toys;
  GtkWidget *toy_none;
  GtkWidget *toy_random_frames;
  GtkWidget *toy_tv;
  GtkWidget *show_file_info;
  GtkWidget *show_file_comments;
  GtkWidget *show_clipboard_info;
  GtkWidget *sel_label;
  GtkAccelGroup *accel_group;
  GtkWidget *sep_image;
  GtkWidget *hruler;
  GtkWidget *vj_menu;
  GtkWidget *vj_save_set;
  GtkWidget *vj_load_set;
  GtkWidget *vj_show_keys;

  // for the fileselection preview
  GtkWidget *fs_playarea;
  GtkWidget *fs_playframe;

  // for the framedraw widget (e.g. in Trim Frames)
  GtkWidget *framedraw; // the eventbox
  GtkWidget *framedraw_redraw; // the 'redraw' button
  GtkWidget *framedraw_image; // the image

  GdkPixmap *framedraw_orig_pixmap; // the original frame pixmap
  GdkPixmap *framedraw_copy_pixmap; // the altered frame pixmap
  GdkBitmap *framedraw_bitmap; // and its mask
  GdkGC *framedraw_bitmapgc;  // the GC for the bitmap
  GdkGC *framedraw_colourgc;  // the GC for the pixmap, we can draw in colours !

  // bars here -> actually text above bars
  GtkWidget *vidbar;
  GtkWidget *laudbar;
  GtkWidget *raudbar;

  GtkWidget *spinbutton_end;
  GtkWidget *spinbutton_start;

  GtkWidget *arrow1;
  GtkWidget *arrow2;

  // for the internal player
  gint fixed_height;
  GtkWidget *image274;
  GtkWidget *play_window;
  GdkPixbuf *frame_image;
  GdkPixbuf *dbl_size;
  GtkWidget *plug1;
  gulong pw_exp_func;

  // frame preview in the separate window
  GtkWidget *preview_box;
  GtkWidget *preview_image;
  GtkWidget *preview_spinbutton;
  gint preview_frame;
  gulong preview_spin_func;
  gint prv_link;
#define PRV_FREE 0
#define PRV_START 1
#define PRV_END 2
#define PRV_PTR 3

  GtkWidget *image272;
  GtkWidget *image273;
  GtkWidget *playarea;
  GtkWidget *hseparator;
  GtkWidget *scrolledwindow;
  GtkWidget *warning_label;

  GtkTextMark *mark;
  GtkWidget *textview1;
  GtkWidget *winmenu;
  GtkWidget *eventbox;
  GtkWidget *eventbox2;
  GtkWidget *eventbox3;
  GtkWidget *eventbox4;
  GtkWidget *eventbox5;

  // toolbar buttons
  GtkWidget *t_stopbutton;
  GtkWidget *t_bckground;
  GtkWidget *t_fullscreen;
  GtkWidget *t_sepwin;
  GtkWidget *t_double;
  GtkWidget *t_single;
  GtkWidget *t_infobutton;
  GtkWidget *t_slower;
  GtkWidget *t_faster;
  GtkWidget *t_forward;
  GtkWidget *t_back;
  GtkWidget *t_dirchange;
  GtkWidget *t_freeze;
  GtkWidget *t_grabkeys;
  GtkWidget *t_rtemode;
  GtkWidget *t_f1;
  GtkWidget *t_f2;
  GtkWidget *t_f3;
  GtkWidget *t_f4;
  GtkWidget *t_f5;
  GtkWidget *t_f6;
  GtkWidget *t_f7;
  GtkWidget *t_f8;
  GtkWidget *t_f9;
  GtkWidget *t_f10;
  GtkWidget *t_f11;
  GtkWidget *t_f12;
  GtkWidget *t_nextclip;
  GtkWidget *t_prevclip;
  GtkWidget *t_hide;
  GtkWidget *t_nervous;
  GtkWidget *toolbar;
  GtkWidget *fs1;

  // real time effects
  GtkWidget *t_rte0;
  GtkWidget *t_rte1;
  GtkWidget *t_rte2;
  GtkWidget *t_rte3;
  GtkWidget *t_rte4;
  GtkWidget *t_rte5;
  GtkWidget *t_rte6;
  GtkWidget *t_rte7;
  GtkWidget *t_rte8;
  GtkWidget *t_rte9;

  // menubar buttons
  GtkWidget *menubuttons;
  GtkWidget *m_playbutton;
  GtkWidget *m_stopbutton;
  GtkWidget *m_playselbutton;
  GtkWidget *m_rewindbutton;
  GtkWidget *m_loopbutton;
  GtkWidget *m_mutebutton;
  GtkWidget *mute_img;
  GtkWidget *menubar;

  // separate window
  gint opwx;
  gint opwy;

  // sepwin buttons
  GtkWidget *p_playbutton;
  GtkWidget *p_playselbutton;
  GtkWidget *p_rewindbutton;
  GtkWidget *p_loopbutton;
  GtkWidget *p_mutebutton;
  GtkWidget *p_mute_img;

  // timer bars
  GtkWidget *video_draw;
  GdkPixmap *video_drawable;
  GtkWidget *laudio_draw;
  GdkPixmap *laudio_drawable;
  GtkWidget *raudio_draw;
  GdkPixmap *raudio_drawable;

  // framecounter
  GtkWidget *framebar;
  GtkWidget *framecounter;
  GtkWidget *spinbutton_pb_fps;
  GtkWidget *vps_label;
  GtkWidget *curf_label;
  GtkWidget *banner;

  // recording from an external window
  guint foreign_key;
  unsigned int foreign_id;
  GdkColormap *foreign_cmap;
  GdkPixmap *foreign_map;
  gint foreign_width;
  gint foreign_height;
  gint foreign_bpp;

} mainwindow;



_palette *palette;



#endif // HAS_MAINWINDOW_H
