// plugins.c
// LiVES
// (c) G. Finch 2003 <salsaman@xs4all.nl>
// see file ../COPYING for licensing details

#include <gdk/gdkkeysyms.h>
#include <dlfcn.h>

#include "plugins.h"

static gboolean list_plugins;


GList *
get_plugin_list (gchar *plugin_type) {
  // returns a GList * of plugins of type plugin_type
  // returns empty list if there are no plugins of that type
  gchar *com;
  if (plugin_type==PLUGIN_THEMES) {
    com=g_strdup_printf ("smogrify list_plugins 1 %s",g_filename_from_utf8 (THEME_DIR,-1,NULL,NULL,NULL));
  }
  else {
    // TODO - use filename from utf8
    com=g_strdup_printf ("smogrify list_plugins 0 %s%s",PLUGIN_DIR,plugin_type);
  }
  list_plugins=TRUE;
  return get_plugin_result (com);
}


// this is an internal function
GList *
get_plugin_result (gchar *command) {
  gchar **array;
  gint pieces;
  int outfile_fd,i;
  gchar *msg,*buf;
  gint error;
  GList *list=NULL;
  gint count=30000000/prefs->sleep_time;  // timeout of 30 seconds
  gchar *outfile=g_strdup_printf ("%s/.smogplugin.%d",prefs->tmpdir,getpid());
  gchar *com=g_strconcat (command,">",outfile,NULL);

  if ((error=system (com))!=0&&error!=126*256&&error!=256) {
    if (list_plugins==FALSE) {
      g_free (com);
      msg=g_strdup_printf ("Plugin error: %s failed with code %d\n",command,error/256);
      d_print (msg);
      g_free (msg);
    }
    g_free (outfile);
    g_free (command);
    return list;
  }
  g_free (com);
  clear_mainw_msg();

  while (!(outfile_fd=open(outfile,O_RDONLY))&&(count-->0||list_plugins==TRUE)) {
    g_usleep (prefs->sleep_time);
  }

  if (!count) {
    g_print ("Plugin timed out on message %s\n",command);
    g_free (command);
    g_free (outfile);
    return list;
  }
  g_free (command);

  read (outfile_fd,mainw->msg,512);
  close (outfile_fd);
  unlink (outfile);
  g_free (outfile);
  if (error==256) return list;

#if defined DEBUG_PLUGINS
  g_print("plugin msg: %s\n",mainw->msg);
#endif

  pieces=get_token_count (mainw->msg,'|');
  array=g_strsplit(mainw->msg,"|",pieces);
  for (i=0;i<pieces;i++) {
    if ((buf=g_strdup(array[i]))!=NULL) {
      buf=(g_strchomp (g_strchug(buf)));
      if (strlen (buf)) {
	list=g_list_append (list, buf);
      }
    }
  }
  g_strfreev (array);
  return list;
}




GList *
plugin_request (gchar *plugin_type, gchar *plugin_name, gchar *request) {
  // returns a GList of responses to -request, or NULL on error
  GList *empty_list=NULL;
  gchar *com=g_strdup_printf ("%s%s%s/%s %s",prefs->prefix_dir,PLUGIN_DIR,plugin_type,plugin_name,request);

  if (plugin_name==NULL||!strlen(plugin_name)) {
    return empty_list;
  }

  list_plugins=FALSE;
  return get_plugin_result (com);
}




void 
do_plugin_error(const gchar *plugin_type, const gchar *plugin_name) {
  gchar *msg;

  if (plugin_name==NULL) {
    msg=g_strdup_printf("LiVES was unable to find its %s plugins. Please make sure you have the plugins installed in\n%s%s%s",plugin_type,prefs->prefix_dir,PLUGIN_DIR,plugin_type);
    do_error_dialog(msg);
    g_free(msg);
    return;
  }
  msg=g_strdup_printf("LiVES did not receive a response from the %s plugins.\nPlease make sure you have the plugin installed correctly in\n%s%s%s",plugin_type,prefs->prefix_dir,PLUGIN_DIR,plugin_type);
  do_error_dialog(msg);
  g_free(msg);
}






gboolean 
open_vid_playback_plugin (const gchar *name) {
  // this is called on startup or when the user selects a new playback plugin

  gchar *plugname=g_strdup_printf ("%s%s%s/%s",prefs->prefix_dir,PLUGIN_DIR,PLUGIN_VID_PLAYBACK,name);
  guint cap;
  GModule *GM=g_module_open (plugname,0);
  gboolean OK=TRUE;
  gchar *msg;

  if (GM==NULL) {
    gchar *msg=g_strdup_printf ("\n\nFailed to open playback module %s\nError was %s\n",plugname,g_module_error());
    do_error_dialog (msg);
    g_free (msg);
    return FALSE;
  }

  if (vid_playback_plugin!=NULL) {
    g_module_close (vid_playback_plugin->gmodule);
    g_free (vid_playback_plugin);
  }

  vid_playback_plugin=(_vid_playback_plugin *) g_malloc (sizeof(_vid_playback_plugin));

  if (g_module_symbol (GM,"version",(gpointer)&vid_playback_plugin->version)==FALSE) {
    OK=FALSE;
  }
  if (g_module_symbol (GM,"set_palette",(gpointer)&vid_playback_plugin->set_palette)==FALSE) {
    OK=FALSE;
  }
  if (g_module_symbol (GM,"set_keyfun",(gpointer)&vid_playback_plugin->set_keyfun)==FALSE) {
    OK=FALSE;
  }
  if (g_module_symbol (GM,"get_capabilities",(gpointer)&vid_playback_plugin->get_capabilities)==FALSE) {
    OK=FALSE;
  }
  if (g_module_symbol (GM,"init_screen",(gpointer)&vid_playback_plugin->init_screen)==FALSE) {
    OK=FALSE;
  }
  if (g_module_symbol (GM,"render_frame",(gpointer)&vid_playback_plugin->render_frame)==FALSE) {
    OK=FALSE;
  }
  if (g_module_symbol (GM,"free_frame",(gpointer)&vid_playback_plugin->free_frame)==FALSE) {
    OK=FALSE;
  }
  if (g_module_symbol (GM,"send_keycodes",(gpointer)&vid_playback_plugin->send_keycodes)==FALSE) {
    OK=FALSE;
  }
  if (g_module_symbol (GM,"exit_screen",(gpointer)&vid_playback_plugin->exit_screen)==FALSE) {
    OK=FALSE;
  }

  if (OK==FALSE) {
    g_free (vid_playback_plugin);
    vid_playback_plugin=NULL;
    g_module_close (GM);
    return FALSE;
  }

  // try different palettes until we get TRUE
  if ((*vid_playback_plugin->set_palette)("UYVY")==FALSE) {
    g_print ("failed to set uyvy\n");
    if ((*vid_playback_plugin->set_palette)("RGB24LE")==FALSE) {
      g_free (vid_playback_plugin);
      vid_playback_plugin=NULL;
      g_module_close (GM);
      return FALSE;
    }
    mainw->ext_palette=EXTP_RGB24LE;
  }
  else {
    mainw->ext_palette=EXTP_UYVY;
  }
  cap=(*vid_playback_plugin->get_capabilities)();
  vid_playback_plugin->can_resize=cap&1;
  if (!(vid_playback_plugin->can_resize&&mainw->ext_palette==EXTP_UYVY)) {
    // UYVY must resize
    if ((*vid_playback_plugin->set_palette)("RGB24LE")==FALSE) {
      g_free (vid_playback_plugin);
      vid_playback_plugin=NULL;
      g_module_close (GM);
      return FALSE;
    }
    mainw->ext_palette=EXTP_RGB24LE;
    cap=(*vid_playback_plugin->get_capabilities)();
    vid_playback_plugin->can_resize=cap&1;
  }
  vid_playback_plugin->can_return=cap&2;

  (*vid_playback_plugin->set_keyfun)(&pl_key_function);

  g_snprintf (vid_playback_plugin->name,256,"%s",name);
  vid_playback_plugin->gmodule=GM;
  cached_key=cached_mod=0;
  msg=g_strdup_printf("*** Using %s plugin for fs playback, agreed to use palette type %d. ***\n",name,mainw->ext_palette);
  d_print (msg);
  g_free (msg);
  return TRUE;
}


gboolean 
plugin_poll_keyboard (gpointer data) {
  // this is a function which should be called periodically when a plugin has
  // control of the keyboard

  // call once to send cached keys
  pl_key_function (TRUE,0,0);

  // let the plugin call pl_key_function itself, with any keycodes it has received
  (*vid_playback_plugin->send_keycodes)();
  return TRUE;
}



void 
pl_key_function (gboolean down, guint16 unicode, guint16 keymod) {
  // create a gdk_key event
  // plugins can call this to pass key events to LiVES

#define NEEDS_TRANSLATION 1<<15
  GdkModifierType state=(keymod&(GDK_CONTROL_MASK|GDK_MOD1_MASK));
  gboolean sent_cached_key=FALSE;

  if (down==FALSE) {
    // up...
    if (keymod&NEEDS_TRANSLATION) {
      switch (unicode) {
	// some keys need translating when a modifier is held down
      case(100) :if (cached_key==GDK_Left) cached_key=0;return;
      case(102) :if (cached_key==GDK_Right) cached_key=0;return;
      case(98)  :if (cached_key==GDK_Up) cached_key=0;return;
      case(104) :if (cached_key==GDK_Down) cached_key=0;return;
      }
    }
    return;
  }

  // translate hardware code into gdk keyval, and call any accelerators
  if (keymod&NEEDS_TRANSLATION) {
    switch (unicode) {
      // some keys need translating when a modifier is held down
    case (65) :unicode=GDK_space;break;
    case (22) :unicode=GDK_BackSpace;break;
    case (24) :unicode=GDK_q;break;
    case (10) :unicode=GDK_1;break;
    case (11) :unicode=GDK_2;break;
    case (12) :unicode=GDK_3;break;
    case (13) :unicode=GDK_4;break;
    case (14) :unicode=GDK_5;break;
    case (15) :unicode=GDK_6;break;
    case (16) :unicode=GDK_7;break;
    case (17) :unicode=GDK_8;break;
    case (18) :unicode=GDK_9;break;
    case (19) :unicode=GDK_0;break;
    case (67) :unicode=GDK_F1;break;
    case (68) :unicode=GDK_F2;break;
    case (69) :unicode=GDK_F3;break;
    case (70) :unicode=GDK_F4;break;
    case (71) :unicode=GDK_F5;break;
    case (72) :unicode=GDK_F6;break;
    case (73) :unicode=GDK_F7;break;
    case (74) :unicode=GDK_F8;break;
    case (75) :unicode=GDK_F9;break;
    case (76) :unicode=GDK_F10;break;
    case (95) :unicode=GDK_F11;break;
    case (96) :unicode=GDK_F12;break;
    case (99) :unicode=GDK_Page_Up;break;
    case(105) :unicode=GDK_Page_Down;break;

      // auto repeat keys
    case(100) :unicode=GDK_Left;cached_key=unicode;cached_mod=keymod^NEEDS_TRANSLATION;sent_cached_key=TRUE;break;
    case(102) :unicode=GDK_Right;cached_key=unicode;cached_mod=keymod^NEEDS_TRANSLATION;sent_cached_key=TRUE;break;
    case(98)  :unicode=GDK_Up;cached_key=unicode;cached_mod=keymod^NEEDS_TRANSLATION;sent_cached_key=TRUE;break;
    case(104) :unicode=GDK_Down;cached_key=unicode;cached_mod=keymod^NEEDS_TRANSLATION;sent_cached_key=TRUE;break;

    default:return;
    }
  }
  if (unicode) {
    gtk_accel_groups_activate (G_OBJECT (mainw->LiVES),(guint)unicode,state);
  }
  if (cached_key&&sent_cached_key==FALSE) {
    gtk_accel_groups_activate (G_OBJECT (mainw->LiVES),(guint)cached_key,cached_mod);
  }
}
