// plugins.h
// LiVES
// (c) G. Finch 2003 <salsaman@xs4all.nl>
// see file ../COPYING for licensing details
#include <gtk/gtk.h>
#include <gmodule.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include "main.h"





// use enum
GList *
get_plugin_list (gchar *plugin_type);
#define PLUGIN_ENCODERS "encoders"
#define PLUGIN_VID_PLAYBACK "playback/video"


// smogrify handles the directory differently for themes
#define PLUGIN_THEMES "themes"



GList *
plugin_request (gchar *plugin_type, gchar *plugin_name, gchar *request);


GList *
get_plugin_result (gchar *command); /* private function */


void 
do_plugin_error(const gchar *plugin_type, const gchar *plugin_name_or_null);


// specific types of plugins
typedef struct {
  // playback
  gchar name[64];
  GModule *gmodule;
  guint kb_timer;

  const gchar * (*version) (void);
  void (*set_keyfun)(void (*fn)(gboolean, guint16, guint16));
  guint (*set_palette)(const gchar *p_name);
  guint (*get_capabilities)(void);
  gboolean (*init_screen)(int width, int height, gboolean fullscreen);
  gboolean (*render_frame)(int hsize, int vsize, void *pixel_data, void *return_data);
  void (*free_frame)(void);
  gboolean (*send_keycodes)(void);
  void (*exit_screen)(gint16 mouse_x, gint16 mouse_y);

  //capabilities
  gboolean can_resize;
  gboolean can_return;
} _vid_playback_plugin;

_vid_playback_plugin *vid_playback_plugin;

gboolean 
open_vid_playback_plugin (const gchar *name);




// keyboard stuff for modules that pass keycodes
#define KEY_RPT_INTERVAL 40


gboolean 
plugin_poll_keyboard (gpointer data);

void 
pl_key_function (gboolean down, guint16 unicode, guint16 keymod);

// repeating keys
guint16 cached_key;
guint16 cached_mod;

// these keys should be cached on a key down and sent every time until a key up
#define key_left 100
#define key_right 102
#define key_up 98
#define key_down 104




