// interface.c
// LiVES
// (c) G. Finch 2003 <salsaman@xs4all.nl>
// Released under the GPL 2.0
// see file ../COPYING for licensing details

// TODO - use gtk_widget_showall where poss.
// and don't forget gtk_box_pack_end (doh)

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "merge.h"
#include "support.h"
#include "framedraw.h"




GtkWidget*
create_fileselection1 (const gchar *title, gint preview_type)
{
  GtkWidget *fileselection1;
  GtkWidget *ok_button1;
  GtkWidget *cancel_button1;

  gchar title1[256];
  g_snprintf(title1,256,"LiVES: - %s",title);

  fileselection1 = gtk_file_selection_new (title1);
  gtk_widget_modify_bg(fileselection1, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_container_set_border_width (GTK_CONTAINER (fileselection1), 10);
  gtk_window_set_position (GTK_WINDOW (fileselection1), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (fileselection1), TRUE);

  gtk_window_set_transient_for(GTK_WINDOW(fileselection1),GTK_WINDOW(mainw->LiVES));

  if (prefs->fileselmax==TRUE) {
    gtk_window_maximize (GTK_WINDOW(fileselection1));
  }

  ok_button1 = GTK_FILE_SELECTION (fileselection1)->ok_button;
  gtk_widget_show (ok_button1);
  GTK_WIDGET_SET_FLAGS (ok_button1, GTK_CAN_DEFAULT);

  cancel_button1 = GTK_FILE_SELECTION (fileselection1)->cancel_button;
  gtk_widget_show (cancel_button1);
  GTK_WIDGET_SET_FLAGS (cancel_button1, GTK_CAN_DEFAULT);

  gtk_widget_grab_focus (fileselection1);
  
  if (preview_type>0) widget_add_preview(GTK_BOX (GTK_FILE_SELECTION(fileselection1)->main_vbox),GTK_BOX (GTK_FILE_SELECTION (fileselection1)->action_area),preview_type);

  g_signal_connect (GTK_FILE_SELECTION(fileselection1)->cancel_button, "clicked",G_CALLBACK (on_cancel_button1_clicked),NULL);
  return fileselection1;
}


void
widget_add_preview(GtkBox *for_preview, GtkBox *for_button, gint preview_type) {
  // preview type 1 - video and audio, fileselector
  // preview type 2 - audio only, fileselector
  // preview type 3 - a/v, open selection

  GtkWidget *preview_button;
  GtkWidget *fs_label;

  mainw->fs_playframe = gtk_frame_new (NULL);
  mainw->fs_playarea = gtk_socket_new ();

  if (preview_type==1||preview_type==3) {

    gtk_widget_show (mainw->fs_playframe);

    gtk_container_set_border_width (GTK_CONTAINER(mainw->fs_playframe), 10);
    gtk_frame_set_shadow_type (GTK_FRAME(mainw->fs_playframe), GTK_SHADOW_IN);
    gtk_widget_modify_bg (mainw->fs_playframe, GTK_STATE_NORMAL, &palette->normal_back);
    
    fs_label = gtk_label_new ("Preview");
    gtk_widget_show (fs_label);
    gtk_frame_set_label_widget (GTK_FRAME (mainw->fs_playframe), fs_label);
    gtk_label_set_justify (GTK_LABEL (fs_label), GTK_JUSTIFY_RIGHT);

    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(fs_label, GTK_STATE_NORMAL, &palette->normal_fore);
    }
    
    gtk_box_pack_start (for_preview, mainw->fs_playframe, FALSE, FALSE, TRUE);
    gtk_widget_set_size_request (mainw->fs_playarea, DEFAULT_FRAME_HSIZE, DEFAULT_FRAME_VSIZE);

    gtk_container_add (GTK_CONTAINER (mainw->fs_playframe), mainw->fs_playarea);

    gtk_widget_modify_bg (mainw->fs_playarea, GTK_STATE_NORMAL, &palette->normal_back);
    gtk_widget_show(mainw->fs_playarea);
   }

  preview_button = gtk_button_new_with_mnemonic ("_Preview");
  gtk_widget_show (preview_button);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(preview_button, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  gtk_box_pack_start (for_button, preview_button, TRUE, TRUE, 0);

  g_signal_connect (GTK_OBJECT (preview_button), "clicked",
		    G_CALLBACK (on_fs_preview_clicked),
		    GINT_TO_POINTER (preview_type));

}


GtkWidget*
create_rotea (void)
{
  GtkWidget *rotea;
  GtkWidget *dialog_vbox16;
  GtkWidget *vbox26;
  GtkWidget *label101;
  GtkWidget *radiobutton34;
  GSList *radiobutton34_group = NULL;
  GtkWidget *radiobutton35;
  GtkWidget *radiobutton36;
  GtkWidget *radiobutton37;
  GtkWidget *hbox15;
  GtkWidget *spinbutton_other;
  GtkObject *spinbutton_other_adj;
  GtkWidget *dialog_action_area16;
  GtkWidget *cancelbutton14;
  GtkWidget *okbutton13;
  GtkWidget *label45;
  GtkWidget *hseparator1;

  rotea = gtk_dialog_new();
  gtk_container_set_border_width (GTK_CONTAINER (rotea), 10);
  gtk_window_set_position (GTK_WINDOW (rotea), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (rotea), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(rotea),GTK_WINDOW(mainw->LiVES));

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(rotea),FALSE);
  }
  gtk_widget_modify_bg (rotea, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_default_size (GTK_WINDOW (rotea), 300, 200);
  gtk_window_set_title (GTK_WINDOW (rotea), _("LiVES: - Rotate Clip"));

  dialog_vbox16 = GTK_DIALOG (rotea)->vbox;
  gtk_widget_show (dialog_vbox16);

  vbox26 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox26);
  gtk_box_pack_start (GTK_BOX (dialog_vbox16), vbox26, TRUE, TRUE, 0);

  label101 = gtk_label_new (_ ("Rotate all frames by..."));
  gtk_widget_show (label101);
  gtk_box_pack_start (GTK_BOX (vbox26), label101, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label101), GTK_JUSTIFY_LEFT);
  
  radiobutton34 = gtk_radio_button_new_with_mnemonic (NULL, 
						      _ ("90 Degrees Clockwise"));
  gtk_widget_show (radiobutton34);
  gtk_box_pack_start (GTK_BOX (vbox26), radiobutton34, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton34), 
			      radiobutton34_group);
  radiobutton34_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON 
						    (radiobutton34));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton34), TRUE);

  radiobutton35 = gtk_radio_button_new_with_mnemonic (NULL, 
						      _ ("90 Degrees Anti-Clockwise"));
  gtk_widget_show (radiobutton35);
  gtk_box_pack_start (GTK_BOX (vbox26), radiobutton35, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton35), 
			      radiobutton34_group);
  radiobutton34_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON 
						    (radiobutton35));

  radiobutton36 = gtk_radio_button_new_with_mnemonic (NULL, _ ("180 Degrees"));
  gtk_widget_show (radiobutton36);
  gtk_box_pack_start (GTK_BOX (vbox26), radiobutton36, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton36), 
			      radiobutton34_group);

  radiobutton34_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON 
						    (radiobutton36));

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox26), hseparator1, TRUE, TRUE, 0);

  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox15);
  gtk_box_pack_start (GTK_BOX (vbox26), hbox15, TRUE, TRUE, 0);

  radiobutton37 = gtk_radio_button_new_with_mnemonic (NULL, _ ("Other"));
  gtk_widget_show (radiobutton37);
  gtk_box_pack_start (GTK_BOX (hbox15), radiobutton37, TRUE, TRUE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton37), 
			      radiobutton34_group);

  radiobutton34_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON 
						    (radiobutton37));

  spinbutton_other_adj = gtk_adjustment_new (0, -179, 179, 1, 10, 90);
  spinbutton_other = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_other_adj), 1, 0);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinbutton_other), TRUE);
  gtk_widget_show (spinbutton_other);
  gtk_box_pack_start (GTK_BOX (hbox15), spinbutton_other, TRUE, TRUE, 0);

  label45 = gtk_label_new (_("Degrees"));
  gtk_widget_show (label45);
  gtk_box_pack_start (GTK_BOX (hbox15), label45, TRUE, FALSE, 0);

  dialog_action_area16 = GTK_DIALOG (rotea)->action_area;
  gtk_widget_show (dialog_action_area16);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area16), 
			     GTK_BUTTONBOX_END);

  cancelbutton14 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton14);
  gtk_dialog_add_action_widget (GTK_DIALOG (rotea), cancelbutton14, 
				GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton14, GTK_CAN_DEFAULT);

  okbutton13 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton13);
  gtk_dialog_add_action_widget (GTK_DIALOG (rotea), okbutton13, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton13, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton13);

  g_signal_connect (GTK_OBJECT (cancelbutton14), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      NULL);

  g_signal_connect (GTK_OBJECT (okbutton13), "clicked",
                      G_CALLBACK (on_rotate_all_ok_clicked),
                      NULL);

  g_signal_connect (GTK_OBJECT (radiobutton34), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (radiobutton35), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (2));
  g_signal_connect (GTK_OBJECT (radiobutton36), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (3));
  g_signal_connect (GTK_OBJECT (radiobutton37), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (4));
  g_signal_connect_after (GTK_OBJECT (spinbutton_other), "value_changed",
                      G_CALLBACK (on_spin_value_changed),
                      GINT_TO_POINTER (1));
  return rotea;
}





process* 
create_processing (const gchar *text)
{

  GtkWidget *dialog_vbox1;
  GtkWidget *vbox2;
  GtkWidget *vbox3;
  GtkWidget *dialog_action_area1;
  process *procw=(process*)(g_malloc(sizeof(process)));
  gchar tmp_label[256];

  procw->processing = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (procw->processing), 10);
  gtk_window_add_accel_group (GTK_WINDOW (procw->processing), mainw->accel_group);

  gtk_widget_set_size_request (procw->processing, 320, 180);
  gtk_window_set_resizable (GTK_WINDOW (procw->processing), FALSE);
  
  gtk_widget_modify_bg(procw->processing, GTK_STATE_NORMAL, &palette->normal_back);
  gtk_window_set_title (GTK_WINDOW (procw->processing), _("LiVES: - Processing..."));
  gtk_window_set_transient_for(GTK_WINDOW(procw->processing),GTK_WINDOW(mainw->LiVES));
  gtk_window_set_position (GTK_WINDOW (procw->processing), GTK_WIN_POS_CENTER);

  dialog_vbox1 = GTK_DIALOG (procw->processing)->vbox;
  gtk_widget_show (dialog_vbox1);

  gtk_dialog_set_has_separator(GTK_DIALOG(procw->processing),FALSE);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox2, TRUE, TRUE, 0);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), vbox3, TRUE, TRUE, 0);

  g_snprintf(tmp_label,256,"%s...",text);
  procw->label = gtk_label_new (tmp_label);
  gtk_widget_show (procw->label);
  gtk_box_pack_start (GTK_BOX (vbox3), procw->label, FALSE, FALSE, 0);
  gtk_widget_modify_fg(procw->label, GTK_STATE_NORMAL, &palette->normal_fore);
  gtk_label_set_justify (GTK_LABEL (procw->label), GTK_JUSTIFY_LEFT);

  procw->progressbar = gtk_progress_bar_new ();
  gtk_widget_show (procw->progressbar);
  gtk_box_pack_start (GTK_BOX (vbox3), procw->progressbar, FALSE, FALSE, 0);
  gtk_widget_modify_fg(procw->progressbar, GTK_STATE_NORMAL, &palette->normal_fore);

  if (mainw->internal_messaging) procw->label2 = gtk_label_new (_("\n\nPlease Wait\n\nRemember to switch off effects (ctrl-0) afterwards !"));
  else procw->label2 = gtk_label_new (_("\nPlease Wait"));
  gtk_widget_show (procw->label2);
  gtk_box_pack_start (GTK_BOX (vbox3), procw->label2, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (procw->label2), GTK_JUSTIFY_CENTER);
  gtk_widget_modify_fg(procw->label2, GTK_STATE_NORMAL, &palette->normal_fore);


  dialog_action_area1 = GTK_DIALOG (procw->processing)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  procw->stop_button = gtk_button_new_with_mnemonic ("_Enough");
  procw->preview_button = gtk_button_new_with_mnemonic ("_Preview");
  procw->pause_button = gtk_button_new_with_mnemonic ("Pau_se");

  gtk_dialog_add_action_widget (GTK_DIALOG (procw->processing), procw->preview_button, 1);
  gtk_widget_hide(procw->preview_button);
  GTK_WIDGET_SET_FLAGS (procw->preview_button, GTK_CAN_DEFAULT);
    
    gtk_dialog_add_action_widget (GTK_DIALOG (procw->processing), procw->pause_button, 0);
    gtk_widget_hide(procw->pause_button);
    GTK_WIDGET_SET_FLAGS (procw->pause_button, GTK_CAN_DEFAULT);


  if (cfile->opening==TRUE&&(cfile->frames>0||cfile->opening_loc==TRUE)) {
  // the "enough" button for opening
   gtk_dialog_add_action_widget (GTK_DIALOG (procw->processing), procw->stop_button, 0);
   gtk_widget_show(procw->stop_button);
   GTK_WIDGET_SET_FLAGS (procw->stop_button, GTK_CAN_DEFAULT);
  }
  
  procw->cancel_button = gtk_button_new_with_mnemonic ("_Cancel");
  gtk_dialog_add_action_widget (GTK_DIALOG (procw->processing), procw->cancel_button, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (procw->cancel_button, GTK_CAN_DEFAULT);

  g_signal_connect (GTK_OBJECT (procw->stop_button), "clicked",
		    G_CALLBACK (on_stop_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (procw->pause_button), "clicked",
                      G_CALLBACK (on_effects_paused),
                      NULL);

  g_signal_connect (GTK_OBJECT (procw->preview_button), "clicked",
                      G_CALLBACK (on_preview_clicked),
                      NULL);


  g_signal_connect (GTK_OBJECT (procw->cancel_button), "clicked",
                      G_CALLBACK (on_button3_clicked),
                      NULL);

  g_signal_connect (GTK_OBJECT (procw->processing), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  gtk_widget_grab_default (procw->pause_button);

  return procw;
}




GtkWidget*
create_colsel (void)
{
  GtkWidget *colorselectiondialog1;
  GtkWidget *ok_button2;
  GtkWidget *cancel_button2;
  GtkWidget *help_button1;

  colorselectiondialog1 = gtk_color_selection_dialog_new (_("LiVES: - Select Colour"));
  gtk_window_set_position (GTK_WINDOW (colorselectiondialog1), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(colorselectiondialog1),GTK_WINDOW(mainw->LiVES));

  gtk_window_set_modal (GTK_WINDOW (colorselectiondialog1), TRUE);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_fg (colorselectiondialog1, GTK_STATE_NORMAL, &palette->normal_fore);
    gtk_widget_modify_bg (colorselectiondialog1, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (colorselectiondialog1, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }
  ok_button2 = GTK_COLOR_SELECTION_DIALOG (colorselectiondialog1)->ok_button;
  gtk_widget_show (ok_button2);
  GTK_WIDGET_SET_FLAGS (ok_button2, GTK_CAN_DEFAULT);

  cancel_button2 = GTK_COLOR_SELECTION_DIALOG (colorselectiondialog1)->cancel_button;
  gtk_widget_show (cancel_button2);
  GTK_WIDGET_SET_FLAGS (cancel_button2, GTK_CAN_DEFAULT);

  help_button1 = GTK_COLOR_SELECTION_DIALOG (colorselectiondialog1)->help_button;
  //GTK_WIDGET_SET_FLAGS (help_button1, GTK_CAN_DEFAULT);

  colorizew->colour_selection = GTK_COLOR_SELECTION_DIALOG (colorselectiondialog1)->colorsel;
  gtk_widget_show (colorizew->colour_selection);
  gtk_color_selection_set_has_opacity_control (GTK_COLOR_SELECTION (colorizew->colour_selection), FALSE);

  g_signal_connect (GTK_OBJECT (cancel_button2), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      NULL);


  return colorselectiondialog1;
}




fileinfo*
create_info_window (gint audio_channels)
{
 

  GtkWidget *vbox5;
  GtkWidget *frame4;
  GtkWidget *fixed3;
  GtkWidget *fixed5;
  GtkWidget *fixed6;
  GtkWidget *label45;
  GtkWidget *label46;
  GtkWidget *label47;
  GtkWidget *label48;
  GtkWidget *label44;
  GtkWidget *label43;
  GtkWidget *label40;
  GtkWidget *label50;
  GtkWidget *label51;
  GtkWidget *label52;
  GtkWidget *label53;
  GtkWidget *frame5;
  GtkWidget *left;
  GtkWidget *frame6;
  GtkWidget *right;
  GtkWidget *button8;

  fileinfo *filew=(fileinfo*)(g_malloc(sizeof(fileinfo)));

  filew->info_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_position (GTK_WINDOW (filew->info_window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (filew->info_window), TRUE);

  gtk_widget_modify_bg(filew->info_window, GTK_STATE_NORMAL, &palette->normal_back);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (filew->info_window), vbox5);

  if (mainw->fs==TRUE&&mainw->sep_win==TRUE&&mainw->playing_file>-1) {
    gtk_window_set_transient_for(GTK_WINDOW(filew->info_window),GTK_WINDOW(mainw->play_window));
  }
  else {
    gtk_window_set_transient_for(GTK_WINDOW(filew->info_window),GTK_WINDOW(mainw->LiVES));
  }

  if (cfile->frames>0) {
    frame4 = gtk_frame_new (NULL);
    gtk_widget_set_size_request (frame4, 460, 340);
    gtk_widget_show (frame4);
    gtk_box_pack_start (GTK_BOX (vbox5), frame4, TRUE, TRUE, 0);
    gtk_widget_modify_bg(frame4, GTK_STATE_NORMAL, &palette->normal_back);
    
    fixed3 = gtk_fixed_new ();
    gtk_widget_show (fixed3);
    gtk_container_add (GTK_CONTAINER (frame4), fixed3);
    
    filew->textview24 = gtk_text_view_new ();
    gtk_widget_show (filew->textview24);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview24, 96, 48);
    gtk_widget_set_size_request (filew->textview24, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview24), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview24), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview24), FALSE);
    
    filew->textview25 = gtk_text_view_new ();
    gtk_widget_show (filew->textview25);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview25, 288, 48);
    gtk_widget_set_size_request (filew->textview25, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview25), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview25), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview25), FALSE);
    
    filew->textview26 = gtk_text_view_new ();
    gtk_widget_show (filew->textview26);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview26, 96, 136);
    gtk_widget_set_size_request (filew->textview26, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview26), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview26), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview26), FALSE);
    
    filew->textview27 = gtk_text_view_new ();
    gtk_widget_show (filew->textview27);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview27, 288, 136);
    gtk_widget_set_size_request (filew->textview27, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview27), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview27), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview27), FALSE);
    
    filew->textview28 = gtk_text_view_new ();
    gtk_widget_show (filew->textview28);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview28, 288, 224);
    gtk_widget_set_size_request (filew->textview28, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview28), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview28), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview28), FALSE);
    
    filew->textview29 = gtk_text_view_new ();
    gtk_widget_show (filew->textview29);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview29, 96, 224);
    gtk_widget_set_size_request (filew->textview29, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview29), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview29), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview29), FALSE);
    
    label45 = gtk_label_new (_("Format"));
    gtk_widget_show (label45);
    gtk_fixed_put (GTK_FIXED (fixed3), label45, 44, 64);
    gtk_widget_set_size_request (label45, 45, 16);
    gtk_label_set_justify (GTK_LABEL (label45), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label45, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label46 = gtk_label_new (_("Size"));
    gtk_widget_show (label46);
    gtk_fixed_put (GTK_FIXED (fixed3), label46, 52, 152);
    gtk_widget_set_size_request (label46, 39, 16);
    gtk_label_set_justify (GTK_LABEL (label46), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label46, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label47 = gtk_label_new (_("File Size"));
    gtk_widget_show (label47);
    gtk_fixed_put (GTK_FIXED (fixed3), label47, 28, 240);
    gtk_widget_set_size_request (label47, 65, 16);
    gtk_label_set_justify (GTK_LABEL (label47), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label47, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label48 = gtk_label_new (_("Total time"));
    gtk_widget_show (label48);
    gtk_fixed_put (GTK_FIXED (fixed3), label48, 220, 240);
    gtk_widget_set_size_request (label48, 63, 16);
    gtk_label_set_justify (GTK_LABEL (label48), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label48, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label44 = gtk_label_new (_("FPS"));
    gtk_widget_show (label44);
    gtk_fixed_put (GTK_FIXED (fixed3), label44, 246, 64);
    gtk_widget_set_size_request (label44, 39, 16);
    gtk_label_set_justify (GTK_LABEL (label44), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label44, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label43 = gtk_label_new ("Frames");
    gtk_widget_show (label43);
    gtk_fixed_put (GTK_FIXED (fixed3), label43, 236, 152);
    gtk_widget_set_size_request (label43, 45, 16);
    gtk_label_set_justify (GTK_LABEL (label43), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label43, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label40 = gtk_label_new ("Video");
    gtk_widget_show (label40);
    gtk_frame_set_label_widget (GTK_FRAME (frame4), label40);
    gtk_label_set_justify (GTK_LABEL (label40), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label40, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  if (audio_channels>0) {
    frame5 = gtk_frame_new (NULL);
    gtk_widget_set_size_request (frame5, 460, 100);
    gtk_widget_show (frame5);
    gtk_box_pack_start (GTK_BOX (vbox5), frame5, TRUE, TRUE, 0);
    gtk_widget_modify_bg(frame5, GTK_STATE_NORMAL, &palette->normal_back);

    if (audio_channels>1) {
      left = gtk_label_new ("Left Audio");
    }
    else {
      left = gtk_label_new ("Audio");
    }

    gtk_widget_show (left);
    gtk_frame_set_label_widget (GTK_FRAME (frame5), left);
    gtk_label_set_justify (GTK_LABEL (left), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(left, GTK_STATE_NORMAL, &palette->normal_fore);

    fixed5 = gtk_fixed_new ();
    gtk_widget_show (fixed5);
    gtk_container_add (GTK_CONTAINER (frame5), fixed5);


    filew->textview_ltime = gtk_text_view_new ();
    gtk_widget_show (filew->textview_ltime);
    gtk_fixed_put (GTK_FIXED (fixed5), filew->textview_ltime, 288, 16);
    gtk_widget_set_size_request (filew->textview_ltime, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview_ltime), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview_ltime), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview_ltime), FALSE);
    
    label50 = gtk_label_new ("Total time");
    gtk_widget_show (label50);
    gtk_fixed_put (GTK_FIXED (fixed5), label50, 220, 32);
    gtk_widget_set_size_request (label50, 63, 16);
    gtk_label_set_justify (GTK_LABEL (label50), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label50, GTK_STATE_NORMAL, &palette->normal_fore);

    filew->textview_lrate = gtk_text_view_new ();
    gtk_widget_show (filew->textview_lrate);
    gtk_fixed_put (GTK_FIXED (fixed5), filew->textview_lrate, 96, 16);
    gtk_widget_set_size_request (filew->textview_lrate, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview_lrate), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview_lrate), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview_lrate), FALSE);
    
    label52 = gtk_label_new (_("Rate/size"));
    gtk_widget_show (label52);
    gtk_fixed_put (GTK_FIXED (fixed5), label52, 30, 32);
    gtk_widget_set_size_request (label52, 63, 16);
    gtk_label_set_justify (GTK_LABEL (label52), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label52, GTK_STATE_NORMAL, &palette->normal_fore);


    if (audio_channels>1) {
      frame6 = gtk_frame_new (NULL);
      gtk_widget_set_size_request (frame6, 460, 100);
      gtk_widget_show (frame6);
      gtk_box_pack_start (GTK_BOX (vbox5), frame6, TRUE, TRUE, 0);
      gtk_widget_modify_bg(frame6, GTK_STATE_NORMAL, &palette->normal_back);

      fixed6 = gtk_fixed_new ();
      gtk_widget_show (fixed6);
      gtk_container_add (GTK_CONTAINER (frame6), fixed6);
      
      filew->textview_rtime = gtk_text_view_new ();
      gtk_widget_show (filew->textview_rtime);
      gtk_fixed_put (GTK_FIXED (fixed6), filew->textview_rtime, 288, 16);
      gtk_widget_set_size_request (filew->textview_rtime, 120, 50);
      gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview_rtime), FALSE);
      gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview_rtime), GTK_JUSTIFY_CENTER);
      gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview_rtime), FALSE);
      
      label51 = gtk_label_new (_("Total time"));
      gtk_widget_show (label51);
      gtk_fixed_put (GTK_FIXED (fixed6), label51, 220, 32);
      gtk_widget_set_size_request (label51, 63, 16);
      gtk_label_set_justify (GTK_LABEL (label51), GTK_JUSTIFY_LEFT);
      gtk_widget_modify_fg(label51, GTK_STATE_NORMAL, &palette->normal_fore);
	  
      filew->textview_rrate = gtk_text_view_new ();
      gtk_widget_show (filew->textview_rrate);
      gtk_fixed_put (GTK_FIXED (fixed6), filew->textview_rrate, 96, 16);
      gtk_widget_set_size_request (filew->textview_rrate, 120, 50);
      gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview_rrate), FALSE);
      gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview_rrate), GTK_JUSTIFY_CENTER);
      gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview_rrate), FALSE);
      
      label53 = gtk_label_new (_("Rate/size"));
      gtk_widget_show (label53);
      gtk_fixed_put (GTK_FIXED (fixed6), label53, 30, 32);
      gtk_widget_set_size_request (label53, 63, 16);
      gtk_label_set_justify (GTK_LABEL (label53), GTK_JUSTIFY_LEFT);
      gtk_widget_modify_fg(label53, GTK_STATE_NORMAL, &palette->normal_fore);

      
      right = gtk_label_new (_("Right audio"));
      gtk_widget_show (right);
      gtk_frame_set_label_widget (GTK_FRAME (frame6), right);
      gtk_label_set_justify (GTK_LABEL (right), GTK_JUSTIFY_LEFT);
      gtk_widget_modify_fg(right, GTK_STATE_NORMAL, &palette->normal_fore);
    }
  }

  button8 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (button8);
  gtk_box_pack_end (GTK_BOX (vbox5), button8, FALSE, FALSE, 0);
  gtk_widget_set_size_request (button8, -1, 78);
  gtk_container_set_border_width (GTK_CONTAINER (button8), 12);
  gtk_button_set_relief (GTK_BUTTON (button8), GTK_RELIEF_NONE);
  GTK_WIDGET_SET_FLAGS (button8, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button8);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(button8, GTK_STATE_SELECTED|GTK_STATE_ACTIVE, &palette->white);
  }

  g_signal_connect (GTK_OBJECT (button8), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      filew);

  g_signal_connect (GTK_OBJECT (filew->info_window), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return filew;
}






//Warning dialog
GtkWidget*
create_dialog2 (gint warn_mask_number)
{
  GtkWidget *dialog2;
  GtkWidget *dialog_vbox2;
  GtkWidget *dialog_action_area2;
  GtkWidget *warning_cancelbutton;
  GtkWidget *warning_okbutton;
  GtkWidget *checkbutton8;

  mainw->warn_mask=FALSE;

  dialog2 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog2), _("LiVES: - Warning !"));
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(dialog2),FALSE);
  }
  gtk_widget_modify_bg(dialog2, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_transient_for(GTK_WINDOW(dialog2),GTK_WINDOW(mainw->LiVES));

  gtk_window_set_modal (GTK_WINDOW (dialog2), TRUE);
  gtk_window_set_position (GTK_WINDOW (dialog2), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (dialog2), 300, 200);

  dialog_vbox2 = GTK_DIALOG (dialog2)->vbox;
  gtk_widget_show (dialog_vbox2);

  mainw->warning_label = gtk_label_new (_("warning"));
  gtk_widget_show (mainw->warning_label);
  gtk_box_pack_start (GTK_BOX (dialog_vbox2), mainw->warning_label, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (mainw->warning_label), GTK_JUSTIFY_FILL);
  gtk_label_set_line_wrap (GTK_LABEL (mainw->warning_label), TRUE);
  gtk_widget_modify_fg(mainw->warning_label, GTK_STATE_NORMAL, &palette->normal_fore);

  if (warn_mask_number>0) {
    checkbutton8 = gtk_check_button_new_with_mnemonic (_("Do not show this warning again"));
    gtk_widget_show (checkbutton8);
    gtk_box_pack_start (GTK_BOX (dialog_vbox2), checkbutton8, FALSE, FALSE, 0);
    g_signal_connect (GTK_OBJECT (checkbutton8), "toggled",
                      G_CALLBACK (on_warn_mask_toggled),
                      NULL);
  }

  dialog_action_area2 = GTK_DIALOG (dialog2)->action_area;
  gtk_widget_show (dialog_action_area2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);

  warning_cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (warning_cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog2), warning_cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (warning_cancelbutton, GTK_CAN_DEFAULT);

  warning_okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (warning_okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog2), warning_okbutton, GTK_RESPONSE_OK);


  GTK_WIDGET_SET_FLAGS (warning_okbutton, GTK_CAN_DEFAULT);

  return dialog2;
}



// Information/error dialog
GtkWidget*
create_dialog3 (const gchar *text, gboolean is_blocking)
{
  GtkWidget *dialog3;
  GtkWidget *dialog_vbox3;
  GtkWidget *info_text;
  GtkWidget *dialog_action_area3;
  GtkWidget *info_ok_button;
  gchar *form_text;
  
  dialog3 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog3), "LiVES");
  gtk_window_set_modal (GTK_WINDOW (dialog3), TRUE);
  gtk_window_set_position (GTK_WINDOW (dialog3), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (dialog3), 300, 200);
  if (mainw!=NULL&&mainw->is_ready) {
    gtk_window_set_transient_for(GTK_WINDOW(dialog3),GTK_WINDOW(mainw->LiVES));
    gtk_widget_modify_bg(dialog3, GTK_STATE_NORMAL, &palette->normal_back);
    if (palette->style&STYLE_1) {
      gtk_dialog_set_has_separator(GTK_DIALOG(dialog3),FALSE);
    }
  }

  dialog_vbox3 = GTK_DIALOG (dialog3)->vbox;
  gtk_widget_show (dialog_vbox3);
  
  form_text=g_strdup_printf("\n\n%s",text);

  info_text = gtk_label_new (form_text);
  g_free(form_text);

  if (mainw!=NULL&&mainw->is_ready) {
    gtk_widget_modify_fg(info_text, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  gtk_label_set_selectable (GTK_LABEL (info_text), TRUE);

  gtk_widget_show (info_text);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), info_text, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (info_text), GTK_JUSTIFY_CENTER);
  gtk_label_set_line_wrap (GTK_LABEL (info_text), TRUE);
  
  dialog_action_area3 = GTK_DIALOG (dialog3)->action_area;
  gtk_widget_show (dialog_action_area3);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area3), GTK_BUTTONBOX_END);
  
  info_ok_button = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (info_ok_button);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog3), info_ok_button, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (info_ok_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (info_ok_button);

  if (is_blocking==FALSE) {
    g_signal_connect (GTK_OBJECT (info_ok_button), "clicked",
		      G_CALLBACK (on_info_ok_button_clicked),
		      NULL);
    
  }

  gtk_widget_grab_focus (info_ok_button);
  return dialog3;
}



GtkWidget*
create_text_window (const gchar *title, const gchar *text)
{
  // general text window
  GtkWidget *text_window;
  GtkWidget *dialog_vbox20;
  GtkWidget *scrolledwindow3;
  GtkWidget *textview2;
  GtkWidget *dialog_action_area20;
  GtkWidget *okbutton17;

  text_window = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (text_window), g_strconcat ("LiVES:- ",title,NULL));
  gtk_window_set_position (GTK_WINDOW (text_window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (text_window), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (text_window), 600, 400);
  gtk_window_set_transient_for(GTK_WINDOW(text_window),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(text_window, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(text_window),FALSE);
  }

  dialog_vbox20 = GTK_DIALOG (text_window)->vbox;
  gtk_widget_show (dialog_vbox20);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox20), scrolledwindow3, TRUE, TRUE, 0);

  textview2 = gtk_text_view_new ();
  gtk_widget_show (textview2);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), textview2);

  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview2), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview2), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview2), FALSE);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_base(textview2, GTK_STATE_NORMAL, &palette->info_base);
    gtk_widget_modify_text(textview2, GTK_STATE_NORMAL, &palette->info_text);
  }

  gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview2)), text, -1);

  dialog_action_area20 = GTK_DIALOG (text_window)->action_area;
  gtk_widget_show (dialog_action_area20);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area20), GTK_BUTTONBOX_END);

  okbutton17 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton17);
  gtk_widget_set_size_request (okbutton17, 586, 78);
  gtk_container_set_border_width (GTK_CONTAINER (okbutton17), 12);
  gtk_button_set_relief (GTK_BUTTON (okbutton17), GTK_RELIEF_NONE);

  gtk_dialog_add_action_widget (GTK_DIALOG (text_window), okbutton17, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton17, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton17);

  g_signal_connect (GTK_OBJECT (okbutton17), "clicked",
		      G_CALLBACK (on_cancel_button1_clicked),
		      NULL);
  
  gtk_widget_grab_focus (okbutton17);

  return text_window;
}









void
add_to_winmenu(void) {
  GtkWidget *active_image;
  cfile->menuentry = gtk_image_menu_item_new_with_label(cfile->name);
  gtk_widget_show (cfile->menuentry);
  gtk_container_add (GTK_CONTAINER (mainw->winmenu), cfile->menuentry);

  gtk_widget_set_sensitive (cfile->menuentry, TRUE);
  g_signal_connect (GTK_OBJECT (cfile->menuentry), "activate",
                      G_CALLBACK (switch_clip_activate),
                      NULL);

  if (cfile->opening==FALSE) {
    active_image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
  }
  else {
    active_image = gtk_image_new_from_stock ("gtk-no", GTK_ICON_SIZE_MENU);
  }
  gtk_widget_show (active_image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (cfile->menuentry), active_image);
  mainw->clips_available++;
  mainw->cliplist = g_list_append (mainw->cliplist, GINT_TO_POINTER (mainw->current_file));
}




void
remove_from_winmenu() {
  gtk_container_remove(GTK_CONTAINER(mainw->winmenu), cfile->menuentry);
  gtk_widget_destroy(cfile->menuentry);
  mainw->clips_available--;
  mainw->cliplist=g_list_remove (mainw->cliplist, GINT_TO_POINTER (mainw->current_file));
}



_insertw*
create_insert_dialog ()
{
  GtkWidget *dialog_vbox3;
  GtkWidget *hbox15;
  GtkWidget *label57;
  GtkObject *spinbutton_times_adj;
  GtkWidget *table2;
  GtkWidget *radiobutton1;
  GSList *radiobutton1_group = NULL;
  GSList *radiobutton2_group = NULL;
  GtkWidget *radiobutton4;
  GtkWidget *vseparator1;
  GtkWidget *vseparator2;
  GtkWidget *hseparator1;
  GtkWidget *dialog_action_area3;
  GtkWidget *cancelbutton1;
  GtkWidget *okbutton1;

  _insertw *insertw=(_insertw*)(g_malloc(sizeof(_insertw)));

  insertw->insert_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (insertw->insert_dialog), _("LiVES: - Insert"));
  gtk_window_set_position (GTK_WINDOW (insertw->insert_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (insertw->insert_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (insertw->insert_dialog), 300, 200);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_bg (insertw->insert_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (insertw->insert_dialog, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }
  gtk_window_set_transient_for(GTK_WINDOW(insertw->insert_dialog),GTK_WINDOW(mainw->LiVES));

  dialog_vbox3 = GTK_DIALOG (insertw->insert_dialog)->vbox;
  gtk_widget_show (dialog_vbox3);

  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox15);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), hbox15, TRUE, TRUE, 0);

  label57 = gtk_label_new (_("Number of times to insert"));
  gtk_widget_show (label57);
  gtk_box_pack_start (GTK_BOX (hbox15), label57, TRUE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label57), GTK_JUSTIFY_LEFT);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_fg (label57, GTK_STATE_NORMAL, &palette->normal_fore);
  }
  spinbutton_times_adj = gtk_adjustment_new (1, 1, 10000, 1, 10, 100);
  insertw->spinbutton_times = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_times_adj), 1, 0);
  gtk_widget_show (insertw->spinbutton_times);
  gtk_box_pack_start (GTK_BOX (hbox15), insertw->spinbutton_times, TRUE, FALSE, 0);

  insertw->fit_checkbutton = gtk_check_button_new_with_mnemonic ("_Insert to fit audio");
  gtk_widget_show (insertw->fit_checkbutton);
  gtk_box_pack_start (GTK_BOX (hbox15), insertw->fit_checkbutton, TRUE, FALSE, 0);
  gtk_widget_set_sensitive(GTK_WIDGET(insertw->fit_checkbutton),cfile->achans>0&&clipboard->achans==0);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), hseparator1, TRUE, TRUE, 0);

  table2 = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), table2, TRUE, TRUE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table2), 43);

  radiobutton1 = gtk_radio_button_new_with_mnemonic (NULL, _("Insert _before selection"));
  gtk_widget_show (radiobutton1);
  gtk_table_attach (GTK_TABLE (table2), radiobutton1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_container_set_border_width (GTK_CONTAINER (radiobutton1), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton1), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobutton1),!(mainw->insert_after));
  if (cfile->frames==0) {
    gtk_widget_set_sensitive (radiobutton1, FALSE);
  }

  radiobutton4 = gtk_radio_button_new_with_mnemonic (NULL, _("Insert _after selection"));
  gtk_widget_show (radiobutton4);
  gtk_table_attach (GTK_TABLE (table2), radiobutton4, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (radiobutton4), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton4), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton4));
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobutton4),mainw->insert_after);

  insertw->with_sound = gtk_radio_button_new_with_mnemonic (NULL, _("Insert _with sound"));
  gtk_widget_show (insertw->with_sound);
  gtk_table_attach (GTK_TABLE (table2), insertw->with_sound, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (insertw->with_sound), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (insertw->with_sound), radiobutton2_group);
  radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (insertw->with_sound));

  insertw->without_sound = gtk_radio_button_new_with_mnemonic (NULL, _("Insert with_out sound"));
  gtk_widget_show (insertw->without_sound);
  gtk_table_attach (GTK_TABLE (table2), insertw->without_sound, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (insertw->without_sound), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (insertw->without_sound), radiobutton2_group);
  radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (insertw->without_sound));

  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(insertw->with_sound),clipboard->achans>0&&mainw->ccpd_with_sound==TRUE);
  gtk_widget_set_sensitive (insertw->with_sound, clipboard->achans>0);
  gtk_widget_set_sensitive (insertw->without_sound, clipboard->achans>0);

  vseparator1 = gtk_vseparator_new ();
  gtk_widget_show (vseparator1);
  gtk_table_attach (GTK_TABLE (table2), vseparator1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  vseparator2 = gtk_vseparator_new ();
  gtk_widget_show (vseparator2);
  gtk_table_attach (GTK_TABLE (table2), vseparator2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  dialog_action_area3 = GTK_DIALOG (insertw->insert_dialog)->action_area;
  gtk_widget_show (dialog_action_area3);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area3), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (insertw->insert_dialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (insertw->insert_dialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
  gtk_widget_grab_default(okbutton1);
  gtk_widget_grab_focus(okbutton1);

  g_signal_connect (GTK_OBJECT (insertw->with_sound), "toggled",
                      G_CALLBACK (on_insertwsound_toggled),
                      NULL);
  g_signal_connect (GTK_OBJECT (radiobutton4), "toggled",
                      G_CALLBACK (on_insert_after_toggled),
                      NULL);
  g_signal_connect (GTK_OBJECT (cancelbutton1), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      insertw);
  g_signal_connect (GTK_OBJECT (okbutton1), "clicked",
                      G_CALLBACK (on_insert_activate),
                      NULL);
  g_signal_connect (GTK_OBJECT (insertw->fit_checkbutton), "toggled",
		    G_CALLBACK (on_insfitaudio_toggled),
		    NULL);
  g_signal_connect_after (GTK_OBJECT (insertw->spinbutton_times), "value_changed",
                      G_CALLBACK (on_spin_value_changed),
                      GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (insertw->insert_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return insertw;
}








GtkWidget*
create_neg_dialog (void)
{
  GtkWidget *neg_dialog;
  GtkWidget *dialog_vbox4;
  GtkWidget *hbox4;
  GtkWidget *label18;
  GtkObject *spinbutton_neg_adj;
  GtkWidget *spinbutton_neg;
  GtkObject *spinbutton_neg_skip_adj;
  GtkWidget *spinbutton_neg_skip;
  GtkWidget *label19;
  GtkWidget *label20;
  GtkWidget *label21;
  GtkWidget *dialog_action_area4;
  GtkWidget *cancelbutton2;
  GtkWidget *okbutton2;

  neg_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (neg_dialog), _("LiVES:  - Negate"));
  gtk_window_set_position (GTK_WINDOW (neg_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (neg_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (neg_dialog), 300, 200);
  gtk_window_set_transient_for(GTK_WINDOW(neg_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(neg_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(neg_dialog),FALSE);
  }

  dialog_vbox4 = GTK_DIALOG (neg_dialog)->vbox;
  gtk_widget_show (dialog_vbox4);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (dialog_vbox4), hbox4, TRUE, TRUE, 0);

  label18 = gtk_label_new (_("Negate "));
  gtk_widget_show (label18);
  gtk_box_pack_start (GTK_BOX (hbox4), label18, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label18), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label18, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton_neg_adj = gtk_adjustment_new (mainw->fx1_val, 1, cfile->end-cfile->start+1, 1, 10, 10);
  spinbutton_neg = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_neg_adj), 1, 0);
  gtk_widget_show (spinbutton_neg);
  gtk_box_pack_start (GTK_BOX (hbox4), spinbutton_neg, TRUE, TRUE, 0);

  label19 = gtk_label_new (_(" frames,"));
  gtk_widget_show (label19);
  gtk_box_pack_start (GTK_BOX (hbox4), label19, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label19), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label19, GTK_STATE_NORMAL, &palette->normal_fore);

  label20 = gtk_label_new (_(" then skip "));
  gtk_widget_show (label20);
  gtk_box_pack_start (GTK_BOX (hbox4), label20, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label20), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label20, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton_neg_skip_adj = gtk_adjustment_new (mainw->fx2_val, 0, cfile->end-cfile->start+1, 1, 10, 10);
  spinbutton_neg_skip = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_neg_skip_adj), 1, 0);
  gtk_widget_show (spinbutton_neg_skip);
  gtk_box_pack_start (GTK_BOX (hbox4), spinbutton_neg_skip, TRUE, TRUE, 0);

  label21 = gtk_label_new (_(" frames"));
  gtk_widget_show (label21);
  gtk_box_pack_start (GTK_BOX (hbox4), label21, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label21), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label21, GTK_STATE_NORMAL, &palette->normal_fore);

  dialog_action_area4 = GTK_DIALOG (neg_dialog)->action_area;
  gtk_widget_show (dialog_action_area4);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area4), GTK_BUTTONBOX_END);

  cancelbutton2 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton2);
  gtk_dialog_add_action_widget (GTK_DIALOG (neg_dialog), cancelbutton2, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton2, GTK_CAN_DEFAULT);

  okbutton2 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton2);
  gtk_dialog_add_action_widget (GTK_DIALOG (neg_dialog), okbutton2, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton2, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton2);

  g_signal_connect_after (GTK_OBJECT (spinbutton_neg), "value_changed",
                      G_CALLBACK (on_spin_value_changed),
                      GINT_TO_POINTER (1));
  g_signal_connect_after (GTK_OBJECT (spinbutton_neg_skip), "value_changed",
                      G_CALLBACK (on_spin_value_changed),
                      GINT_TO_POINTER (2));
  g_signal_connect (GTK_OBJECT (cancelbutton2), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      NULL);
  g_signal_connect (GTK_OBJECT (okbutton2), "clicked",
                      G_CALLBACK (on_neg_ok_clicked),
                      NULL);

  return neg_dialog;
}







_resizew*
create_resize_dialog (gint hsize, gint vsize)
{
  GtkWidget *dialog_vbox5;
  GtkWidget *vbox5;
  GtkWidget *resize_label;
  GtkWidget *hbox5;
  GtkObject *spinbutton17_adj;
  GtkWidget *label21;
  GtkObject *spinbutton18_adj;
  GtkWidget *aspect_button;
  GtkWidget *dialog_action_area5;
  GtkWidget *cancelbutton3;
  GtkWidget *okbutton3;
  gchar label_text[256];

  _resizew *resizew=(_resizew*)(g_malloc(sizeof(_resizew)));

  resizew->resize_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (resizew->resize_dialog), _("LiVES: - Resize"));
  gtk_window_set_position (GTK_WINDOW (resizew->resize_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (resizew->resize_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (resizew->resize_dialog), 300, 200);
  gtk_window_set_transient_for(GTK_WINDOW(resizew->resize_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(resizew->resize_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(resizew->resize_dialog),FALSE);
  }

  dialog_vbox5 = GTK_DIALOG (resizew->resize_dialog)->vbox;
  gtk_widget_show (dialog_vbox5);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox5);
  gtk_box_pack_start (GTK_BOX (dialog_vbox5), vbox5, TRUE, TRUE, 0);

  g_snprintf(label_text,256,"\n   Current size is\n%d X %d\n\n\n  New size",hsize,vsize);
  resize_label = gtk_label_new (label_text);
  gtk_widget_show (resize_label);
  gtk_box_pack_start (GTK_BOX (vbox5), resize_label, FALSE, FALSE, 0);
  gtk_widget_modify_fg(resize_label, GTK_STATE_NORMAL, &palette->normal_fore);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox5, TRUE, TRUE, 0);

  spinbutton17_adj = gtk_adjustment_new (hsize, 1, 2000, 1, 10, 100);
  resizew->spinbutton_hsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton17_adj), 1, 0);
  gtk_widget_show (resizew->spinbutton_hsize);
  gtk_box_pack_start (GTK_BOX (hbox5), resizew->spinbutton_hsize, TRUE, FALSE, 0);

  label21 = gtk_label_new (_(" X "));
  gtk_widget_show (label21);
  gtk_box_pack_start (GTK_BOX (hbox5), label21, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label21), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label21, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton18_adj = gtk_adjustment_new (vsize, 1, 2000, 1, 10, 100);
  resizew->spinbutton_vsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton18_adj), 1, 0);
  gtk_widget_show (resizew->spinbutton_vsize);
  gtk_box_pack_start (GTK_BOX (hbox5), resizew->spinbutton_vsize, TRUE, FALSE, 0);

  aspect_button = gtk_check_button_new_with_mnemonic (_("Keep aspect ratio"));
  gtk_widget_show (aspect_button);
  gtk_box_pack_start (GTK_BOX (vbox5), aspect_button, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (aspect_button), 20);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (aspect_button), TRUE);

  dialog_action_area5 = GTK_DIALOG (resizew->resize_dialog)->action_area;
  gtk_widget_show (dialog_action_area5);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area5), GTK_BUTTONBOX_END);

  cancelbutton3 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton3);
  gtk_dialog_add_action_widget (GTK_DIALOG (resizew->resize_dialog), cancelbutton3, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton3, GTK_CAN_DEFAULT);

  okbutton3 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton3);
  gtk_dialog_add_action_widget (GTK_DIALOG (resizew->resize_dialog), okbutton3, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton3, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton3);

  resizew->spin_hsize_func=g_signal_connect_after (GTK_OBJECT (resizew->spinbutton_hsize), 
				    "value_changed",
				    G_CALLBACK (on_resize_hsize_value_changed),
				    NULL);
  resizew->spin_vsize_func=g_signal_connect_after (GTK_OBJECT (resizew->spinbutton_vsize), 
				    "value_changed",
				    G_CALLBACK (on_resize_vsize_value_changed),
				    NULL);
  g_signal_connect (GTK_OBJECT (aspect_button), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (cancelbutton3), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    resizew);
  g_signal_connect (GTK_OBJECT (okbutton3), "clicked",
		    G_CALLBACK (on_resize_ok_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (resizew->resize_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return resizew;
}







GtkWidget*
create_new_pb_speed (gshort type)
{
  GtkWidget *new_pb_speed;
  GtkWidget *dialog_vbox6;
  GtkWidget *vbox6;
  GtkWidget *pb_speed_text;
  GtkObject *spinbutton_pb_speed_adj;
  GtkWidget *spinbutton_pb_speed;
  GtkWidget *dialog_action_area6;
  GtkWidget *cancelbutton4;
  GtkWidget *change_pb_ok;
  GtkWidget *change_audio_speed;
  gchar label_text[256];

  new_pb_speed = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (new_pb_speed), 10);
  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (new_pb_speed), _("LiVES: - Change playback speed"));
  }
  else if (type==2) {
    gtk_window_set_title (GTK_WINDOW (new_pb_speed), _("LiVES: - Resample Video"));
  }
  gtk_window_set_position (GTK_WINDOW (new_pb_speed), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (new_pb_speed), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (new_pb_speed), 300, 200);
  gtk_window_set_transient_for(GTK_WINDOW(new_pb_speed),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(new_pb_speed, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(new_pb_speed),FALSE);
  }

  dialog_vbox6 = GTK_DIALOG (new_pb_speed)->vbox;
  gtk_widget_show (dialog_vbox6);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox6);
  gtk_box_pack_start (GTK_BOX (dialog_vbox6), vbox6, TRUE, TRUE, 0);

  if (type==1) {
    g_snprintf(label_text,256,"\n\nCurrent playback speed is %.3f frames per second.\n\nPlease enter the desired playback speed\nin frames per second",cfile->fps);
  }
  else if (type==2) {
    g_snprintf(label_text,256,"\n\nCurrent playback speed is %.3f frames per second.\n\nPlease enter the resampled rate\nin frames per second",cfile->fps);
  }
  pb_speed_text = gtk_label_new (label_text);
  gtk_widget_show (pb_speed_text);
  gtk_box_pack_start (GTK_BOX (vbox6), pb_speed_text, TRUE, TRUE, 0);
  gtk_widget_modify_fg(pb_speed_text, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton_pb_speed_adj = gtk_adjustment_new (cfile->fps, 1, FPS_MAX, 0.01, .1, .5);
  spinbutton_pb_speed = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_pb_speed_adj), 1, 3);
  gtk_widget_show (spinbutton_pb_speed);
  gtk_box_pack_start (GTK_BOX (vbox6), spinbutton_pb_speed, FALSE, FALSE, 8);
  gtk_widget_set_size_request (spinbutton_pb_speed, 10, -1);
  gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbutton_pb_speed)->entry)), TRUE);

  change_audio_speed = gtk_check_button_new_with_mnemonic (_("Change the audio speed as well"));
  if (type==1&&cfile->achans>0) {
    gtk_widget_show (change_audio_speed);
  }
  gtk_box_pack_start (GTK_BOX (vbox6), change_audio_speed, FALSE, FALSE, 0);

  dialog_action_area6 = GTK_DIALOG (new_pb_speed)->action_area;
  gtk_widget_show (dialog_action_area6);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area6), GTK_BUTTONBOX_END);

  cancelbutton4 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton4);
  gtk_dialog_add_action_widget (GTK_DIALOG (new_pb_speed), cancelbutton4, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton4, GTK_CAN_DEFAULT);

  change_pb_ok = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (change_pb_ok);
  gtk_dialog_add_action_widget (GTK_DIALOG (new_pb_speed), change_pb_ok, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (change_pb_ok, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (change_pb_ok);

  g_signal_connect (GTK_OBJECT (change_audio_speed), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (cancelbutton4), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    NULL);
  if (type==1) {
    g_signal_connect (GTK_OBJECT (change_pb_ok), "clicked",
		      G_CALLBACK (on_change_speed_ok_clicked),
		      NULL);
  }
  else if (type==2) {
    g_signal_connect (GTK_OBJECT (change_pb_ok), "clicked",
		      G_CALLBACK (on_resample_vid_ok),
		      NULL);
  }
  g_signal_connect_after (GTK_OBJECT (spinbutton_pb_speed), "value_changed",
			  G_CALLBACK (on_spin_value_changed),
			  GINT_TO_POINTER (1));


  return new_pb_speed;
}








GtkWidget*
create_rotate_dialog (gshort type)
{
  // type 1 == rotate
  // type 2 == translate x
  // type 3 == translate y

  GtkWidget *rotate_dialog;
  GtkWidget *dialog_vbox7;
  GtkWidget *table3;
  GtkObject *spinbutton_rot_fr_adj;
  GtkWidget *spinbutton_rot_fr;
  GtkObject *spinbutton_rot_sel_adj;
  GtkWidget *spinbutton_rot_sel;
  GtkWidget *rotate_fr_button;
  GSList *rotate_fr_button_group = NULL;
  GtkWidget *rotate_sel_button;
  GtkWidget *label23;
  GtkWidget *label22;
  GtkWidget *dialog_action_area7;
  GtkWidget *cancelbutton5;
  GtkWidget *okbutton4;
  GtkWidget *checkbutton_wrap = NULL;

  rotate_dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (rotate_dialog), 10);

  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (rotate_dialog), _("LiVES: - Spin"));
  }
  else if (type==2) {
    gtk_window_set_title (GTK_WINDOW (rotate_dialog), _("LiVES: - Shift Horizontally"));
  }
  else {
    gtk_window_set_title (GTK_WINDOW (rotate_dialog), _("LiVES: - Shift Vertically"));
  }

  gtk_window_set_position (GTK_WINDOW (rotate_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (rotate_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (rotate_dialog), 300, 200);
  gtk_window_set_transient_for(GTK_WINDOW(rotate_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(rotate_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(rotate_dialog),FALSE);
  }

  dialog_vbox7 = GTK_DIALOG (rotate_dialog)->vbox;
  gtk_widget_show (dialog_vbox7);

  if (type==1) {
    table3 = gtk_table_new (2, 3, FALSE);
  }
  else {
    table3 = gtk_table_new (3, 3, FALSE);
  }
  gtk_widget_show (table3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox7), table3, TRUE, TRUE, 0);

  if (type==1) {
    spinbutton_rot_fr_adj = gtk_adjustment_new (1, -180, 180, 1, 10, 10);
    spinbutton_rot_fr = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_rot_fr_adj), 1, 0);
  }
  else {
    spinbutton_rot_fr_adj = gtk_adjustment_new (mainw->fx1_val, -cfile->hsize, cfile->hsize, 1, 1, 1);
    spinbutton_rot_fr = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_rot_fr_adj), 1, 2);
  }
  gtk_widget_show (spinbutton_rot_fr);
  gtk_table_attach (GTK_TABLE (table3), spinbutton_rot_fr, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton_rot_fr), TRUE);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinbutton_rot_fr), TRUE);

  if (type==1) {
    spinbutton_rot_sel_adj = gtk_adjustment_new (360, -1e+06, 1e+06, 180, 1, 360);
  }
  else {
    spinbutton_rot_sel_adj = gtk_adjustment_new (mainw->fx2_val, -cfile->hsize, cfile->hsize, 1, 10, 100);
  }
  spinbutton_rot_sel = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_rot_sel_adj), 1, 0);
  gtk_widget_show (spinbutton_rot_sel);
  gtk_table_attach (GTK_TABLE (table3), spinbutton_rot_sel, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton_rot_sel), TRUE);
  
  if (type==1) {
    rotate_fr_button = gtk_radio_button_new_with_mnemonic (NULL, _("Spin each frame by"));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rotate_fr_button), mainw->fx1_bool);
  }
  else {
    rotate_fr_button = gtk_radio_button_new_with_mnemonic (NULL, _("Shift step"));
  }
  gtk_widget_show (rotate_fr_button);
  gtk_table_attach (GTK_TABLE (table3), rotate_fr_button, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (rotate_fr_button), rotate_fr_button_group);
  rotate_fr_button_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (rotate_fr_button));

  if (type==1) {
    rotate_sel_button = gtk_radio_button_new_with_mnemonic (NULL, _("Spin selection by"));
  }
  else {
    rotate_sel_button = gtk_radio_button_new_with_mnemonic (NULL, _("Shift selection by"));
  }
  gtk_widget_show (rotate_sel_button);
  gtk_table_attach (GTK_TABLE (table3), rotate_sel_button, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (rotate_sel_button), rotate_fr_button_group);
  rotate_fr_button_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (rotate_sel_button));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rotate_sel_button),TRUE);

  if (type==1) {
    label23 = gtk_label_new (_("      degrees"));
  }
  else {
    label23 = gtk_label_new (_("       pixels"));
  }
  gtk_widget_show (label23);
  gtk_table_attach (GTK_TABLE (table3), label23, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label23), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label23), 0, 0.5);
  gtk_widget_modify_fg(label23, GTK_STATE_NORMAL, &palette->normal_fore);

  if (type==1) {
    label22 = gtk_label_new (_("      degrees"));
  }
  else {
    label22 = gtk_label_new (_("       pixels"));
  }
  gtk_widget_show (label22);
  gtk_table_attach (GTK_TABLE (table3), label22, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label22), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label22), 0, 0.5);
  gtk_widget_modify_fg(label22, GTK_STATE_NORMAL, &palette->normal_fore);

  if (type>1) {
    checkbutton_wrap = gtk_check_button_new_with_mnemonic (_("Wrap around"));
    gtk_widget_show (checkbutton_wrap);
    gtk_table_attach (GTK_TABLE (table3), checkbutton_wrap, 0, 1, 3, 4,
		      (GtkAttachOptions) (GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_wrap), mainw->fx2_bool);
  }

  dialog_action_area7 = GTK_DIALOG (rotate_dialog)->action_area;
  gtk_widget_show (dialog_action_area7);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area7), GTK_BUTTONBOX_END);

  cancelbutton5 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton5);
  gtk_dialog_add_action_widget (GTK_DIALOG (rotate_dialog), cancelbutton5, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton5, GTK_CAN_DEFAULT);

  okbutton4 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton4);
  gtk_dialog_add_action_widget (GTK_DIALOG (rotate_dialog), okbutton4, GTK_RESPONSE_OK);

  GTK_WIDGET_SET_FLAGS (okbutton4, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(okbutton4);

  g_signal_connect_after (GTK_OBJECT (spinbutton_rot_fr), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (1));
  g_signal_connect_after (GTK_OBJECT (spinbutton_rot_sel), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (2));
  g_signal_connect (GTK_OBJECT (rotate_fr_button), "toggled",
                      G_CALLBACK (on_boolean_toggled),
                      GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (cancelbutton5), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      NULL);
  if (type==1) {
    g_signal_connect (GTK_OBJECT (okbutton4), "clicked",
                      G_CALLBACK (on_spin_ok_clicked),
                      NULL);
  }
  else {
    g_signal_connect (GTK_OBJECT (checkbutton_wrap), "toggled",
		      G_CALLBACK (on_boolean_toggled),
		      GINT_TO_POINTER (2));
    if (type==2) {
      // shift x
      g_signal_connect (GTK_OBJECT (okbutton4), "clicked",
			G_CALLBACK (on_shift_ok_clicked),
			GINT_TO_POINTER (1));
    }
    else {
      // shift y
      g_signal_connect (GTK_OBJECT (okbutton4), "clicked",
			G_CALLBACK (on_shift_ok_clicked),
			GINT_TO_POINTER (2));
    }
  }

  return rotate_dialog;
}



GtkWidget*
create_shrink_dialog (void)
{
  GtkWidget *shrink_dialog;
  GtkWidget *dialog_vbox19;
  GtkWidget *vbox28;
  GtkWidget *hbox29;
  GtkWidget *label116;
  GtkObject *spinbutton58_adj;
  GtkWidget *spinbutton58;
  GtkWidget *label117;
  GtkWidget *hbox129;
  GtkWidget *label1116;
  GtkObject *spinbutton158_adj;
  GtkWidget *spinbutton158;
  GtkWidget *label1117;
  GtkWidget *frame9;
  GtkWidget *vbox29;
  GtkWidget *table12;
  GtkWidget *label119;
  GtkWidget *label120;
  GtkWidget *label121;
  GtkWidget *label122;
  GtkWidget *radiobutton38;
  GSList *radiobutton38_group = NULL;
  GtkWidget *radiobutton39;
  GtkWidget *radiobutton41;
  GtkWidget *radiobutton40;
  GtkWidget *radiobutton37;
  GtkWidget *hseparator12;
  GtkWidget *hbox30;
  GtkWidget *radiobutton42;
  GtkObject *spinbutton59_adj;
  GtkWidget *spinbutton59;
  GtkWidget *label124;
  GtkObject *spinbutton60_adj;
  GtkWidget *spinbutton60;
  GtkWidget *label125;
  GtkWidget *label118;
  GtkWidget *dialog_action_area19;
  GtkWidget *cancelbutton16;
  GtkWidget *okbutton16;

  shrink_dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (shrink_dialog), 10);
  gtk_window_set_title (GTK_WINDOW (shrink_dialog), _("LiVES: - Shrink/Expand Selection"));
  gtk_window_set_position (GTK_WINDOW (shrink_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (shrink_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (shrink_dialog), 450, 400);
  gtk_window_set_transient_for(GTK_WINDOW(shrink_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(shrink_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(shrink_dialog),FALSE);
  }

  dialog_vbox19 = GTK_DIALOG (shrink_dialog)->vbox;
  gtk_widget_show (dialog_vbox19);

  vbox28 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox28);
  gtk_box_pack_start (GTK_BOX (dialog_vbox19), vbox28, TRUE, TRUE, 0);

  hbox29 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox29);
  gtk_box_pack_start (GTK_BOX (vbox28), hbox29, TRUE, TRUE, 0);

  label116 = gtk_label_new (_("Shrink/Expand Start frame to:         "));
  gtk_widget_show (label116);
  gtk_box_pack_start (GTK_BOX (hbox29), label116, FALSE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label116), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label116, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton58_adj = gtk_adjustment_new (mainw->fx1_val, 0, 100000, 1, 10, 100);
  spinbutton58 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton58_adj), 1, 2);
  gtk_widget_show (spinbutton58);
  gtk_box_pack_start (GTK_BOX (hbox29), spinbutton58, FALSE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton58), TRUE);

  label117 = gtk_label_new (_("  % of normal size          "));
  gtk_widget_show (label117);
  gtk_box_pack_start (GTK_BOX (hbox29), label117, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label117), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label117, GTK_STATE_NORMAL, &palette->normal_fore);

  hbox129 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox129);
  gtk_box_pack_start (GTK_BOX (vbox28), hbox129, TRUE, TRUE, 0);

  label1116 = gtk_label_new (_("Shrink/Expand End frame to:         "));
  gtk_widget_show (label1116);
  gtk_box_pack_start (GTK_BOX (hbox129), label1116, FALSE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label1116), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label1116, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton158_adj = gtk_adjustment_new (mainw->fx2_val, 0, 100000, 1, 10, 100);
  spinbutton158 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton158_adj), 1, 2);
  gtk_widget_show (spinbutton158);
  gtk_box_pack_start (GTK_BOX (hbox129), spinbutton158, FALSE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton158), TRUE);

  label1117 = gtk_label_new (_("  % of normal size          "));
  gtk_widget_show (label1117);
  gtk_box_pack_start (GTK_BOX (hbox129), label1117, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label1117), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label1117, GTK_STATE_NORMAL, &palette->normal_fore);

  frame9 = gtk_frame_new (NULL);
  gtk_widget_show (frame9);
  gtk_box_pack_start (GTK_BOX (vbox28), frame9, TRUE, TRUE, 0);
  gtk_widget_modify_bg (frame9, GTK_STATE_NORMAL, &palette->normal_back);

  vbox29 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox29);
  gtk_container_add (GTK_CONTAINER (frame9), vbox29);

  table12 = gtk_table_new (3, 3, FALSE);
  gtk_widget_show (table12);
  gtk_box_pack_start (GTK_BOX (vbox29), table12, TRUE, TRUE, 0);

  label119 = gtk_label_new ("");
  gtk_widget_show (label119);
  gtk_table_attach (GTK_TABLE (table12), label119, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label119), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label119), 0, 0.5);

  label120 = gtk_label_new ("");
  gtk_widget_show (label120);
  gtk_table_attach (GTK_TABLE (table12), label120, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label120), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label120), 0, 0.5);

  label121 = gtk_label_new ("");
  gtk_widget_show (label121);
  gtk_table_attach (GTK_TABLE (table12), label121, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label121), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label121), 0, 0.5);

  label122 = gtk_label_new ("");
  gtk_widget_show (label122);
  gtk_table_attach (GTK_TABLE (table12), label122, 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label122), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label122), 0, 0.5);

  radiobutton38 = gtk_radio_button_new_with_mnemonic (NULL, _("Left"));
  gtk_widget_show (radiobutton38);
  gtk_table_attach (GTK_TABLE (table12), radiobutton38, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton38), radiobutton38_group);
  radiobutton38_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton38));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton38), mainw->fx2_bool);

  radiobutton39 = gtk_radio_button_new_with_mnemonic (NULL, _("Right"));
  gtk_widget_show (radiobutton39);
  gtk_table_attach (GTK_TABLE (table12), radiobutton39, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton39), radiobutton38_group);
  radiobutton38_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton39));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton39), mainw->fx4_bool);

  radiobutton41 = gtk_radio_button_new_with_mnemonic (NULL, _("Bottom"));
  gtk_widget_show (radiobutton41);
  gtk_table_attach (GTK_TABLE (table12), radiobutton41, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton41), radiobutton38_group);
  radiobutton38_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton41));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton41), mainw->fx5_bool);

  radiobutton40 = gtk_radio_button_new_with_mnemonic (NULL, _("Centre"));
  gtk_widget_show (radiobutton40);
  gtk_table_attach (GTK_TABLE (table12), radiobutton40, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton40), radiobutton38_group);
  radiobutton38_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton40));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton40), mainw->fx3_bool);

  radiobutton37 = gtk_radio_button_new_with_mnemonic (NULL, _("Top   "));
  gtk_widget_show (radiobutton37);
  gtk_table_attach (GTK_TABLE (table12), radiobutton37, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton37), radiobutton38_group);
  radiobutton38_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton37));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton37), mainw->fx1_bool);

  hseparator12 = gtk_hseparator_new ();
  gtk_widget_show (hseparator12);
  gtk_box_pack_start (GTK_BOX (vbox29), hseparator12, TRUE, TRUE, 0);

  hbox30 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox30);
  gtk_box_pack_start (GTK_BOX (vbox29), hbox30, TRUE, TRUE, 0);

  radiobutton42 = gtk_radio_button_new_with_mnemonic (NULL, _("Other"));
  gtk_widget_show (radiobutton42);
  gtk_box_pack_start (GTK_BOX (hbox30), radiobutton42, TRUE, TRUE, 40);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton42), radiobutton38_group);
  radiobutton38_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton42));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton42), mainw->fx6_bool);

  spinbutton59_adj = gtk_adjustment_new (mainw->fx3_val, 0, cfile->hsize, 1, 10, 10);
  spinbutton59 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton59_adj), 1, 0);
  gtk_widget_show (spinbutton59);
  gtk_box_pack_start (GTK_BOX (hbox30), spinbutton59, FALSE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton59), TRUE);

  label124 = gtk_label_new (_("    X          "));
  gtk_widget_show (label124);
  gtk_box_pack_start (GTK_BOX (hbox30), label124, FALSE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label124), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label124, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton60_adj = gtk_adjustment_new (mainw->fx4_val, 0, cfile->vsize, 1, 10, 10);
  spinbutton60 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton60_adj), 1, 0);
  gtk_widget_show (spinbutton60);
  gtk_box_pack_start (GTK_BOX (hbox30), spinbutton60, FALSE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton60), TRUE);

  label125 = gtk_label_new (_("    Y          "));
  gtk_widget_show (label125);
  gtk_box_pack_start (GTK_BOX (hbox30), label125, FALSE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label125), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label125, GTK_STATE_NORMAL, &palette->normal_fore);

  label118 = gtk_label_new (_("Shrink/Expand Centre"));
  gtk_widget_show (label118);
  gtk_frame_set_label_widget (GTK_FRAME (frame9), label118);
  gtk_label_set_justify (GTK_LABEL (label118), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg(label118, GTK_STATE_NORMAL, &palette->normal_fore);

  dialog_action_area19 = GTK_DIALOG (shrink_dialog)->action_area;
  gtk_widget_show (dialog_action_area19);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area19), GTK_BUTTONBOX_END);

  cancelbutton16 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton16);
  gtk_dialog_add_action_widget (GTK_DIALOG (shrink_dialog), cancelbutton16, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton16, GTK_CAN_DEFAULT);

  okbutton16 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton16);
  gtk_dialog_add_action_widget (GTK_DIALOG (shrink_dialog), okbutton16, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton16, GTK_CAN_DEFAULT);

  g_signal_connect (GTK_OBJECT (radiobutton37), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (radiobutton38), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (2));
  g_signal_connect (GTK_OBJECT (radiobutton40), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (3));
  g_signal_connect (GTK_OBJECT (radiobutton39), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (4));
  g_signal_connect (GTK_OBJECT (radiobutton41), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (5));
  g_signal_connect (GTK_OBJECT (radiobutton42), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (6));

  g_signal_connect_after (GTK_OBJECT (spinbutton58), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (1));

  g_signal_connect_after (GTK_OBJECT (spinbutton158), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (2));

  g_signal_connect_after (GTK_OBJECT (spinbutton59), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (3));

  g_signal_connect_after (GTK_OBJECT (spinbutton60), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (4));

  g_signal_connect (GTK_OBJECT (cancelbutton16), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      NULL);
  g_signal_connect (GTK_OBJECT (okbutton16), "clicked",
                      G_CALLBACK (on_shrink_ok_clicked),
                      NULL);

  return shrink_dialog;
}



GtkWidget*
create_solar_dialog (void)
{
  GtkWidget *solar_dialog;
  GtkWidget *dialog_vbox9;
  GtkWidget *vbox15;
  GtkWidget *table5;
  GtkWidget *label46;
  GtkWidget *label47;
  GtkObject *spinbutton23_adj;
  GtkWidget *spinbutton23;
  GtkObject *spinbutton24_adj;
  GtkWidget *spinbutton24;
  GtkWidget *dialog_action_area9;
  GtkWidget *cancelbutton7;
  GtkWidget *okbutton6;

  solar_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (solar_dialog), _("LiVES: - Solarize"));
  gtk_window_set_position (GTK_WINDOW (solar_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (solar_dialog), TRUE);
  gtk_widget_modify_bg(solar_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  gtk_window_set_transient_for(GTK_WINDOW(solar_dialog),GTK_WINDOW(mainw->LiVES));

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(solar_dialog),FALSE);
  }

  gtk_container_set_border_width (GTK_CONTAINER (solar_dialog), 10);
  gtk_window_set_default_size (GTK_WINDOW (solar_dialog), 300, 200);

  dialog_vbox9 = GTK_DIALOG (solar_dialog)->vbox;
  gtk_widget_show (dialog_vbox9);

  vbox15 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox15);
  gtk_box_pack_start (GTK_BOX (dialog_vbox9), vbox15, TRUE, TRUE, 0);

  table5 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table5);
  gtk_box_pack_start (GTK_BOX (vbox15), table5, TRUE, TRUE, 12);

  label46 = gtk_label_new (_("    Solarization start"));
  gtk_widget_show (label46);
  gtk_table_attach (GTK_TABLE (table5), label46, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label46), 0, 0.5);
  gtk_widget_modify_fg(label46, GTK_STATE_NORMAL, &palette->normal_fore);

  label47 = gtk_label_new (_("    Solarization step"));
  gtk_widget_show (label47);
  gtk_table_attach (GTK_TABLE (table5), label47, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label47), 0, 0.5);
  gtk_widget_modify_fg(label47, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton23_adj = gtk_adjustment_new (65535, 0, 65535, 1, 100, 1000);
  spinbutton23 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton23_adj), 1, 0);
  gtk_widget_show (spinbutton23);
  gtk_table_attach (GTK_TABLE (table5), spinbutton23, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 42, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton23), TRUE);

  spinbutton24_adj = gtk_adjustment_new (-1000, -65535, 65535, 1, 10, 100);
  spinbutton24 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton24_adj), 1, 0);
  gtk_widget_show (spinbutton24);
  gtk_table_attach (GTK_TABLE (table5), spinbutton24, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 42, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton24), TRUE);

  dialog_action_area9 = GTK_DIALOG (solar_dialog)->action_area;
  gtk_widget_show (dialog_action_area9);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area9), GTK_BUTTONBOX_END);

  cancelbutton7 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton7);
  gtk_dialog_add_action_widget (GTK_DIALOG (solar_dialog), cancelbutton7, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton7, GTK_CAN_DEFAULT);

  okbutton6 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton6);
  gtk_dialog_add_action_widget (GTK_DIALOG (solar_dialog), okbutton6, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton6, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton6);

  g_signal_connect_after (GTK_OBJECT (spinbutton23), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (1));
  g_signal_connect_after (GTK_OBJECT (spinbutton24), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (2));
  g_signal_connect (GTK_OBJECT (cancelbutton7), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      NULL);
  g_signal_connect (GTK_OBJECT (okbutton6), "clicked",
                      G_CALLBACK (on_solarize_ok_clicked),
                      NULL);

   return solar_dialog;
}




GtkWidget*
create_opensel_dialog ()
{
  GtkWidget *opensel_dialog;
  GtkWidget *dialog_vbox9;
  GtkWidget *vbox15;
  GtkWidget *table5;
  GtkWidget *label46;
  GtkWidget *label47;
  GtkObject *spinbutton23_adj;
  GtkWidget *spinbutton23;
  GtkObject *spinbutton24_adj;
  GtkWidget *spinbutton24;
  GtkWidget *dialog_action_area9;
  GtkWidget *cancelbutton7;
  GtkWidget *okbutton6;

  opensel_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (opensel_dialog), _("LiVES: - Open Selection"));
  gtk_window_set_position (GTK_WINDOW (opensel_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (opensel_dialog), TRUE);
  gtk_widget_modify_bg(opensel_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  gtk_window_set_transient_for(GTK_WINDOW(opensel_dialog),GTK_WINDOW(mainw->LiVES));
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(opensel_dialog),FALSE);
  }

  gtk_container_set_border_width (GTK_CONTAINER (opensel_dialog), 10);
  gtk_window_set_default_size (GTK_WINDOW (opensel_dialog), 300, 200);

  dialog_vbox9 = GTK_DIALOG (opensel_dialog)->vbox;
  gtk_widget_show (dialog_vbox9);

  vbox15 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox15);
  gtk_box_pack_start (GTK_BOX (dialog_vbox9), vbox15, TRUE, TRUE, 0);

  table5 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table5);
  gtk_box_pack_start (GTK_BOX (vbox15), table5, TRUE, TRUE, 12);

  gtk_table_set_row_spacings (GTK_TABLE (table5), 20);
  label46 = gtk_label_new ("    Selection start time (sec)");
  gtk_widget_show (label46);
  gtk_table_attach (GTK_TABLE (table5), label46, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label46), 0, 0.5);
  gtk_widget_modify_fg(label46, GTK_STATE_NORMAL, &palette->normal_fore);

  label47 = gtk_label_new ("    Number of frames to open");
  gtk_widget_show (label47);
  gtk_table_attach (GTK_TABLE (table5), label47, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label47), 0, 0.5);
  gtk_widget_modify_fg(label47, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton23_adj = gtk_adjustment_new (0, 0, 65535, 1, 10, 100);
  spinbutton23 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton23_adj), 1, 2);
  gtk_widget_show (spinbutton23);
  gtk_table_attach (GTK_TABLE (table5), spinbutton23, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 42, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton23), TRUE);

  spinbutton24_adj = gtk_adjustment_new (1000, 1, 100000, 1, 10, 100);
  spinbutton24 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton24_adj), 1, 0);
  gtk_widget_show (spinbutton24);
  gtk_table_attach (GTK_TABLE (table5), spinbutton24, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 42, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton24), TRUE);


  dialog_action_area9 = GTK_DIALOG (opensel_dialog)->action_area;
  gtk_widget_show (dialog_action_area9);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area9), GTK_BUTTONBOX_END);

  cancelbutton7 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton7);
  gtk_dialog_add_action_widget (GTK_DIALOG (opensel_dialog), cancelbutton7, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton7, GTK_CAN_DEFAULT);

  okbutton6 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton6);
  gtk_dialog_add_action_widget (GTK_DIALOG (opensel_dialog), okbutton6, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton6, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(okbutton6);

  widget_add_preview (GTK_BOX (dialog_vbox9), GTK_BOX (dialog_vbox9), 3);

  g_signal_connect_after (GTK_OBJECT (spinbutton23), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (1));
  g_signal_connect_after (GTK_OBJECT (spinbutton24), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (2));
  g_signal_connect (GTK_OBJECT (cancelbutton7), "clicked",
                      G_CALLBACK (on_cancel_opensel_clicked),
                      NULL);
  g_signal_connect (GTK_OBJECT (okbutton6), "clicked",
                      G_CALLBACK (on_opensel_range_ok_clicked),
                      NULL);

   return opensel_dialog;
}





_colorizew *
create_colorize_dialog (gint type)
{
  // type 1 == colorize
  // type 2 == colour filter


  GtkWidget *dialog_vbox10;
  GtkWidget *table6;
  GtkWidget *label48;
  GtkWidget *label49;
  GtkWidget *label50;
  GtkWidget *label51;
  GtkObject *spinbutton27_adj;
  GtkObject *spinbutton29_adj;
  GtkObject *spinbutton30_adj;
  GtkObject *spinbutton28_adj;
  GtkObject *spinbutton31_adj;
  GtkWidget *spinbutton31;
  GtkObject *spinbutton32_adj;
  GtkWidget *spinbutton32;
  GtkObject *spinbutton25_adj;
  GtkObject *spinbutton26_adj;
  GtkWidget *button12;
  GtkWidget *button11;
  GtkWidget *hseparator5;
  GtkWidget *hseparator6;
  GtkWidget *hseparator7;
  GtkWidget *label52;
  GtkWidget *label53;
  GtkWidget *label54;
  GtkWidget *label55;
  GtkWidget *dialog_action_area10;
  GtkWidget *cancelbutton8;
  GtkWidget *okbutton7;
  GtkWidget *checkbutton8;

  _colorizew *colorizew=(_colorizew*)(g_malloc(sizeof(_colorizew)));

  colorizew->colorize_dialog = gtk_dialog_new ();
  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (colorizew->colorize_dialog), _("LiVES: - Colorize"));
  }
  else {
    gtk_window_set_title (GTK_WINDOW (colorizew->colorize_dialog), _("LiVES: - Colour Filter"));
  }
  gtk_window_set_position (GTK_WINDOW (colorizew->colorize_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (colorizew->colorize_dialog), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(colorizew->colorize_dialog),GTK_WINDOW(mainw->LiVES));

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(colorizew->colorize_dialog),FALSE);
  }
  gtk_widget_modify_bg(colorizew->colorize_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_container_set_border_width (GTK_CONTAINER (colorizew->colorize_dialog), 10);
  gtk_window_set_default_size (GTK_WINDOW (colorizew->colorize_dialog), 300, 200);

  dialog_vbox10 = GTK_DIALOG (colorizew->colorize_dialog)->vbox;
  gtk_widget_show (dialog_vbox10);

  table6 = gtk_table_new (7, 6, FALSE);
  gtk_widget_show (table6);
  gtk_box_pack_start (GTK_BOX (dialog_vbox10), table6, TRUE, TRUE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table6), 50);
  gtk_table_set_row_spacings (GTK_TABLE (table6), 20);

  label48 = gtk_label_new (_("    Colour start"));
  gtk_widget_show (label48);
  gtk_table_attach (GTK_TABLE (table6), label48, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label48), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label48), 0, 0.5);
  gtk_widget_modify_fg (label48, GTK_STATE_NORMAL, &palette->normal_fore);

  label49 = gtk_label_new (_("    Colour end"));
  gtk_widget_show (label49);
  gtk_table_attach (GTK_TABLE (table6), label49, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label49), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label49), 0, 0.5);
  gtk_widget_modify_fg (label49, GTK_STATE_NORMAL, &palette->normal_fore);

  label50 = gtk_label_new (_("    % start"));
  gtk_widget_show (label50);
  gtk_table_attach (GTK_TABLE (table6), label50, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label50), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label50), 0, 0.5);
  gtk_widget_modify_fg (label50, GTK_STATE_NORMAL, &palette->normal_fore);

  label51 = gtk_label_new (_("    % end"));
  gtk_widget_show (label51);
  gtk_table_attach (GTK_TABLE (table6), label51, 0, 1, 6, 7,
                    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label51), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (label51), 0, 0.5);
  gtk_widget_modify_fg (label51, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton27_adj = gtk_adjustment_new (255, 0, 255, 1, 10, 10);
  colorizew->spinbutton27 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton27_adj), 1, 0);
  gtk_widget_show (colorizew->spinbutton27);
  gtk_table_attach (GTK_TABLE (table6), colorizew->spinbutton27, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton29_adj = gtk_adjustment_new (0, 0, 255, 1, 10, 10);
  colorizew->spinbutton29 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton29_adj), 1, 0);
  gtk_widget_show (colorizew->spinbutton29);
  gtk_table_attach (GTK_TABLE (table6), colorizew->spinbutton29, 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton30_adj = gtk_adjustment_new (0, 0, 255, 1, 10, 10);
  colorizew->spinbutton30 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton30_adj), 1, 0);
  gtk_widget_show (colorizew->spinbutton30);
  gtk_table_attach (GTK_TABLE (table6), colorizew->spinbutton30, 3, 4, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton28_adj = gtk_adjustment_new (0, 0, 255, 1, 10, 10);
  colorizew->spinbutton28 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton28_adj), 1, 0);
  gtk_widget_show (colorizew->spinbutton28);
  gtk_table_attach (GTK_TABLE (table6), colorizew->spinbutton28, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);

  spinbutton31_adj = gtk_adjustment_new (10, 0, 100, 10, 1, 100);
  spinbutton31 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton31_adj), 1, 0);
  gtk_widget_show (spinbutton31);
  gtk_table_attach (GTK_TABLE (table6), spinbutton31, 2, 3, 5, 6,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);

  spinbutton32_adj = gtk_adjustment_new (50, 0, 100, 10, 1, 100);
  spinbutton32 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton32_adj), 1, 0);
  gtk_widget_show (spinbutton32);
  gtk_table_attach (GTK_TABLE (table6), spinbutton32, 2, 3, 6, 7,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);

  spinbutton25_adj = gtk_adjustment_new (255, 0, 255, 1, 10, 10);
  colorizew->spinbutton25 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton25_adj), 1, 0);
  gtk_widget_show (colorizew->spinbutton25);
  gtk_table_attach (GTK_TABLE (table6), colorizew->spinbutton25, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);

  spinbutton26_adj = gtk_adjustment_new (255, 0, 255, 1, 10, 10);
  colorizew->spinbutton26 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton26_adj), 1, 0);
  gtk_widget_show (colorizew->spinbutton26);
  gtk_table_attach (GTK_TABLE (table6), colorizew->spinbutton26, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  button12 = gtk_button_new_from_stock ("gtk-select-color");
  gtk_widget_show (button12);
  gtk_table_attach (GTK_TABLE (table6), button12, 4, 5, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  button11 = gtk_button_new_from_stock ("gtk-select-color");
  gtk_widget_show (button11);
  gtk_table_attach (GTK_TABLE (table6), button11, 4, 5, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  checkbutton8 = gtk_check_button_new_with_mnemonic (_("Random colour each frame"));
  gtk_widget_show (checkbutton8);
  gtk_table_attach (GTK_TABLE (table6), checkbutton8, 5, 6, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hseparator5 = gtk_hseparator_new ();
  gtk_widget_show (hseparator5);
  gtk_table_attach (GTK_TABLE (table6), hseparator5, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  hseparator6 = gtk_hseparator_new ();
  gtk_widget_show (hseparator6);
  gtk_table_attach (GTK_TABLE (table6), hseparator6, 2, 3, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  hseparator7 = gtk_hseparator_new ();
  gtk_widget_show (hseparator7);
  gtk_table_attach (GTK_TABLE (table6), hseparator7, 3, 4, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label52 = gtk_label_new (_("    Red"));
  gtk_widget_show (label52);
  gtk_table_attach (GTK_TABLE (table6), label52, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label52), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label52), 0, 0.5);
  gtk_widget_modify_fg (label52, GTK_STATE_NORMAL, &palette->normal_fore);

  label53 = gtk_label_new (_(" Green"));
  gtk_widget_show (label53);
  gtk_table_attach (GTK_TABLE (table6), label53, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label53), 0, 0.5);
  gtk_widget_modify_fg (label53, GTK_STATE_NORMAL, &palette->normal_fore);

  label54 = gtk_label_new (_("   Blue"));
  gtk_widget_show (label54);
  gtk_table_attach (GTK_TABLE (table6), label54, 3, 4, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label54), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label54), 0, 0.5);
  gtk_widget_modify_fg (label54, GTK_STATE_NORMAL, &palette->normal_fore);

  label55 = gtk_label_new (_("Value"));
  gtk_widget_show (label55);
  gtk_table_attach (GTK_TABLE (table6), label55, 2, 3, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label55), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label55), 0, 0.5);
  gtk_widget_modify_fg (label55, GTK_STATE_NORMAL, &palette->normal_fore);

  dialog_action_area10 = GTK_DIALOG (colorizew->colorize_dialog)->action_area;
  gtk_widget_show (dialog_action_area10);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area10), GTK_BUTTONBOX_END);

  cancelbutton8 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton8);
  gtk_dialog_add_action_widget (GTK_DIALOG (colorizew->colorize_dialog), cancelbutton8, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton8, GTK_CAN_DEFAULT);

  okbutton7 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton7);
  gtk_dialog_add_action_widget (GTK_DIALOG (colorizew->colorize_dialog), okbutton7, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton7, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton7);


  g_signal_connect_after (GTK_OBJECT (colorizew->spinbutton25), "value_changed",
                            G_CALLBACK (on_spin_start_value_changed),
                            GINT_TO_POINTER (1));
  g_signal_connect_after (GTK_OBJECT (colorizew->spinbutton26), "value_changed",
                            G_CALLBACK (on_spin_start_value_changed),
                            GINT_TO_POINTER (2));
  g_signal_connect_after (GTK_OBJECT (colorizew->spinbutton27), "value_changed",
                            G_CALLBACK (on_spin_start_value_changed),
                            GINT_TO_POINTER (3));
  g_signal_connect_after (GTK_OBJECT (colorizew->spinbutton28), "value_changed",
                            G_CALLBACK (on_spin_end_value_changed),
                            GINT_TO_POINTER (1));
  g_signal_connect_after (GTK_OBJECT (colorizew->spinbutton29), "value_changed",
                            G_CALLBACK (on_spin_end_value_changed),
                            GINT_TO_POINTER (2));
  g_signal_connect_after (GTK_OBJECT (colorizew->spinbutton30), "value_changed",
                            G_CALLBACK (on_spin_end_value_changed),
                            GINT_TO_POINTER (3));
  g_signal_connect_after (GTK_OBJECT (spinbutton31), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (1));
  g_signal_connect_after (GTK_OBJECT (spinbutton32), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (2));
  g_signal_connect (GTK_OBJECT (checkbutton8), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (button12), "clicked",
		    G_CALLBACK (on_colorize_colour1_clicked),
		    colorizew);
  g_signal_connect (GTK_OBJECT (button11), "clicked",
		    G_CALLBACK (on_colorize_colour2_clicked),
		    colorizew);
  g_signal_connect (GTK_OBJECT (cancelbutton8), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    colorizew);
  g_signal_connect (GTK_OBJECT (okbutton7), "clicked",
		    G_CALLBACK (on_colorize_ok_clicked),
		    GINT_TO_POINTER (type));
  g_signal_connect (GTK_OBJECT (colorizew->colorize_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);
  
  return colorizew;
}


_resaudw *
create_resaudw (gshort type)
{
  // type 1 == resample
  // type 2 == insert silence

  GtkWidget *dialog_vbox15;
  GtkWidget *vbox21;
  GtkWidget *frame6;
  GtkWidget *hbox24;
  GtkWidget *label94;
  GtkWidget *combo_entry2;
  GtkWidget *label95;
  GtkWidget *combo_entry3;
  GtkWidget *label96;
  GtkWidget *combo_entry1;
  GtkWidget *vseparator3;
  GtkWidget *vbox22;
  GtkWidget *radiobutton26;
  GSList *radiobutton26_group = NULL;
  GtkWidget *radiobutton27;
  GtkWidget *vseparator4;
  GtkWidget *vbox23;
  GtkWidget *radiobutton28;
  GSList *radiobutton28_group = NULL;
  GtkWidget *radiobutton29;
  GtkWidget *label92;
  GtkWidget *frame7;
  GtkWidget *hbox25;
  GtkWidget *label98;
  GtkWidget *combo4;
  GtkWidget *label99;
  GtkWidget *combo5;
  GtkWidget *label100;
  GtkWidget *combo6;
  GtkWidget *vseparator5;
  GtkWidget *vbox24;
  GSList *radiobutton30_group = NULL;
  GtkWidget *vseparator6;
  GtkWidget *vbox25;
  GSList *radiobutton32_group = NULL;
  GtkWidget *label93;
  GtkWidget *dialog_action_area15;
  GtkWidget *cancelbutton13;
  GtkWidget *okbutton12;
  GList *channels = NULL;
  GList *sampsize = NULL;
  GList *rate = NULL;
  gchar *tmp;

  _resaudw *resaudw=(_resaudw*)(g_malloc(sizeof(_resaudw)));

  channels = g_list_append (channels, "1");
  channels = g_list_append (channels, "2");

  sampsize = g_list_append (sampsize, "8");
  sampsize = g_list_append (sampsize, "16");

  rate = g_list_append (rate, "8000");
  rate = g_list_append (rate, "11025");
  rate = g_list_append (rate, "22050");
  rate = g_list_append (rate, "32000");
  rate = g_list_append (rate, "44100");
  rate = g_list_append (rate, "48000");
  rate = g_list_append (rate, "88200");
  rate = g_list_append (rate, "96000");
  rate = g_list_append (rate, "128000");


  resaudw->dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (resaudw->dialog), 10);
  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (resaudw->dialog), _("LiVES: - Resample Audio"));
  }
  if (type==2) {
    gtk_window_set_title (GTK_WINDOW (resaudw->dialog), _("LiVES: - Insert Silence"));
  }
  gtk_window_set_position (GTK_WINDOW (resaudw->dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (resaudw->dialog), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(resaudw->dialog),GTK_WINDOW(mainw->LiVES));

  dialog_vbox15 = GTK_DIALOG (resaudw->dialog)->vbox;
  gtk_widget_show (dialog_vbox15);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_bg (resaudw->dialog, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (resaudw->dialog, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }


  vbox21 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox21);
  gtk_box_pack_start (GTK_BOX (dialog_vbox15), vbox21, TRUE, TRUE, 0);

  frame6 = gtk_frame_new (NULL);
 
  if (type==1) {

    gtk_widget_show (frame6);
    gtk_box_pack_start (GTK_BOX (vbox21), frame6, TRUE, TRUE, 0);
    
    if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
      gtk_widget_modify_bg (frame6, GTK_STATE_NORMAL, &palette->normal_back);
    }
    else {
      gtk_widget_modify_bg (frame6, GTK_STATE_NORMAL, &palette->menu_and_bars);
    }
    
    hbox24 = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox24);
    gtk_container_add (GTK_CONTAINER (frame6), hbox24);
    gtk_container_set_border_width (GTK_CONTAINER (hbox24), 10);
    
    label94 = gtk_label_new (_("Rate (Hz) "));
    gtk_widget_show (label94);
    gtk_box_pack_start (GTK_BOX (hbox24), label94, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label94), GTK_JUSTIFY_LEFT);
    
    combo_entry2 = gtk_entry_new();
    gtk_widget_show (combo_entry2);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry2, TRUE, TRUE, 1);

    gtk_editable_set_editable (GTK_EDITABLE (combo_entry2), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry2), 8);
    
    tmp=g_strdup_printf ("%d",(gint)mainw->fx1_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry2),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry2, GTK_CAN_FOCUS);
    
    label95 = gtk_label_new (_("         Channels "));
    gtk_widget_show (label95);
    gtk_box_pack_start (GTK_BOX (hbox24), label95, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label95), GTK_JUSTIFY_LEFT);
    
    combo_entry3 = gtk_entry_new();
    gtk_widget_show (combo_entry3);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry3, TRUE, TRUE, 1);
    
    gtk_editable_set_editable (GTK_EDITABLE (combo_entry3), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry3), 3);
    
    tmp=g_strdup_printf ("%d",(gint)mainw->fx2_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry3),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry3, GTK_CAN_FOCUS);
    
    label96 = gtk_label_new (_("        Sample Size "));
    gtk_widget_show (label96);
    gtk_box_pack_start (GTK_BOX (hbox24), label96, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label96), GTK_JUSTIFY_LEFT);

    combo_entry1 = gtk_entry_new();
    gtk_widget_show (combo_entry1);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry1, TRUE, TRUE, 1);

    gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry1), 3);

    tmp=g_strdup_printf ("%d",(gint)mainw->fx3_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry1),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry1, GTK_CAN_FOCUS);

    vseparator3 = gtk_vseparator_new ();
    gtk_widget_show (vseparator3);
    gtk_box_pack_start (GTK_BOX (hbox24), vseparator3, FALSE, FALSE, 12);

    vbox22 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox22);
    gtk_box_pack_start (GTK_BOX (hbox24), vbox22, TRUE, TRUE, 0);

    radiobutton26 = gtk_radio_button_new_with_mnemonic (NULL, _("Signed"));
    gtk_widget_show (radiobutton26);
    gtk_box_pack_start (GTK_BOX (vbox22), radiobutton26, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton26), radiobutton26_group);
    radiobutton26_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton26));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton26), TRUE);

    radiobutton27 = gtk_radio_button_new_with_mnemonic (NULL, _("Unsigned"));
    gtk_widget_show (radiobutton27);
    gtk_box_pack_start (GTK_BOX (vbox22), radiobutton27, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton27), radiobutton26_group);
    radiobutton26_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton27));

    if ((gint)mainw->fx4_val&AFORM_UNSIGNED) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton27), TRUE);
    }
    else {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton26), TRUE);
    }

    gtk_widget_set_sensitive (radiobutton26, FALSE);
    gtk_widget_set_sensitive (radiobutton27, FALSE);

    vseparator4 = gtk_vseparator_new ();
    gtk_widget_show (vseparator4);
    gtk_box_pack_start (GTK_BOX (hbox24), vseparator4, FALSE, FALSE, 12);
    
    vbox23 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox23);
    gtk_box_pack_start (GTK_BOX (hbox24), vbox23, TRUE, TRUE, 0);

    radiobutton28 = gtk_radio_button_new_with_mnemonic (NULL, _("Little Endian"));
    gtk_widget_show (radiobutton28);
    gtk_box_pack_start (GTK_BOX (vbox23), radiobutton28, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton28), radiobutton28_group);
    radiobutton28_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton28));
    
    radiobutton29 = gtk_radio_button_new_with_mnemonic (NULL, _("Big Endian"));
    gtk_widget_show (radiobutton29);
    gtk_box_pack_start (GTK_BOX (vbox23), radiobutton29, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton29), radiobutton28_group);
    radiobutton28_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton29));
    
    if ((gint)mainw->fx4_val&AFORM_BIG_ENDIAN) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton29), TRUE);
    }
    else {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton28), TRUE);
    }

    gtk_widget_set_sensitive (radiobutton29, FALSE);
    gtk_widget_set_sensitive (radiobutton28, FALSE);

    label92 = gtk_label_new (_("Current"));
    
    gtk_widget_show (label92);
    gtk_frame_set_label_widget (GTK_FRAME (frame6), label92);
    gtk_label_set_justify (GTK_LABEL (label92), GTK_JUSTIFY_LEFT);
    
  }

  frame7 = gtk_frame_new (NULL);
  gtk_widget_show (frame7);
  gtk_box_pack_start (GTK_BOX (vbox21), frame7, TRUE, TRUE, 0);

  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_bg (frame7, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (frame7, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  hbox25 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox25);
  gtk_container_add (GTK_CONTAINER (frame7), hbox25);
  gtk_container_set_border_width (GTK_CONTAINER (hbox25), 10);

  label98 = gtk_label_new (_("Rate (Hz) "));
  gtk_widget_show (label98);
  gtk_box_pack_start (GTK_BOX (hbox25), label98, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label98), GTK_JUSTIFY_LEFT);

  combo4 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo4), rate);
  gtk_widget_show (combo4);
  gtk_box_pack_start (GTK_BOX (hbox25), combo4, TRUE, TRUE, 1);

  resaudw->entry_arate = GTK_COMBO (combo4)->entry;
  gtk_widget_show (resaudw->entry_arate);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_arate), 8);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx1_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_arate),tmp);
  g_free (tmp);

  label99 = gtk_label_new (_("    Channels "));
  gtk_widget_show (label99);
  gtk_box_pack_start (GTK_BOX (hbox25), label99, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label99), GTK_JUSTIFY_LEFT);

  combo5 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo5), channels);
  gtk_widget_show (combo5);
  gtk_box_pack_start (GTK_BOX (hbox25), combo5, FALSE, FALSE, 0);

  resaudw->entry_achans = GTK_COMBO (combo5)->entry;
  gtk_widget_show (resaudw->entry_achans);
  gtk_editable_set_editable (GTK_EDITABLE (resaudw->entry_achans), FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_achans), 3);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx2_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_achans),tmp);
  g_free (tmp);

  label100 = gtk_label_new (_("    Sample Size "));
  gtk_widget_show (label100);
  gtk_box_pack_start (GTK_BOX (hbox25), label100, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label100), GTK_JUSTIFY_LEFT);

  combo6 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo6), sampsize);
  gtk_widget_show (combo6);
  gtk_box_pack_start (GTK_BOX (hbox25), combo6, TRUE, TRUE, 0);

  resaudw->entry_asamps = GTK_COMBO (combo6)->entry;
  gtk_widget_show (resaudw->entry_asamps);
  gtk_entry_set_max_length (GTK_ENTRY (resaudw->entry_asamps), 2);
  gtk_editable_set_editable (GTK_EDITABLE (resaudw->entry_asamps), FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_asamps), 3);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx3_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_asamps),tmp);
  g_free (tmp);

  vseparator5 = gtk_vseparator_new ();
  gtk_widget_show (vseparator5);
  gtk_box_pack_start (GTK_BOX (hbox25), vseparator5, FALSE, FALSE, 12);

  vbox24 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox24);
  gtk_box_pack_start (GTK_BOX (hbox25), vbox24, TRUE, TRUE, 0);

  resaudw->rb_signed = gtk_radio_button_new_with_mnemonic (NULL, _("Signed"));
  gtk_widget_show (resaudw->rb_signed);
  gtk_box_pack_start (GTK_BOX (vbox24), resaudw->rb_signed, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_signed), radiobutton30_group);
  radiobutton30_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_signed));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);

  resaudw->rb_unsigned = gtk_radio_button_new_with_mnemonic (NULL, _("Unsigned"));
  gtk_widget_show (resaudw->rb_unsigned);
  gtk_box_pack_start (GTK_BOX (vbox24), resaudw->rb_unsigned, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_unsigned), radiobutton30_group);
  radiobutton30_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_unsigned));

  if ((gint)mainw->fx4_val&AFORM_UNSIGNED) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_unsigned), TRUE);
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);
  }

  if (cfile->asampsize==8) {
    gtk_widget_set_sensitive (resaudw->rb_signed, FALSE);
    gtk_widget_set_sensitive (resaudw->rb_unsigned, FALSE);
  }

  vseparator6 = gtk_vseparator_new ();
  gtk_widget_show (vseparator6);
  gtk_box_pack_start (GTK_BOX (hbox25), vseparator6, FALSE, FALSE, 12);

  vbox25 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox25);
  gtk_box_pack_start (GTK_BOX (hbox25), vbox25, TRUE, TRUE, 0);

  resaudw->rb_littleend = gtk_radio_button_new_with_mnemonic (NULL, _("Little Endian"));
  gtk_widget_show (resaudw->rb_littleend);
  gtk_box_pack_start (GTK_BOX (vbox25), resaudw->rb_littleend, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_littleend), radiobutton32_group);
  radiobutton32_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_littleend));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_littleend), TRUE);

  resaudw->rb_bigend = gtk_radio_button_new_with_mnemonic (NULL, _("Big Endian"));
  gtk_widget_show (resaudw->rb_bigend);
  gtk_box_pack_start (GTK_BOX (vbox25), resaudw->rb_bigend, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_bigend), radiobutton32_group);
  radiobutton32_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_bigend));

  if ((gint)mainw->fx4_val&AFORM_BIG_ENDIAN) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_bigend), TRUE);
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_littleend), TRUE);
  }

  if (cfile->asampsize==8) {
    gtk_widget_set_sensitive (resaudw->rb_littleend, FALSE);
    gtk_widget_set_sensitive (resaudw->rb_bigend, FALSE);
    }

  label93 = gtk_label_new (_("New"));
  gtk_widget_show (label93);
  gtk_frame_set_label_widget (GTK_FRAME (frame7), label93);
  gtk_label_set_justify (GTK_LABEL (label93), GTK_JUSTIFY_LEFT);

  dialog_action_area15 = GTK_DIALOG (resaudw->dialog)->action_area;
  gtk_widget_show (dialog_action_area15);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area15), GTK_BUTTONBOX_END);

  cancelbutton13 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton13);
  gtk_dialog_add_action_widget (GTK_DIALOG (resaudw->dialog), cancelbutton13, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton13, GTK_CAN_DEFAULT);

  okbutton12 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton12);
  gtk_dialog_add_action_widget (GTK_DIALOG (resaudw->dialog), okbutton12, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton12, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton12);

  g_signal_connect (GTK_OBJECT(resaudw->entry_asamps), "changed",
		    G_CALLBACK (on_resaudw_asamps_changed),
		    NULL);

  g_signal_connect (GTK_OBJECT (cancelbutton13), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    resaudw);

  if (type==1) {
    g_signal_connect (GTK_OBJECT (okbutton12), "clicked",
		      G_CALLBACK (on_resaudio_ok_clicked),
		      NULL);
  }
  if (type==2) {
    g_signal_connect (GTK_OBJECT (okbutton12), "clicked",
		      G_CALLBACK (on_ins_silence_details_clicked),
		      NULL);
  }

  g_signal_connect (GTK_OBJECT (resaudw->dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  g_list_free (channels);
  g_list_free (sampsize);
  g_list_free (rate);

  return resaudw;
}



_prefsw *
create_prefs_dialog (void)
{
  GtkWidget *dialog_vbox8;
  GtkWidget *vbox7;
  GtkWidget *ins_resample;
  GtkWidget *hbox6;
  GtkWidget *hbox100;
  GtkObject *spinbutton_warn_fsize_adj;
  GtkObject *spinbutton_bwidth_adj;
  GtkWidget *label28;
  GtkWidget *hseparator2;
  GtkWidget *hbox8;
  GtkWidget *hseparator3;
  GtkWidget *hbox7;
  GtkWidget *pref_gui;
  GtkWidget *vbox9;
  GtkWidget *vbox99;
  GtkWidget *vbox69;
  GtkWidget *vbox98;
  GtkWidget *frame4;
  GtkWidget *hbox9;
  GtkWidget *hbox99;
  GtkWidget *hbox19;
  GtkWidget *label33;
  GtkWidget *label34;
  GtkWidget *label134;
  GtkWidget *label31;
  GtkWidget *frame5;
  GtkWidget *hbox10;
  GtkWidget *label35;
  GtkWidget *label36;
  GtkWidget *label32;
  GtkWidget *label25;
  GtkWidget *label84;
  GtkWidget *label86;
  GtkWidget *label87;
  GtkWidget *label97;
  GtkWidget *label88;
  GtkWidget *vbox18;
  GtkWidget *vbox12;
  GtkWidget *hbox11;
  GtkWidget *hbox94;
  GtkWidget *hbox101;
  GtkWidget *label37;
  GtkWidget *vbox13;
  GtkWidget *label45;
  GtkWidget *hseparator8;
  GtkWidget *hbox14;
  GtkWidget *label56;
  GtkWidget *label94;
  GtkWidget *hbox93;
  GtkWidget *hseparator4;
  GtkWidget *hbox12;
  GtkWidget *vbox15;
  GtkWidget *label38;
  GtkWidget *label26;
  GtkWidget *table4;
  GtkWidget *label39;
  GtkWidget *label99;
  GtkWidget *label98;
  GtkWidget *label100;
  GtkWidget *label40;
  GtkWidget *label41;
  GtkWidget *label42;
  GtkWidget *label95;
  GtkWidget *label52;
  GtkWidget *label43;
  GtkWidget *label27;
  GtkWidget *vbox14;
  GtkWidget *hbox13;
  GtkWidget *label44;
  GtkObject *spinbutton_def_fps_adj;
  GtkWidget *label29;
  GtkWidget *dialog_action_area8;
  GtkWidget *cancelbutton6;
  GtkWidget *okbutton5;
  GtkWidget *applybutton;
  GtkWidget *dirbutton1;
  GtkWidget *dirbutton2;
  GtkWidget *dirbutton3;
  GtkWidget *dirbutton4;
  GtkWidget *dirbutton5;
  GtkWidget *dirbutton6;
  GtkWidget *dirimage1;
  GtkWidget *dirimage2;
  GtkWidget *dirimage3;
  GtkWidget *dirimage4;
  GtkWidget *dirimage5;
  GtkWidget *dirimage6;
  GtkWidget *hseparator13;
  GtkWidget *hbox31;
  GtkWidget *label126;
  GtkWidget *pp_combo;
  GtkWidget *combo_entry4;
  GtkWidget *label128;
  GtkWidget *entry20;
  GtkWidget *label129;
  GtkWidget *entry21;

  GtkTooltips *tooltips;

  GSList *rb_group = NULL;
  GSList *rb_group2 = NULL;

  GList *themes = NULL;
  GList *ofmt = NULL;
  GList *ofmt_all = NULL;
  GList *vidp = NULL;
  GList *audp = NULL;
  GList *encoders = NULL;
  GList *vid_playback_plugins = NULL;
  
  gchar *tmp;

  int i;

  _prefsw *prefsw=(_prefsw*)(g_malloc(sizeof(_prefsw)));

  tooltips = gtk_tooltips_new ();

  prefsw->prefs_dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->prefs_dialog), 10);
  gtk_window_set_title (GTK_WINDOW (prefsw->prefs_dialog), _("LiVES: - Preferences"));
  gtk_window_set_position (GTK_WINDOW (prefsw->prefs_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (prefsw->prefs_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (prefsw->prefs_dialog), 660, 440);
  gtk_window_set_transient_for(GTK_WINDOW(prefsw->prefs_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(prefsw->prefs_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(prefsw->prefs_dialog),FALSE);
  }

  dialog_vbox8 = GTK_DIALOG (prefsw->prefs_dialog)->vbox;
  gtk_widget_show (dialog_vbox8);

  prefsw->prefs_notebook = gtk_notebook_new ();
  gtk_widget_show (prefsw->prefs_notebook);
  gtk_box_pack_start (GTK_BOX (dialog_vbox8), prefsw->prefs_notebook, TRUE, TRUE, 0);
  gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (prefsw->prefs_notebook), 10);
  gtk_notebook_popup_enable(GTK_NOTEBOOK(prefsw->prefs_notebook));

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox7);
  gtk_container_set_border_width (GTK_CONTAINER (vbox7), 20);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox6, TRUE, TRUE, 0);

  label28 = gtk_label_new (_("Background colour (for Blank Background and effects)"));
  gtk_widget_show (label28);
  gtk_box_pack_start (GTK_BOX (hbox6), label28, FALSE, FALSE, 16);
  gtk_label_set_justify (GTK_LABEL (label28), GTK_JUSTIFY_LEFT);

  prefsw->bgcolour_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->bgcolour_entry),255);

  gtk_entry_set_text(GTK_ENTRY(prefsw->bgcolour_entry),prefs->bgcolour);

  gtk_widget_show (prefsw->bgcolour_entry);
  gtk_box_pack_start (GTK_BOX (hbox6), prefsw->bgcolour_entry, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, prefsw->bgcolour_entry, _("What is your favourite colour ?"), NULL);

  prefsw->show_tool = gtk_check_button_new_with_mnemonic (_("Show toolbar when background is blanked"));
  gtk_widget_show (prefsw->show_tool);
  gtk_box_pack_start (GTK_BOX (vbox7), prefsw->show_tool, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->show_tool), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->show_tool), future_prefs->show_tool);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox7), hseparator2, FALSE, TRUE, 0);

  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox8, TRUE, TRUE, 0);

  prefsw->fs_max_check = gtk_check_button_new_with_mnemonic (_("Open file selection maximised"));
  gtk_widget_show (prefsw->fs_max_check);
  gtk_box_pack_start (GTK_BOX (hbox8), prefsw->fs_max_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->fs_max_check), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->fs_max_check), prefs->fileselmax);

  prefsw->recent_check = gtk_check_button_new_with_mnemonic (_("Show recent files in the File menu"));
  gtk_widget_show (prefsw->recent_check);
  gtk_box_pack_start (GTK_BOX (hbox8), prefsw->recent_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->recent_check), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->recent_check), prefs->show_recent);

  hseparator3 = gtk_hseparator_new ();
  gtk_widget_show (hseparator3);
  gtk_box_pack_start (GTK_BOX (vbox7), hseparator3, FALSE, TRUE, 0);
  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox7, FALSE, FALSE, 0);

  prefsw->stop_screensaver_check = gtk_check_button_new_with_mnemonic (_("Stop xscreensaver on playback    "));
  gtk_widget_show (prefsw->stop_screensaver_check);
  gtk_box_pack_start (GTK_BOX (hbox7), prefsw->stop_screensaver_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->stop_screensaver_check), 22);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->stop_screensaver_check), prefs->stop_screensaver);

  prefsw->open_maximised_check = gtk_check_button_new_with_mnemonic (_("Open main window maximised"));
  gtk_widget_show (prefsw->open_maximised_check);
  gtk_box_pack_start (GTK_BOX (hbox7), prefsw->open_maximised_check, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->open_maximised_check), prefs->open_maximised);

  pref_gui = gtk_label_new (_("GUI"));
  gtk_widget_show (pref_gui);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 0), pref_gui);
  gtk_label_set_justify (GTK_LABEL (pref_gui), GTK_JUSTIFY_LEFT);

  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox9);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox9);

  frame4 = gtk_frame_new (NULL);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (vbox9), frame4, TRUE, TRUE, 0);

  vbox69=gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox69);
  gtk_container_add (GTK_CONTAINER (frame4), vbox69);
  gtk_container_set_border_width (GTK_CONTAINER (vbox69), 10);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox9);
  gtk_container_add (GTK_CONTAINER (vbox69), hbox9);

  label33 = gtk_label_new (_("Player      "));
  gtk_widget_show (label33);
  gtk_box_pack_start (GTK_BOX (hbox9), label33, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label33), GTK_JUSTIFY_LEFT);

  vidp = g_list_append (vidp, "internal (recommended)");
  vidp = g_list_append (vidp, "mplayer");

  prefsw->vidp_combo = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->vidp_combo), vidp);
  gtk_box_pack_start (GTK_BOX (hbox9), prefsw->vidp_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->vidp_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->vidp_combo))->entry),FALSE);
  if (prefs->video_player==VID_PLAYER_INT) {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->vidp_combo))->entry),"internal (recommended)");
  }
  else {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->vidp_combo))->entry),"mplayer");
  }

  label34 = gtk_label_new (_("     Video play command      "));
  gtk_widget_show (label34);
  gtk_box_pack_start (GTK_BOX (hbox9), label34, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label34), GTK_JUSTIFY_LEFT);

  prefsw->video_command_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->video_command_entry),255);
  gtk_widget_show (prefsw->video_command_entry);
  gtk_box_pack_start (GTK_BOX (hbox9), prefsw->video_command_entry, TRUE, TRUE, 0);

  gtk_entry_set_text(GTK_ENTRY(prefsw->video_command_entry),prefs->video_play_command);

  if (prefs->video_player==VID_PLAYER_INT) {
    gtk_widget_hide(prefsw->video_command_entry);
    gtk_widget_hide(label34);
  }
  
  hbox101 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox101);
  gtk_box_pack_start (GTK_BOX (vbox69), hbox101, FALSE, FALSE, 0);

  prefsw->checkbutton_show_stats = gtk_check_button_new_with_mnemonic ("_Show FPS statistics");
  gtk_widget_show (prefsw->checkbutton_show_stats);
  gtk_box_pack_start (GTK_BOX (hbox101), prefsw->checkbutton_show_stats, FALSE, FALSE, 0);

  gtk_widget_set_sensitive (prefsw->checkbutton_show_stats,prefs->video_player==VID_PLAYER_INT);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_show_stats),prefs->show_player_stats);

  prefsw->checkbutton_no_key_rpt = gtk_check_button_new_with_mnemonic ("Disable fast _key repeat");
  gtk_widget_show (prefsw->checkbutton_no_key_rpt);
  gtk_box_pack_start (GTK_BOX (hbox101), prefsw->checkbutton_no_key_rpt, FALSE, FALSE, 0);

  gtk_widget_set_sensitive (prefsw->checkbutton_no_key_rpt,prefs->video_player!=VID_PLAYER_MPLAYER);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_no_key_rpt),prefs->no_fast_keys);

  hseparator13 = gtk_hseparator_new ();
  gtk_widget_show (hseparator13);
  gtk_box_pack_start (GTK_BOX (vbox69), hseparator13, TRUE, TRUE, 0);

  hbox31 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox31);
  gtk_box_pack_start (GTK_BOX (vbox69), hbox31, FALSE, FALSE, 0);

  label126 = gtk_label_new (_("Plugin      "));
  gtk_widget_show (label126);
  gtk_box_pack_start (GTK_BOX (hbox31), label126, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label126), GTK_JUSTIFY_LEFT);

  pp_combo = gtk_combo_new ();
  gtk_widget_show (pp_combo);
  gtk_box_pack_start (GTK_BOX (hbox31), pp_combo, FALSE, FALSE, 0);

  combo_entry4 = GTK_COMBO (pp_combo)->entry;
  gtk_editable_set_editable (GTK_EDITABLE(combo_entry4),FALSE);
  gtk_widget_show (combo_entry4);

  vid_playback_plugins=get_plugin_list (PLUGIN_VID_PLAYBACK);
  vid_playback_plugins=g_list_prepend (vid_playback_plugins, "None");
  gtk_combo_set_popdown_strings (GTK_COMBO (pp_combo), vid_playback_plugins);
  g_list_free (vid_playback_plugins);

  if (vid_playback_plugin!=NULL) {
    gtk_entry_set_text (GTK_ENTRY(combo_entry4),vid_playback_plugin->name);
  }
  else {
    gtk_entry_set_text (GTK_ENTRY(combo_entry4),"None");
  }

  label128 = gtk_label_new (_("    Host "));
  gtk_widget_show (label128);
  gtk_box_pack_start (GTK_BOX (hbox31), label128, TRUE, TRUE, 6);
  gtk_label_set_justify (GTK_LABEL (label128), GTK_JUSTIFY_LEFT);

  entry20 = gtk_entry_new ();
  gtk_widget_show (entry20);
  gtk_entry_set_max_length(GTK_ENTRY(entry20),15);

  // fixed for now
  gtk_entry_set_text(GTK_ENTRY(entry20),"127.0.0.1");
  gtk_box_pack_start (GTK_BOX (hbox31), entry20, FALSE, FALSE, 0);
  gtk_entry_set_editable (GTK_ENTRY (entry20), FALSE);

  label129 = gtk_label_new (_("    Port"));
  gtk_widget_show (label129);
  gtk_box_pack_start (GTK_BOX (hbox31), label129, TRUE, TRUE, 6);
  gtk_label_set_justify (GTK_LABEL (label129), GTK_JUSTIFY_LEFT);

  entry21 = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(entry21),5);

  // fixed for now
  gtk_entry_set_text(GTK_ENTRY(entry21),"N/A");
  gtk_widget_show (entry21);
  gtk_box_pack_start (GTK_BOX (hbox31), entry21, FALSE, FALSE, 0);
  gtk_entry_set_editable (GTK_ENTRY (entry21), FALSE);

  label31 = gtk_label_new (_("Video"));
  gtk_widget_show (label31);
  gtk_frame_set_label_widget (GTK_FRAME (frame4), label31);
  gtk_label_set_justify (GTK_LABEL (label31), GTK_JUSTIFY_LEFT);

  frame5 = gtk_frame_new (NULL);
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox9), frame5, TRUE, TRUE, 0);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox10);
  gtk_container_add (GTK_CONTAINER (frame5), hbox10);
  gtk_container_set_border_width (GTK_CONTAINER (hbox10), 10);

  label35 = gtk_label_new (_("Player     "));
  gtk_widget_show (label35);
  gtk_box_pack_start (GTK_BOX (hbox10), label35, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label35), GTK_JUSTIFY_LEFT);

  audp = g_list_append (audp, "sox (recommended)");
  audp = g_list_append (audp, "mplayer");

  prefsw->audp_combo = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->audp_combo), audp);
  gtk_box_pack_start (GTK_BOX (hbox10), prefsw->audp_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->audp_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->audp_combo))->entry),FALSE);
  if (prefs->audio_player==AUD_PLAYER_SOX) {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->audp_combo))->entry),"sox (recommended)");
  }
  else {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->audp_combo))->entry),"mplayer");
  }

  label36 = gtk_label_new (_("     Audio play command      "));
  gtk_widget_show (label36);
  gtk_box_pack_start (GTK_BOX (hbox10), label36, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label36), GTK_JUSTIFY_LEFT);

  prefsw->audio_command_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->audio_command_entry),255);
  gtk_widget_show (prefsw->audio_command_entry);
  gtk_box_pack_start (GTK_BOX (hbox10), prefsw->audio_command_entry, TRUE, TRUE, 0);

  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->audio_command_entry),prefs->audio_play_command);

  label32 = gtk_label_new (_("Audio"));
  gtk_widget_show (label32);
  gtk_frame_set_label_widget (GTK_FRAME (frame5), label32);
  gtk_label_set_justify (GTK_LABEL (label32), GTK_JUSTIFY_LEFT);

  label25 = gtk_label_new (_("Playback"));
  gtk_widget_show (label25);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 1), label25);
  gtk_label_set_justify (GTK_LABEL (label25), GTK_JUSTIFY_LEFT);

  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox12);
  gtk_container_set_border_width (GTK_CONTAINER (vbox12), 20);

  hbox11 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox11);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox11, TRUE, TRUE, 0);

  label37 = gtk_label_new (_("      Encoder                  "));
  gtk_widget_show (label37);
  gtk_box_pack_start (GTK_BOX (hbox11), label37, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label37), GTK_JUSTIFY_LEFT);

  vbox13 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox13);
  gtk_box_pack_start (GTK_BOX (hbox11), vbox13, TRUE, TRUE, 0);

  prefsw->encoder_command_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->encoder_command_entry),128);
  prefsw->encoder_combo = gtk_combo_new ();
  gtk_box_pack_start (GTK_BOX (hbox11), prefsw->encoder_combo, FALSE, FALSE, 0);

  prefsw->debug_encoders = gtk_check_button_new_with_mnemonic (_("_Debug Mode"));
  gtk_widget_show (prefsw->debug_encoders);
  gtk_box_pack_start (GTK_BOX (vbox12), prefsw->debug_encoders, TRUE, FALSE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->debug_encoders),prefs->debug_encoders);

  if (capable->has_encoder_plugins==TRUE) {
    // scan for encoder plugins
    if ((encoders=get_plugin_list (PLUGIN_ENCODERS))==NULL) {
      do_plugin_error(PLUGIN_ENCODERS,NULL);
    }
    else {
      gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->encoder_combo), encoders);
      g_list_free (encoders);
    }
  }
  else {
    gtk_widget_set_sensitive (prefsw->debug_encoders,FALSE);
  }

  gtk_widget_show(prefsw->encoder_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->encoder_combo))->entry),FALSE);
  gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->encoder_combo))->entry),prefs->encoder.name);

  label45 = gtk_label_new (_("    Encoder command    "));
  gtk_widget_show (label45);
  gtk_box_pack_start (GTK_BOX (hbox11), label45, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label45), GTK_JUSTIFY_LEFT);

  gtk_widget_show (prefsw->encoder_command_entry);
  gtk_box_pack_start (GTK_BOX (hbox11), prefsw->encoder_command_entry, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(prefsw->encoder_command_entry),prefs->encoder.command);

  hseparator8 = gtk_hseparator_new ();
  gtk_widget_show (hseparator8);
  gtk_box_pack_start (GTK_BOX (vbox12), hseparator8, TRUE, TRUE, 0);

  hbox14 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox14);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox14, TRUE, TRUE, 0);

  label56 = gtk_label_new (_("      Output format          "));
  gtk_widget_show (label56);
  gtk_box_pack_start (GTK_BOX (hbox14), label56, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label56), GTK_JUSTIFY_LEFT);
  
  prefsw->ofmt_combo = gtk_combo_new ();

  if (capable->has_encoder_plugins==TRUE) {
    // reqest formats from the encoder plugin
    if ((ofmt_all=plugin_request(PLUGIN_ENCODERS,prefs->encoder.name,"get_formats"))!=NULL) {
      for (i=0;i<g_list_length(ofmt_all)-3;i+=4) {
	tmp=g_strdup(g_list_nth_data(ofmt_all,i));
	if (!strcmp(tmp,prefs->encoder.of_name)) {
	  g_free(tmp);
	  tmp=g_strdup(g_list_nth_data(ofmt_all,i+2));
	  prefs->encoder.of_allowed_audq=atoi(tmp);
	}
	g_free(tmp);
	ofmt=g_list_append(ofmt,g_strdup(g_list_nth_data(ofmt_all,i+1)));
      }
      memcpy (&future_prefs->encoder,&prefs->encoder,sizeof(_encoder));
      gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->ofmt_combo), ofmt);
      g_list_free(ofmt);
      g_list_free(ofmt_all);
    }
    else {
      do_plugin_error(PLUGIN_ENCODERS,prefs->encoder.name);
    }
  }
    
  gtk_box_pack_start (GTK_BOX (hbox14), prefsw->ofmt_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->ofmt_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->ofmt_combo))->entry),FALSE);
  gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->ofmt_combo))->entry),prefs->encoder.of_name);
  
  label95 = gtk_label_new (_("    Audio Quality       "));
  gtk_widget_show (label95);
  gtk_box_pack_start (GTK_BOX (hbox14), label95, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label95), GTK_JUSTIFY_LEFT);

  vbox15 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox15);
  gtk_box_pack_start (GTK_BOX (hbox14), vbox15, FALSE, FALSE, 0);

  prefsw->rb_aq_high = gtk_radio_button_new_with_mnemonic (NULL, _("_High (pcm)"));
  gtk_widget_show (prefsw->rb_aq_high);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->rb_aq_high), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (prefsw->rb_aq_high), rb_group);
  rb_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (prefsw->rb_aq_high));
  gtk_box_pack_start (GTK_BOX (vbox15), prefsw->rb_aq_high, FALSE, FALSE, 0);
  gtk_widget_set_sensitive(prefsw->rb_aq_high,prefs->encoder.of_allowed_audq&AUD_Q_HIGH);

  prefsw->rb_aq_low = gtk_radio_button_new_with_mnemonic (NULL, _("_Compressed (mp3)"));
  gtk_widget_show (prefsw->rb_aq_low);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->rb_aq_low), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (prefsw->rb_aq_low), rb_group);
  rb_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (prefsw->rb_aq_low));
  gtk_box_pack_start (GTK_BOX (vbox15), prefsw->rb_aq_low, FALSE, FALSE, 0);
  gtk_widget_set_sensitive(prefsw->rb_aq_low,prefs->encoder.of_allowed_audq&AUD_Q_COMP);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(prefsw->rb_aq_low),prefs->encoder.audio_quality==AUD_Q_COMP);

  hseparator4 = gtk_hseparator_new ();
  gtk_widget_show (hseparator4);
  gtk_box_pack_start (GTK_BOX (vbox12), hseparator4, FALSE, TRUE, 0);
  
  hbox12 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox12);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox12, TRUE, TRUE, 0);
  
  label38 = gtk_label_new (_("     Effects command      "));
  gtk_widget_show (label38);
  gtk_box_pack_start (GTK_BOX (hbox12), label38, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label38), GTK_JUSTIFY_LEFT);
  
  prefsw->effects_command_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->effects_command_entry),128);
  gtk_widget_show (prefsw->effects_command_entry);
  gtk_box_pack_start (GTK_BOX (hbox12), prefsw->effects_command_entry, FALSE, TRUE, 0);
  gtk_widget_set_size_request (prefsw->effects_command_entry, 300, -1);

   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->effects_command_entry),prefs->effects_command);

   label26 = gtk_label_new (_("Encoder / Effects"));
   gtk_widget_show (label26);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 2), label26);
   gtk_label_set_justify (GTK_LABEL (label26), GTK_JUSTIFY_LEFT);
   
   table4 = gtk_table_new (7, 3, FALSE);
   gtk_widget_show (table4);
   gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), table4);
   gtk_container_set_border_width (GTK_CONTAINER (table4), 20);
   
   label39 = gtk_label_new (_("      Video load directory (default)     "));
   gtk_widget_show (label39);
   gtk_table_attach (GTK_TABLE (table4), label39, 0, 1, 0, 1,
		     (GtkAttachOptions) (GTK_FILL),
		     (GtkAttachOptions) (GTK_EXPAND), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label39), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (label39), 0, 0.5);

   label40 = gtk_label_new (_("      Video save directory (default)"));
   gtk_widget_show (label40);
   gtk_table_attach (GTK_TABLE (table4), label40, 0, 1, 1, 2,
		     (GtkAttachOptions) (GTK_FILL),
		     (GtkAttachOptions) (GTK_EXPAND), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label40), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (label40), 0, 0.5);
   
   label41 = gtk_label_new (_("      Audio load directory (default)"));
   gtk_widget_show (label41);
   gtk_table_attach (GTK_TABLE (table4), label41, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label41), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (label41), 0, 0.5);

   label42 = gtk_label_new (_("      Image directory (default)"));
   gtk_widget_show (label42);
   gtk_table_attach (GTK_TABLE (table4), label42, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label42), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (label42), 0, 0.5);

   label52 = gtk_label_new (_("      Backup/Restore directory (default)"));
   gtk_widget_show (label52);
   gtk_table_attach (GTK_TABLE (table4), label52, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label52), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (label52), 0, 0.5);
   
   label43 = gtk_label_new (_("      Temp directory"));
   gtk_widget_show (label43);
   gtk_table_attach (GTK_TABLE (table4), label43, 0, 1, 5, 6,
		     (GtkAttachOptions) (GTK_FILL),
		     (GtkAttachOptions) (GTK_EXPAND), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label43), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (label43), 0, 0.5);

   prefsw->vid_load_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->vid_load_dir_entry),255);
   gtk_widget_show (prefsw->vid_load_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->vid_load_dir_entry, 1, 2, 0, 1,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->vid_load_dir_entry),prefs->def_vid_load_dir);
   
   
   prefsw->vid_save_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->vid_save_dir_entry),255);
   gtk_widget_show (prefsw->vid_save_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->vid_save_dir_entry, 1, 2, 2, 3,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->vid_save_dir_entry),prefs->def_vid_save_dir);
   
   prefsw->audio_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->audio_dir_entry),255);
   gtk_widget_show (prefsw->audio_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->audio_dir_entry, 1, 2, 1, 2,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->audio_dir_entry),prefs->def_audio_dir);
   
   prefsw->image_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->image_dir_entry),255);
   gtk_widget_show (prefsw->image_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->image_dir_entry, 1, 2, 3, 4,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->image_dir_entry),prefs->def_image_dir);
   
   prefsw->proj_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->proj_dir_entry),255);
   gtk_widget_show (prefsw->proj_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->proj_dir_entry, 1, 2, 4, 5,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->proj_dir_entry),prefs->def_proj_dir);
   
   prefsw->tmpdir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->tmpdir_entry),255);
   gtk_widget_show (prefsw->tmpdir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->tmpdir_entry, 1, 2, 5, 6,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->tmpdir_entry),future_prefs->tmpdir);
   
   dirbutton1 = gtk_button_new ();
   gtk_widget_show (dirbutton1);
   
   
   dirimage1 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage1);
   gtk_container_add (GTK_CONTAINER (dirbutton1), dirimage1);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton1, 2, 3, 0, 1,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton2 = gtk_button_new ();
   gtk_widget_show (dirbutton2);
   
   
   dirimage2 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage2);
   gtk_container_add (GTK_CONTAINER (dirbutton2), dirimage2);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton2, 2, 3, 1, 2,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton3 = gtk_button_new ();
   gtk_widget_show (dirbutton3);

   
   dirimage3 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage3);
   gtk_container_add (GTK_CONTAINER (dirbutton3), dirimage3);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton3, 2, 3, 2, 3,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton4 = gtk_button_new ();
   gtk_widget_show (dirbutton4);
   
   
   dirimage4 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage4);
   gtk_container_add (GTK_CONTAINER (dirbutton4), dirimage4);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton4, 2, 3, 3, 4,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   

   dirbutton5 = gtk_button_new ();
   gtk_widget_show (dirbutton5);

   
   dirimage5 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage5);
  gtk_container_add (GTK_CONTAINER (dirbutton5), dirimage5);
  
  gtk_table_attach (GTK_TABLE (table4), dirbutton5, 2, 3, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  
  
  dirbutton6 = gtk_button_new ();
  gtk_widget_show (dirbutton6);


  dirimage6 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (dirimage6);
  gtk_container_add (GTK_CONTAINER (dirbutton6), dirimage6);

  gtk_table_attach (GTK_TABLE (table4), dirbutton6, 2, 3, 5, 6,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);


  label27 = gtk_label_new (_("Directories"));
  gtk_widget_show (label27);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 3), label27);
  gtk_label_set_justify (GTK_LABEL (label27), GTK_JUSTIFY_LEFT);


  vbox18 = gtk_vbox_new (FALSE, 10);
  gtk_widget_show (vbox18);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox18);
  gtk_container_set_border_width (GTK_CONTAINER (vbox18), 20);

  prefsw->checkbutton_warn_fps = gtk_check_button_new_with_mnemonic (_("Warn on Insert / Merge if _frame rate of clipboard does not match frame rate of selection"));
  gtk_widget_show (prefsw->checkbutton_warn_fps);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_fps, FALSE, FALSE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_fps), !(prefs->warning_mask&WARN_MASK_FPS));

  hbox100 = gtk_hbox_new(FALSE,10);
  gtk_box_pack_start (GTK_BOX (vbox18), hbox100, FALSE, FALSE, 0);
  gtk_widget_show(hbox100);
  prefsw->checkbutton_warn_fsize = gtk_check_button_new_with_mnemonic (_("Warn on Open if file _size exceeds "));
  gtk_widget_show (prefsw->checkbutton_warn_fsize);
  gtk_box_pack_start (GTK_BOX (hbox100), prefsw->checkbutton_warn_fsize, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_fsize), !(prefs->warning_mask&WARN_MASK_FSIZE));
  spinbutton_warn_fsize_adj = gtk_adjustment_new (prefs->warn_file_size, 1, 2048, 1, 10, 100);

  prefsw->spinbutton_warn_fsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_warn_fsize_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_warn_fsize);
  gtk_box_pack_start (GTK_BOX (hbox100), prefsw->spinbutton_warn_fsize, FALSE, TRUE, 0);

  label100 = gtk_label_new (" MB");
  gtk_widget_show (label100);
  gtk_box_pack_start (GTK_BOX (hbox100), label100, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label100), GTK_JUSTIFY_LEFT);

  prefsw->checkbutton_warn_save_quality = gtk_check_button_new_with_mnemonic (_("Warn about loss of _quality when saving to an existing file"));
  gtk_widget_show (prefsw->checkbutton_warn_save_quality);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_save_quality, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_save_quality), !(prefs->warning_mask&WARN_MASK_SAVE_QUALITY));

  prefsw->checkbutton_warn_set = gtk_check_button_new_with_mnemonic (_("Show a warning before saving a se_t"));
  gtk_widget_show (prefsw->checkbutton_warn_set);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_set, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_set), !(prefs->warning_mask&WARN_MASK_SAVE_SET));

  label86 = gtk_label_new ("");
  gtk_widget_show (label86);
  gtk_box_pack_start (GTK_BOX (vbox18), label86, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label86), GTK_JUSTIFY_LEFT);

  label87 = gtk_label_new ("");
  gtk_widget_show (label87);
  gtk_box_pack_start (GTK_BOX (vbox18), label87, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label87), GTK_JUSTIFY_LEFT);

  label84 = gtk_label_new (_("Warnings"));
  gtk_widget_show (label84);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 4), label84);
  gtk_label_set_justify (GTK_LABEL (label84), GTK_JUSTIFY_LEFT);

  vbox14 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox14);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox14);
  gtk_container_set_border_width (GTK_CONTAINER (vbox14), 20);

  prefsw->check_midi = gtk_check_button_new_with_mnemonic (_("Midi synch (requires the files midistart and midistop)"));
  gtk_widget_show (prefsw->check_midi);
  gtk_box_pack_start (GTK_BOX (vbox14), prefsw->check_midi, FALSE, FALSE, 16);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->check_midi), 10);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->check_midi), prefs->midisynch);
  gtk_widget_set_sensitive(prefsw->check_midi,capable->has_midistartstop);

  hbox99 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox99);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox99, TRUE, TRUE, 0);

  label97 = gtk_label_new ("When inserting/merging frames:  ");
  gtk_widget_show (label97);
  gtk_box_pack_start (GTK_BOX (hbox99), label97, FALSE, FALSE, 16);
  gtk_label_set_justify (GTK_LABEL (label97), GTK_JUSTIFY_LEFT);

  prefsw->ins_speed = gtk_radio_button_new_with_mnemonic (NULL, _("_Speed Up/Slow Down Insertion"));
  gtk_widget_show (prefsw->ins_speed);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->ins_speed), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (prefsw->ins_speed), rb_group2);
  rb_group2 = gtk_radio_button_get_group (GTK_RADIO_BUTTON (prefsw->ins_speed));
  gtk_box_pack_start (GTK_BOX (hbox99), prefsw->ins_speed, FALSE, FALSE, 0);

  ins_resample = gtk_radio_button_new_with_mnemonic (NULL, _("_Resample Insertion"));
  gtk_widget_show (ins_resample);
  gtk_container_set_border_width (GTK_CONTAINER (ins_resample), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (ins_resample), rb_group2);
  rb_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (ins_resample));
  gtk_box_pack_start (GTK_BOX (hbox99), ins_resample, FALSE, FALSE, 0);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ins_resample),prefs->ins_resample);

  prefsw->check_xmms_pause = gtk_check_button_new_with_mnemonic ("Pause xmms during audio playback");
  gtk_box_pack_start (GTK_BOX (vbox14), prefsw->check_xmms_pause, FALSE, FALSE, 16);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->check_xmms_pause), 10);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->check_xmms_pause), prefs->pause_xmms);

  if (capable->has_xmms==TRUE) {
    gtk_widget_show (prefsw->check_xmms_pause);
  }

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox19, TRUE, TRUE, 0);
  
  label134 = gtk_label_new (_("CD device           "));
  gtk_widget_show (label134);
  gtk_box_pack_start (GTK_BOX (hbox19), label134, FALSE, FALSE, 18);
  gtk_label_set_justify (GTK_LABEL (label134), GTK_JUSTIFY_LEFT);
  
  prefsw->cdplay_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->cdplay_entry),255);
  gtk_box_pack_start (GTK_BOX (hbox19), prefsw->cdplay_entry, FALSE, TRUE, 0);

  if (capable->has_cdda2wav==TRUE) {
    gtk_widget_show (prefsw->cdplay_entry);
    gtk_widget_show (hbox19);
  }
  else {
    gtk_widget_hide (prefsw->cdplay_entry);
    gtk_widget_hide (hbox19);
  }

  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->cdplay_entry),prefs->cdplay_device);

  gtk_tooltips_set_tip (tooltips, prefsw->cdplay_entry, "LiVES can load audio tracks from this CD", NULL);

  hbox13 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox13);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox13, TRUE, TRUE, 0);

  label44 = gtk_label_new (_("Default FPS        "));
  gtk_widget_show (label44);
  gtk_box_pack_start (GTK_BOX (hbox13), label44, FALSE, FALSE, 18);
  gtk_label_set_justify (GTK_LABEL (label44), GTK_JUSTIFY_LEFT);

  spinbutton_def_fps_adj = gtk_adjustment_new (prefs->default_fps, 1, 2048, 1, 10, 10);

  prefsw->spinbutton_def_fps = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_def_fps_adj), 1, 2);
  gtk_widget_show (prefsw->spinbutton_def_fps);
  gtk_box_pack_start (GTK_BOX (hbox13), prefsw->spinbutton_def_fps, FALSE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, prefsw->spinbutton_def_fps, _("Frames per second to use when none is specified"), NULL);

  label29 = gtk_label_new (_("Misc"));
  gtk_widget_show (label29);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 5), label29);
  gtk_label_set_justify (GTK_LABEL (label29), GTK_JUSTIFY_LEFT);

  vbox99 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox99);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox99);
  gtk_container_set_border_width (GTK_CONTAINER (vbox99), 20);

  hbox93 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox93);
  gtk_box_pack_start (GTK_BOX (vbox99), hbox93, TRUE, TRUE, 0);

  label94 = gtk_label_new (_("New theme           "));
  gtk_widget_show (label94);
  gtk_box_pack_start (GTK_BOX (hbox93), label94, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label94), GTK_JUSTIFY_LEFT);

  prefsw->theme_combo = gtk_combo_new ();


  // scan for themes
  themes=get_plugin_list (PLUGIN_THEMES);
  themes=g_list_prepend (themes, "none");

  gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->theme_combo), themes);
  g_list_free (themes);

  gtk_box_pack_start (GTK_BOX (hbox93), prefsw->theme_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->theme_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->theme_combo))->entry),FALSE);
  gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->theme_combo))->entry),future_prefs->theme);

  label99 = gtk_label_new (_("Themes"));
  gtk_widget_show (label99);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 6), label99);
  gtk_label_set_justify (GTK_LABEL (label99), GTK_JUSTIFY_LEFT);

  vbox98 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox98);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox98);
  gtk_container_set_border_width (GTK_CONTAINER (vbox98), 20);

  hbox94 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox94);
  gtk_box_pack_start (GTK_BOX (vbox98), hbox94, TRUE, TRUE, 0);

  label88 = gtk_label_new (_("Download bandwidth (Kb/s)       "));
  gtk_widget_show (label88);
  gtk_box_pack_start (GTK_BOX (hbox94), label88, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label88), GTK_JUSTIFY_LEFT);

  spinbutton_bwidth_adj = gtk_adjustment_new (prefs->dl_bandwidth, 0, 100000, 1, 10, 100);

  prefsw->spinbutton_bwidth = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_bwidth_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_bwidth);
  gtk_box_pack_start (GTK_BOX (hbox94), prefsw->spinbutton_bwidth, FALSE, TRUE, 0);

  label98 = gtk_label_new (_("Streaming"));
  gtk_widget_show (label98);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 7), label98);
  gtk_label_set_justify (GTK_LABEL (label98), GTK_JUSTIFY_LEFT);


  dialog_action_area8 = GTK_DIALOG (prefsw->prefs_dialog)->action_area;
  gtk_widget_show (dialog_action_area8);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area8), GTK_BUTTONBOX_END);

  cancelbutton6 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton6);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), cancelbutton6, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton6, GTK_CAN_DEFAULT);

  applybutton = gtk_button_new_from_stock ("gtk-apply");
  gtk_widget_show (applybutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), applybutton, 0);
  GTK_WIDGET_SET_FLAGS (applybutton, GTK_CAN_DEFAULT);

  okbutton5 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton5);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), okbutton5, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton5, GTK_CAN_DEFAULT);


  g_signal_connect(dirbutton1, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->vid_load_dir_entry);
  g_signal_connect(dirbutton2, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->vid_save_dir_entry);
  g_signal_connect(dirbutton3, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->audio_dir_entry);
  g_signal_connect(dirbutton4, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->image_dir_entry);
  g_signal_connect(dirbutton5, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->proj_dir_entry);
  g_signal_connect(dirbutton6, "clicked", G_CALLBACK (on_filesel_complex_clicked),prefsw->tmpdir_entry);


  g_signal_connect (GTK_OBJECT((GTK_COMBO(prefsw->vidp_combo))->entry),"changed",
		    G_CALLBACK (on_vidp_entry_changed),
		    NULL);

  if (capable->has_encoder_plugins==TRUE) {
    prefsw->encoder_name_fn=g_signal_connect (GTK_OBJECT((GTK_COMBO(prefsw->encoder_combo))->entry),"changed",G_CALLBACK (on_encoder_entry_changed),NULL);
    prefsw->encoder_ofmt_fn=g_signal_connect (GTK_OBJECT((GTK_COMBO(prefsw->ofmt_combo))->entry),"changed",G_CALLBACK (on_encoder_ofmt_changed),NULL);
  }
  g_signal_connect (GTK_OBJECT(combo_entry4),"changed",G_CALLBACK (on_vid_playback_plugin_changed),NULL);

  g_signal_connect (GTK_OBJECT (cancelbutton6), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    prefsw);
  g_signal_connect (GTK_OBJECT (okbutton5), "clicked",
		    G_CALLBACK (on_prefs_ok_clicked),
		    NULL);
  g_signal_connect (GTK_OBJECT (applybutton), "clicked",
		    G_CALLBACK (on_prefs_apply_clicked),
		    NULL);
  g_signal_connect (GTK_OBJECT (prefsw->prefs_dialog), "delete_event",
		    G_CALLBACK (on_cancel_button1_clicked),
		    prefsw);

  g_list_free (vidp);
  g_list_free (audp);

  return prefsw;
}



_locw*
create_location_dialog (void)
{
  GtkWidget *dialog_vbox13;
  GtkWidget *hbox19;
  GtkWidget *label83;
  GtkWidget *dialog_action_area13;
  GtkWidget *cancelbutton11;
  GtkWidget *okbutton10;
  GtkWidget *label;

  _locw *locw=(_locw*)(g_malloc(sizeof(_locw)));

  locw->loc_dialog = gtk_dialog_new ();
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(locw->loc_dialog),FALSE);
  }
  gtk_widget_modify_bg (locw->loc_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  gtk_window_set_transient_for(GTK_WINDOW(locw->loc_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_window_set_position (GTK_WINDOW (locw->loc_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (locw->loc_dialog), TRUE);

  gtk_window_set_default_size (GTK_WINDOW (locw->loc_dialog), 300, 200);
  
  gtk_container_set_border_width (GTK_CONTAINER (locw->loc_dialog), 10);
  gtk_window_set_title (GTK_WINDOW (locw->loc_dialog), _("LiVES: - Open Location"));

  dialog_vbox13 = GTK_DIALOG (locw->loc_dialog)->vbox;
  gtk_widget_show (dialog_vbox13);

  label = gtk_label_new ("\n\nTo open a stream, you must make sure that you have the correct libraries compiled in mplayer.\nAlso make sure you have set your bandwidth in Preferences|Streaming");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (dialog_vbox13), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
  gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &palette->normal_fore);

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox19);
  gtk_box_pack_start (GTK_BOX (dialog_vbox13), hbox19, TRUE, TRUE, 0);

  label83 = gtk_label_new ("URL:");
  gtk_widget_show (label83);
  gtk_box_pack_start (GTK_BOX (hbox19), label83, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label83), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label83, GTK_STATE_NORMAL, &palette->normal_fore);

  locw->entry = gtk_entry_new_with_max_length (255);
  gtk_widget_show (locw->entry);
  gtk_box_pack_start (GTK_BOX (hbox19), locw->entry, TRUE, TRUE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY (locw->entry), TRUE);

  dialog_action_area13 = GTK_DIALOG (locw->loc_dialog)->action_area;
  gtk_widget_show (dialog_action_area13);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area13), GTK_BUTTONBOX_END);

  cancelbutton11 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton11);
  gtk_dialog_add_action_widget (GTK_DIALOG (locw->loc_dialog), cancelbutton11, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton11, GTK_CAN_DEFAULT);

  okbutton10 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton10);
  gtk_dialog_add_action_widget (GTK_DIALOG (locw->loc_dialog), okbutton10, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton10, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton10);


  g_signal_connect (GTK_OBJECT (cancelbutton11), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    locw);
  g_signal_connect (GTK_OBJECT (okbutton10), "clicked",
		    G_CALLBACK (on_location_select),
		    NULL);

  g_signal_connect (GTK_OBJECT (locw->loc_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);


  return locw;
}



_renamew*
create_rename_dialog (gint type)
{
  GtkWidget *dialog_vbox13;
  GtkWidget *hbox19;
  GtkWidget *label83;
  GtkWidget *dialog_action_area13;
  GtkWidget *cancelbutton11;
  GtkWidget *okbutton10;

  _renamew *renamew=(_renamew*)(g_malloc(sizeof(_renamew)));

  renamew->rename_dialog = gtk_dialog_new ();

  gtk_window_set_position (GTK_WINDOW (renamew->rename_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(renamew->rename_dialog),GTK_WINDOW(mainw->LiVES));
  gtk_window_set_modal (GTK_WINDOW (renamew->rename_dialog), TRUE);

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(renamew->rename_dialog),FALSE);
  }
  gtk_widget_modify_bg (renamew->rename_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_default_size (GTK_WINDOW (renamew->rename_dialog), 300, 200);

  gtk_container_set_border_width (GTK_CONTAINER (renamew->rename_dialog), 10);

  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (renamew->rename_dialog), _("LiVES: - Rename Clip"));
  }
  else if (type==2||type==3) {
    gtk_window_set_title (GTK_WINDOW (renamew->rename_dialog), _("LiVES: - Enter Set Name"));
  }

  dialog_vbox13 = GTK_DIALOG (renamew->rename_dialog)->vbox;
  gtk_widget_show (dialog_vbox13);

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox19);
  gtk_box_pack_start (GTK_BOX (dialog_vbox13), hbox19, TRUE, TRUE, 0);

  if (type==1) {
    label83 = gtk_label_new ("New name ");
  }
  else if (type==2||type==3) {
    label83 = gtk_label_new ("Set name ");
  }
  else {
    label83 = gtk_label_new ("");
  }
  gtk_widget_show (label83);
  gtk_box_pack_start (GTK_BOX (hbox19), label83, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label83), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label83, GTK_STATE_NORMAL, &palette->normal_fore);

  renamew->entry = gtk_entry_new_with_max_length (255);
  if (type==2&&strlen (mainw->set_name)) {
    gtk_entry_set_text (GTK_ENTRY (renamew->entry),mainw->set_name);
  }

  gtk_widget_show (renamew->entry);
  gtk_box_pack_start (GTK_BOX (hbox19), renamew->entry, TRUE, TRUE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY (renamew->entry), TRUE);

  dialog_action_area13 = GTK_DIALOG (renamew->rename_dialog)->action_area;
  gtk_widget_show (dialog_action_area13);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area13), GTK_BUTTONBOX_END);

  cancelbutton11 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton11);
  gtk_dialog_add_action_widget (GTK_DIALOG (renamew->rename_dialog), cancelbutton11, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton11, GTK_CAN_DEFAULT);

  okbutton10 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton10);
  gtk_dialog_add_action_widget (GTK_DIALOG (renamew->rename_dialog), okbutton10, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton10, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton10);


  g_signal_connect (GTK_OBJECT (cancelbutton11), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    renamew);
  if (type==1) {
    g_signal_connect (GTK_OBJECT (okbutton10), "clicked",
		      G_CALLBACK (on_rename_set_name),
		      NULL);
  }
  else if (type==2) {
    g_signal_connect (GTK_OBJECT (okbutton10), "clicked",
		      G_CALLBACK (on_save_set_ok),
		      NULL);
  }
  else if (type==3) {
    g_signal_connect (GTK_OBJECT (okbutton10), "clicked",
		      G_CALLBACK (on_load_set_ok),
		      NULL);
  }


  g_signal_connect (GTK_OBJECT (renamew->rename_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return renamew;
}





GtkWidget*
create_cdtrack_dialog (gint type)
{
  // type 0 = cd track
  // type 1 = dvd title/chapter
  // type 2 = vcd title -- do we need chapter as well ?

  // TODO - add pref for dvd/vcd device
  // TODO - prompt for audio channel

  // TODO - for CD make this nicer - get track names

  GtkWidget *cd_dialog;
  GtkWidget *dialog_vbox11;
  GtkWidget *hbox16;
  GtkWidget *hbox17;
  GtkWidget *label61;
  GtkWidget *label62;
  GtkObject *spinbutton35_adj;
  GtkWidget *spinbutton35;
  GtkObject *spinbutton36_adj;
  GtkWidget *spinbutton36;
  GtkWidget *dialog_action_area11;
  GtkWidget *cancelbutton9;
  GtkWidget *okbutton8;
  gchar *label;
  
  cd_dialog = gtk_dialog_new ();
  if (type==0) {
    gtk_window_set_title (GTK_WINDOW (cd_dialog), _("LiVES:- Load CD Track"));
  }
  else if (type==1) {
    gtk_window_set_title (GTK_WINDOW (cd_dialog), _("LiVES:- Select DVD Title/Chapter"));
  }
  else {
    gtk_window_set_title (GTK_WINDOW (cd_dialog), _("LiVES:- Select VCD Title"));
  }

  gtk_window_set_position (GTK_WINDOW (cd_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (cd_dialog), TRUE);

  gtk_container_set_border_width (GTK_CONTAINER (cd_dialog), 10);
  gtk_window_set_default_size (GTK_WINDOW (cd_dialog), 300, 200);
  gtk_window_set_transient_for(GTK_WINDOW(cd_dialog),GTK_WINDOW(mainw->LiVES));

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(cd_dialog),FALSE);
  }
  gtk_widget_modify_bg (cd_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  dialog_vbox11 = GTK_DIALOG (cd_dialog)->vbox;
  gtk_widget_show (dialog_vbox11);

  hbox16 = gtk_hbox_new (FALSE, 50);
  gtk_widget_show (hbox16);
  gtk_box_pack_start (GTK_BOX (dialog_vbox11), hbox16, TRUE, TRUE, 0);

  if (type==0) {
    label=g_strdup_printf(_("Track to load (from %s)"),prefs->cdplay_device);
  }
  else if (type==1) {
    label=g_strdup_printf(_("DVD Title"));
  }
  else {
    label=g_strdup_printf(_("VCD Title"));
  }
  label61 = gtk_label_new (label);
  g_free(label);
  gtk_widget_modify_fg (label61, GTK_STATE_NORMAL, &palette->normal_fore);

  gtk_widget_show (label61);
  gtk_box_pack_start (GTK_BOX (hbox16), label61, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label61), GTK_JUSTIFY_LEFT);

  spinbutton35_adj = gtk_adjustment_new (mainw->fx1_val, 1, 256, 1, 10, 10);
  spinbutton35 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton35_adj), 1, 0);
  gtk_widget_show (spinbutton35);
  gtk_box_pack_start (GTK_BOX (hbox16), spinbutton35, FALSE, TRUE, 0);

  hbox17 = gtk_hbox_new (FALSE, 50);
  spinbutton36_adj = gtk_adjustment_new (mainw->fx2_val, 1, 1024, 1, 10, 10);
  spinbutton36 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton36_adj), 1, 0);

  if (type==1) {
    gtk_widget_show (hbox17);
    gtk_box_pack_start (GTK_BOX (dialog_vbox11), hbox17, TRUE, TRUE, 0);

    label62=gtk_label_new(_("Chapter  "));
    gtk_widget_modify_fg (label62, GTK_STATE_NORMAL, &palette->normal_fore);

    gtk_widget_show (label62);
    gtk_box_pack_start (GTK_BOX (hbox17), label62, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label62), GTK_JUSTIFY_LEFT);

    gtk_widget_show (spinbutton36);
    gtk_box_pack_start (GTK_BOX (hbox17), spinbutton36, FALSE, TRUE, 0);
  }

  dialog_action_area11 = GTK_DIALOG (cd_dialog)->action_area;
  gtk_widget_show (dialog_action_area11);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area11), GTK_BUTTONBOX_END);

  cancelbutton9 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton9);
  gtk_dialog_add_action_widget (GTK_DIALOG (cd_dialog), cancelbutton9, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton9, GTK_CAN_DEFAULT);

  okbutton8 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton8);
  gtk_dialog_add_action_widget (GTK_DIALOG (cd_dialog), okbutton8, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton8, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton8);
  
  g_signal_connect_after (GTK_OBJECT (spinbutton35), "value_changed",
			  G_CALLBACK (on_spin_value_changed),
			  GINT_TO_POINTER (1));

  g_signal_connect_after (GTK_OBJECT (spinbutton36), "value_changed",
			  G_CALLBACK (on_spin_value_changed),
			  GINT_TO_POINTER (2));

  g_signal_connect (GTK_OBJECT (cancelbutton9), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    NULL);

  if (type==0) {
    g_signal_connect (GTK_OBJECT (okbutton8), "clicked",
		      G_CALLBACK (on_load_cdtrack_ok_clicked),
		      NULL);
  }
  else {
    g_signal_connect (GTK_OBJECT (okbutton8), "clicked",
		      G_CALLBACK (on_load_vcd_ok_clicked),
		      GINT_TO_POINTER (type));
  }
  return cd_dialog;
}




_xranw *
create_rp_dialog (void)
{
  GtkWidget *dialog_vbox14;
  GtkWidget *vbox19;
  GtkWidget *hbox21;
  GtkWidget *label89;
  GtkObject *spinbutton50_adj;
  GtkWidget *hbox20;
  GtkWidget *label88;
  GtkWidget *button13;
  GtkWidget *image413;
  GtkWidget *hbox22;
  GtkWidget *label90;
  GtkObject *spinbutton51_adj;
  GtkWidget *hbox23;
  GtkWidget *label91;
  GtkObject *spinbutton52_adj;
  GtkWidget *dialog_action_area14;
  GtkWidget *cancelbutton12;
  GtkWidget *okbutton11;

  _xranw *xranw=(_xranw*)(g_malloc(sizeof(_xranw)));

  xranw->rp_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (xranw->rp_dialog), _("LiVES: - Play Random Music"));

  dialog_vbox14 = GTK_DIALOG (xranw->rp_dialog)->vbox;
  gtk_widget_show (dialog_vbox14);
  gtk_window_set_transient_for(GTK_WINDOW(xranw->rp_dialog),GTK_WINDOW(mainw->LiVES));

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(xranw->rp_dialog),FALSE);
  }
  gtk_widget_modify_bg (xranw->rp_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_position (GTK_WINDOW (xranw->rp_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (xranw->rp_dialog), TRUE);

  gtk_container_set_border_width (GTK_CONTAINER (xranw->rp_dialog), 10);
  gtk_window_set_default_size (GTK_WINDOW (xranw->rp_dialog), 600, 300);

  vbox19 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox19);
  gtk_box_pack_start (GTK_BOX (dialog_vbox14), vbox19, TRUE, TRUE, 0);

  hbox21 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox21);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox21, TRUE, TRUE, 0);

  label89 = gtk_label_new (_("Number of Tracks Requested:    "));
  gtk_widget_show (label89);
  gtk_box_pack_start (GTK_BOX (hbox21), label89, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label89), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label89, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton50_adj = gtk_adjustment_new (10, 1, 100, 1, 10, 10);
  xranw->numtracks = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton50_adj), 1, 0);
  gtk_widget_show (xranw->numtracks);
  gtk_box_pack_start (GTK_BOX (hbox21), xranw->numtracks, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (xranw->numtracks), TRUE);
  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (xranw->numtracks), GTK_UPDATE_IF_VALID);

  hbox20 = gtk_hbox_new (FALSE, 16);
  gtk_widget_show (hbox20);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox20, TRUE, TRUE, 0);

  label88 = gtk_label_new (_("Directory:"));
  gtk_widget_show (label88);
  gtk_box_pack_start (GTK_BOX (hbox20), label88, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label88), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label88, GTK_STATE_NORMAL, &palette->normal_fore);

  xranw->dir = gtk_entry_new ();
  gtk_widget_show (xranw->dir);
  gtk_box_pack_start (GTK_BOX (hbox20), xranw->dir, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(xranw->dir),mainw->audio_dir);

  button13 = gtk_button_new ();
  gtk_widget_show (button13);
  gtk_box_pack_start (GTK_BOX (hbox20), button13, FALSE, FALSE, 0);

  image413 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image413);
  gtk_container_add (GTK_CONTAINER (button13), image413);

  xranw->subdir_check = gtk_check_button_new_with_mnemonic (_("Include Subdirectories"));
  gtk_widget_show (xranw->subdir_check);
  gtk_box_pack_start (GTK_BOX (hbox20), xranw->subdir_check, FALSE, FALSE, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(xranw->subdir_check),TRUE);
  gtk_widget_modify_fg (xranw->subdir_check, GTK_STATE_NORMAL, &palette->normal_fore);

  hbox22 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox22);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox22, TRUE, TRUE, 0);

  label90 = gtk_label_new (_("Minimum File Size (MB)         "));
  gtk_widget_show (label90);
  gtk_box_pack_start (GTK_BOX (hbox22), label90, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label90), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label90, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton51_adj = gtk_adjustment_new (2, 0, 100, 1, 10, 10);
  xranw->minsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton51_adj), 1, 0);
  gtk_widget_show (xranw->minsize);
  gtk_box_pack_start (GTK_BOX (hbox22),xranw->minsize, TRUE, FALSE, 0);

  hbox23 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox23);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox23, TRUE, TRUE, 0);

  label91 = gtk_label_new (_("Maximum File Size (MB)        "));
  gtk_widget_show (label91);
  gtk_box_pack_start (GTK_BOX (hbox23), label91, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label91), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label91, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton52_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
  xranw->maxsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton52_adj), 1, 0);
  gtk_widget_show (xranw->maxsize);
  gtk_box_pack_start (GTK_BOX (hbox23), xranw->maxsize, TRUE, FALSE, 0);

  dialog_action_area14 = GTK_DIALOG (xranw->rp_dialog)->action_area;
  gtk_widget_show (dialog_action_area14);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area14), GTK_BUTTONBOX_END);

  cancelbutton12 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton12);
  gtk_dialog_add_action_widget (GTK_DIALOG (xranw->rp_dialog), cancelbutton12, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton12, GTK_CAN_DEFAULT);

  okbutton11 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton11);
  gtk_dialog_add_action_widget (GTK_DIALOG (xranw->rp_dialog), okbutton11, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton11, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton11);

  g_signal_connect(okbutton11, "clicked", G_CALLBACK (on_xmms_ran_ok_clicked),NULL);
  g_signal_connect(cancelbutton12, "clicked", G_CALLBACK (on_cancel_button1_clicked),xranw);
  g_signal_connect(button13, "clicked", G_CALLBACK (on_filesel_simple_clicked),xranw->dir);
  g_signal_connect (GTK_OBJECT (xranw->rp_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return xranw;
}


_commentsw*
create_comments_dialog (void)
{
  GtkWidget *dialog_vbox17;
  GtkWidget *table10;
  GtkWidget *label102;
  GtkWidget *label103;
  GtkWidget *label105;
  GtkWidget *dialog_action_area17;
  GtkWidget *okbutton14;
  GtkWidget *cancelbutton14;

  _commentsw *commentsw=(_commentsw*)(g_malloc(sizeof(_commentsw)));

  commentsw->comments_dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (commentsw->comments_dialog), 10);
  gtk_window_set_title (GTK_WINDOW (commentsw->comments_dialog), _("LiVES: - File Comments (optional)"));
  gtk_window_set_position (GTK_WINDOW (commentsw->comments_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (commentsw->comments_dialog), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(commentsw->comments_dialog),GTK_WINDOW(mainw->LiVES));
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(commentsw->comments_dialog),FALSE);
  }
  gtk_widget_modify_bg (commentsw->comments_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_default_size (GTK_WINDOW (commentsw->comments_dialog), 600, 200);

  dialog_vbox17 = GTK_DIALOG (commentsw->comments_dialog)->vbox;
  gtk_widget_show (dialog_vbox17);

  table10 = gtk_table_new (4, 2, FALSE);
  gtk_widget_show (table10);
  gtk_box_pack_start (GTK_BOX (dialog_vbox17), table10, TRUE, TRUE, 0);

  label102 = gtk_label_new (_("Title/Name : "));
  gtk_widget_show (label102);
  gtk_table_attach (GTK_TABLE (table10), label102, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label102), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label102), 0, 0.5);

  label103 = gtk_label_new (_("Author/Artist : "));
  gtk_widget_show (label103);
  gtk_table_attach (GTK_TABLE (table10), label103, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label103), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label103), 0, 0.5);

  label105 = gtk_label_new (_("Comments : "));
  gtk_widget_show (label105);
  gtk_table_attach (GTK_TABLE (table10), label105, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label105), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label105), 0, 0.5);

  commentsw->title_entry = gtk_entry_new ();
  gtk_widget_show (commentsw->title_entry);
  gtk_table_attach (GTK_TABLE (table10), commentsw->title_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (commentsw->title_entry),cfile->title);

  commentsw->author_entry = gtk_entry_new ();
  gtk_widget_show (commentsw->author_entry);
  gtk_table_attach (GTK_TABLE (table10), commentsw->author_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (commentsw->author_entry),cfile->author);

  commentsw->comment_entry = gtk_entry_new ();
  gtk_widget_show (commentsw->comment_entry);
  gtk_table_attach (GTK_TABLE (table10), commentsw->comment_entry, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_entry_set_max_length (GTK_ENTRY (commentsw->comment_entry), 250);

  gtk_entry_set_text (GTK_ENTRY (commentsw->comment_entry),cfile->comment);

  dialog_action_area17 = GTK_DIALOG (commentsw->comments_dialog)->action_area;
  gtk_widget_show (dialog_action_area17);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area17), GTK_BUTTONBOX_END);

  cancelbutton14 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton14);
  gtk_dialog_add_action_widget (GTK_DIALOG (commentsw->comments_dialog), cancelbutton14, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton14, GTK_CAN_DEFAULT);

  okbutton14 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton14);
  gtk_dialog_add_action_widget (GTK_DIALOG (commentsw->comments_dialog), okbutton14, GTK_RESPONSE_OK);

  GTK_WIDGET_SET_FLAGS (okbutton14, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
  gtk_widget_grab_focus (okbutton14);
  gtk_widget_grab_default (okbutton14);

  return commentsw;
}

_ftrimw*
create_ftrimw (void)
{
  GtkWidget *dialog_vbox18;
  GtkWidget *hbox26;
  GtkWidget *table11;
  GtkObject *spinbutton54_adj;
  GtkObject *spinbutton55_adj;
  GtkObject *spinbutton56_adj;
  GtkObject *spinbutton57_adj;
  GtkWidget *checkbutton12;
  GtkWidget *label111;
  GtkWidget *label113;
  GtkWidget *label112;
  GtkWidget *label110;
  GtkWidget *dialog_action_area18;
  GtkWidget *cancelbutton15;
  GtkWidget *okbutton15;

  _ftrimw *ftrimw=(_ftrimw*)(g_malloc(sizeof(_ftrimw)));

  ftrimw->dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (ftrimw->dialog), _("LiVES: - Trim Frame Size"));

  gtk_widget_modify_bg(ftrimw->dialog, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_transient_for(GTK_WINDOW(ftrimw->dialog),GTK_WINDOW(mainw->LiVES));

  gtk_window_set_modal (GTK_WINDOW (ftrimw->dialog), TRUE);
  gtk_window_set_position (GTK_WINDOW (ftrimw->dialog), GTK_WIN_POS_CENTER);

  dialog_vbox18 = GTK_DIALOG (ftrimw->dialog)->vbox;
  gtk_widget_show (dialog_vbox18);

  hbox26 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox26);
  gtk_box_pack_start (GTK_BOX (dialog_vbox18), hbox26, TRUE, TRUE, 0);

  table11 = gtk_table_new (3, 4, FALSE);
  gtk_widget_show (table11);
  gtk_box_pack_start (GTK_BOX (hbox26), table11, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table11), 10);

  spinbutton54_adj = gtk_adjustment_new (0, 0, cfile->hsize-1, 1, 10, 10);
  ftrimw->spinbutton_x = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton54_adj), 1, 0);
  gtk_widget_show (ftrimw->spinbutton_x);
  gtk_table_attach (GTK_TABLE (table11), ftrimw->spinbutton_x, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton55_adj = gtk_adjustment_new (0, 0, cfile->vsize-1, 1, 10, 10);
  ftrimw->spinbutton_y = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton55_adj), 1, 0);
  gtk_widget_show (ftrimw->spinbutton_y);
  gtk_table_attach (GTK_TABLE (table11), ftrimw->spinbutton_y, 3, 4, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton56_adj = gtk_adjustment_new (cfile->hsize, 1, cfile->hsize, 1, 10, 10);
  ftrimw->spinbutton_width = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton56_adj), 1, 0);
  gtk_widget_show (ftrimw->spinbutton_width);
  gtk_table_attach (GTK_TABLE (table11), ftrimw->spinbutton_width, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton57_adj = gtk_adjustment_new (cfile->vsize, 1, cfile->vsize, 1, 10, 10);
  ftrimw->spinbutton_height = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton57_adj), 1, 0);
  gtk_widget_show (ftrimw->spinbutton_height);
  gtk_table_attach (GTK_TABLE (table11), ftrimw->spinbutton_height, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label111 = gtk_label_new (_("      Y Start   "));
  gtk_widget_show (label111);
  gtk_table_attach (GTK_TABLE (table11), label111, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label111), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label111), 0, 0.5);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label111, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  label113 = gtk_label_new (_("      Height   "));
  gtk_widget_show (label113);
  gtk_table_attach (GTK_TABLE (table11), label113, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label113), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label113), 0, 0.5);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label113, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  label112 = gtk_label_new (_("Width   "));
  gtk_widget_show (label112);
  gtk_table_attach (GTK_TABLE (table11), label112, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 62);
  gtk_label_set_justify (GTK_LABEL (label112), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label112), 0, 0.5);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label112, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  label110 = gtk_label_new (_("X Start   "));
  gtk_widget_show (label110);
  gtk_table_attach (GTK_TABLE (table11), label110, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 42);
  gtk_label_set_justify (GTK_LABEL (label110), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label110), 0, 0.5);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label110, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  checkbutton12 = gtk_check_button_new_with_mnemonic (_("Keep centred"));
  gtk_widget_show (checkbutton12);
  gtk_table_attach (GTK_TABLE (table11), checkbutton12, 0, 4, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 4, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton12), FALSE);

  widget_add_framedraw (ftrimw,GTK_BOX (hbox26),1,cfile->frames, FALSE);
  on_trim_frames_redraw (NULL, GINT_TO_POINTER (1));

  dialog_action_area18 = GTK_DIALOG (ftrimw->dialog)->action_area;
  gtk_widget_show (dialog_action_area18);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area18), GTK_BUTTONBOX_END);

  cancelbutton15 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton15);
  gtk_dialog_add_action_widget (GTK_DIALOG (ftrimw->dialog), cancelbutton15, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton15, GTK_CAN_DEFAULT);

  okbutton15 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton15);
  gtk_dialog_add_action_widget (GTK_DIALOG (ftrimw->dialog), okbutton15, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton15, GTK_CAN_DEFAULT);

  ftrimw->spin_x_func=g_signal_connect_after (GTK_OBJECT (ftrimw->spinbutton_x), "value_changed",
			  G_CALLBACK (on_ftrim_x_changed),
			  NULL);
  ftrimw->spin_y_func=g_signal_connect_after (GTK_OBJECT (ftrimw->spinbutton_y), "value_changed",
					      G_CALLBACK (on_ftrim_y_changed),
					      NULL);
  ftrimw->spin_width_func=g_signal_connect_after (GTK_OBJECT (ftrimw->spinbutton_width), "value_changed",
						  G_CALLBACK (on_ftrim_width_changed),
						  NULL);
  ftrimw->spin_height_func=g_signal_connect_after (GTK_OBJECT (ftrimw->spinbutton_height), "value_changed",
						   G_CALLBACK (on_ftrim_height_changed),
						   NULL);

  g_signal_connect (GTK_OBJECT (checkbutton12), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (1));

  // not used, just an example
  g_signal_connect (GTK_OBJECT (mainw->framedraw_redraw), "clicked",
		    G_CALLBACK (on_trim_frames_redraw),
		    GINT_TO_POINTER (1));

  g_signal_connect (GTK_OBJECT (okbutton15), "clicked",
		    G_CALLBACK (on_trim_frames_ok),
		    NULL);

  g_signal_connect (GTK_OBJECT (cancelbutton15), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      (gpointer)ftrimw);

  return ftrimw;
}




void 
make_preview_box (void) {
  // create a box to show frames in, this will go in the sepwin when we are not playing
  GtkWidget *hbox;
  GtkWidget *hbox_rb;
  GtkWidget *hbox_buttons;
  GtkObject *spinbutton_adj;
  GtkWidget *radiobutton_free;
  GtkWidget *radiobutton_start;
  GtkWidget *radiobutton_end;
  GtkWidget *radiobutton_ptr;
  GtkWidget *preview_scale;
  GSList *radiobutton_group = NULL;
  GtkWidget *hseparator;
  GtkWidget *rewind_img;
  GtkWidget *playsel_img;
  GtkWidget *play_img;
  GtkWidget *loop_img;
  GtkWidget *eventbox;

  gchar buff[256];

  mainw->preview_box = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (mainw->preview_box);
  gtk_widget_ref(mainw->preview_box);

  eventbox=gtk_event_box_new();
  gtk_widget_show (eventbox);
  gtk_box_pack_start (GTK_BOX (mainw->preview_box), eventbox, TRUE, TRUE, 0);

  mainw->preview_image = gtk_image_new_from_pixbuf (NULL);
  gtk_widget_show (mainw->preview_image);
  gtk_container_add (GTK_CONTAINER (eventbox), mainw->preview_image);

  hbox = gtk_hbox_new (FALSE, 10);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (mainw->preview_box), hbox, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);

  spinbutton_adj = gtk_adjustment_new (1, 1, cfile->frames, 1, 10, 0);
  mainw->preview_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
  gtk_entry_set_width_chars (GTK_ENTRY (mainw->preview_spinbutton),8);
  gtk_widget_show (mainw->preview_spinbutton);
  gtk_box_pack_start (GTK_BOX (hbox), mainw->preview_spinbutton, FALSE, FALSE, 0);

  gtk_widget_modify_text(mainw->preview_spinbutton, GTK_STATE_INSENSITIVE, &palette->black);

  preview_scale=gtk_hscale_new_with_range(0,cfile->frames,1);
  gtk_widget_show(preview_scale);
  gtk_box_pack_start (GTK_BOX (hbox), preview_scale, TRUE, TRUE, 0);
  gtk_range_set_adjustment(GTK_RANGE(preview_scale),GTK_ADJUSTMENT(spinbutton_adj));
  gtk_scale_set_draw_value(GTK_SCALE(preview_scale),FALSE);

  hbox_rb = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox_rb);
  gtk_box_pack_start (GTK_BOX (mainw->preview_box), hbox_rb, FALSE, FALSE, 0);

  radiobutton_free = gtk_radio_button_new_with_mnemonic (NULL, 
						      _ ("Free"));
  gtk_widget_show (radiobutton_free);
  gtk_box_pack_start (GTK_BOX (hbox_rb), radiobutton_free, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_free), 
			      radiobutton_group);
  radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON 
						  (radiobutton_free));

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_free), mainw->prv_link==PRV_FREE);

  radiobutton_start = gtk_radio_button_new_with_mnemonic (NULL, 
						      _ ("Start"));
  gtk_widget_show (radiobutton_start);
  gtk_box_pack_start (GTK_BOX (hbox_rb), radiobutton_start, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_start), 
			      radiobutton_group);
  radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON 
						  (radiobutton_start));

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_start), mainw->prv_link==PRV_START);

  radiobutton_end = gtk_radio_button_new_with_mnemonic (NULL, 
							_ ("End"));
  gtk_widget_show (radiobutton_end);
  gtk_box_pack_start (GTK_BOX (hbox_rb), radiobutton_end, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_end), 
			      radiobutton_group);
  radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON 
						  (radiobutton_end));

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_end), mainw->prv_link==PRV_END);

  radiobutton_ptr = gtk_radio_button_new_with_mnemonic (NULL, 
							_ ("Pointer"));
  gtk_widget_show (radiobutton_ptr);
  gtk_box_pack_start (GTK_BOX (hbox_rb), radiobutton_ptr, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_ptr), 
			      radiobutton_group);
  radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON 
						  (radiobutton_ptr));

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_ptr), mainw->prv_link==PRV_PTR);

  hseparator = gtk_hseparator_new ();
  gtk_widget_show (hseparator);
  gtk_box_pack_start (GTK_BOX (mainw->preview_box), hseparator, TRUE, TRUE, 0);

  // buttons
  hbox_buttons = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox_buttons);
  gtk_box_pack_start (GTK_BOX (mainw->preview_box), hbox_buttons, FALSE, FALSE, 0);

  g_snprintf (buff,256,"%s%s/rewind.png",prefs->prefix_dir,ICON_DIR);
  rewind_img=gtk_image_new_from_file (buff);
  mainw->p_rewindbutton=gtk_button_new();
  gtk_widget_modify_bg (mainw->p_rewindbutton, GTK_STATE_ACTIVE, &palette->menu_and_bars);
  gtk_widget_set_size_request (mainw->p_rewindbutton, 28, 18);
  gtk_button_set_relief (GTK_BUTTON (mainw->p_rewindbutton), GTK_RELIEF_NONE);
  gtk_container_add (GTK_CONTAINER(mainw->p_rewindbutton), rewind_img);
  gtk_box_pack_start (GTK_BOX (hbox_buttons), mainw->p_rewindbutton, TRUE, TRUE, 0);
  gtk_widget_show (mainw->p_rewindbutton);
  gtk_widget_show (rewind_img);
  gtk_tooltips_set_tip (mainw->tooltips, mainw->p_rewindbutton,"Rewind", NULL);
  gtk_widget_set_sensitive (mainw->p_rewindbutton, cfile->pointer_time>0.);

  g_snprintf (buff,256,"%s%s/play.png",prefs->prefix_dir,ICON_DIR);
  play_img=gtk_image_new_from_file (buff);
  mainw->p_playbutton=gtk_button_new();
  gtk_widget_modify_bg (mainw->p_playbutton, GTK_STATE_ACTIVE, &palette->menu_and_bars);
  gtk_widget_set_size_request (mainw->p_playbutton, 28, 18);
  gtk_button_set_relief (GTK_BUTTON (mainw->p_playbutton), GTK_RELIEF_NONE);
  gtk_container_add (GTK_CONTAINER(mainw->p_playbutton), play_img);
  gtk_box_pack_start (GTK_BOX (hbox_buttons), mainw->p_playbutton, TRUE, TRUE, 0);
  gtk_widget_show (mainw->p_playbutton);
  gtk_widget_show (play_img);
  gtk_tooltips_set_tip (mainw->tooltips, mainw->p_playbutton,"Play", NULL);
  gtk_widget_set_sensitive (mainw->p_playbutton, TRUE);

  g_snprintf (buff,256,"%s%s/playsel.png",prefs->prefix_dir,ICON_DIR);
  playsel_img=gtk_image_new_from_file (buff);
  mainw->p_playselbutton=gtk_button_new();
  gtk_widget_modify_bg (mainw->p_playselbutton, GTK_STATE_ACTIVE, &palette->menu_and_bars);
  gtk_widget_set_size_request (mainw->p_playselbutton, 28, 18);
  gtk_button_set_relief (GTK_BUTTON (mainw->p_playselbutton), GTK_RELIEF_NONE);
  gtk_container_add (GTK_CONTAINER(mainw->p_playselbutton), playsel_img);
  gtk_box_pack_start (GTK_BOX (hbox_buttons), mainw->p_playselbutton, TRUE, TRUE, 0);
  gtk_widget_show (mainw->p_playselbutton);
  gtk_widget_show (playsel_img);
  gtk_tooltips_set_tip (mainw->tooltips, mainw->p_playselbutton,"Play Selection", NULL);
  gtk_widget_set_sensitive (mainw->p_playselbutton, (mainw->loop==FALSE||mainw->loop_cont==FALSE||prefs->video_player!=VID_PLAYER_MPLAYER)&&cfile->frames>0);

  g_snprintf (buff,256,"%s%s/loop.png",prefs->prefix_dir,ICON_DIR);
  loop_img=gtk_image_new_from_file (buff);
  mainw->p_loopbutton=gtk_button_new();
  gtk_widget_modify_bg (mainw->p_loopbutton, GTK_STATE_ACTIVE, &palette->menu_and_bars);
  gtk_widget_set_size_request (mainw->p_loopbutton, 28, 18);
  gtk_button_set_relief (GTK_BUTTON (mainw->p_loopbutton), GTK_RELIEF_NONE);
  gtk_container_add (GTK_CONTAINER(mainw->p_loopbutton), loop_img);
  gtk_box_pack_start (GTK_BOX (hbox_buttons), mainw->p_loopbutton, TRUE, TRUE, 0);
  gtk_widget_show (mainw->p_loopbutton);
  gtk_widget_show (loop_img);
  gtk_tooltips_set_tip (mainw->tooltips, mainw->p_loopbutton,"Loop On/Off", NULL);
  gtk_widget_set_sensitive (mainw->p_loopbutton, TRUE);

  if (mainw->mute==FALSE) {
    g_snprintf (buff,256,"%s%s/volume.png",prefs->prefix_dir,ICON_DIR);
  }
  else {
    g_snprintf (buff,256,"%s%s/volume_mute.png",prefs->prefix_dir,ICON_DIR);
  }
  mainw->p_mute_img=gtk_image_new_from_file (buff);
  mainw->p_mutebutton=gtk_button_new();
  gtk_widget_modify_bg (mainw->p_mutebutton, GTK_STATE_ACTIVE, &palette->menu_and_bars);
  gtk_widget_set_size_request (mainw->p_mutebutton, 28, 18);
  gtk_button_set_relief (GTK_BUTTON (mainw->p_mutebutton), GTK_RELIEF_NONE);
  gtk_container_add (GTK_CONTAINER(mainw->p_mutebutton), mainw->p_mute_img);
  gtk_box_pack_start (GTK_BOX (hbox_buttons), mainw->p_mutebutton, TRUE, TRUE, 0);
  gtk_widget_show (mainw->p_mutebutton);
  gtk_widget_show (mainw->p_mute_img);
  gtk_tooltips_set_tip (mainw->tooltips, mainw->p_mutebutton,"Mute/Unmute", NULL);
  gtk_widget_set_sensitive (mainw->p_mutebutton, cfile->achans>0);

  g_signal_connect (GTK_OBJECT (radiobutton_free), "toggled",
		    G_CALLBACK (on_prv_link_toggled),
		    GINT_TO_POINTER (PRV_FREE));
  g_signal_connect (GTK_OBJECT (radiobutton_start), "toggled",
		    G_CALLBACK (on_prv_link_toggled),
		    GINT_TO_POINTER (PRV_START));
  g_signal_connect (GTK_OBJECT (radiobutton_end), "toggled",
		    G_CALLBACK (on_prv_link_toggled),
		    GINT_TO_POINTER (PRV_END));
  g_signal_connect (GTK_OBJECT (radiobutton_ptr), "toggled",
		    G_CALLBACK (on_prv_link_toggled),
		    GINT_TO_POINTER (PRV_PTR));

  g_signal_connect (GTK_OBJECT (mainw->p_playbutton), "clicked",
                      G_CALLBACK (on_playall_activate),
                      NULL);
  g_signal_connect (GTK_OBJECT (mainw->p_playselbutton), "clicked",
                      G_CALLBACK (on_playsel_activate),
                      NULL);
  g_signal_connect (GTK_OBJECT (mainw->p_rewindbutton), "clicked",
                      G_CALLBACK (on_rewind_activate),
                      NULL);
  g_signal_connect (GTK_OBJECT (mainw->p_mutebutton), "clicked",
                      G_CALLBACK (on_mute_button_activate),
                      NULL);
  g_signal_connect (GTK_OBJECT (mainw->p_loopbutton), "clicked",
                      G_CALLBACK (on_loop_button_activate),
                      NULL);

  mainw->preview_spin_func=g_signal_connect_after (GTK_OBJECT (mainw->preview_spinbutton), "value_changed",
						   G_CALLBACK (on_preview_spinbutton_changed),
						   NULL);
  g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
                      G_CALLBACK (frame_context),
                      GINT_TO_POINTER (3));

}




void
make_play_window(void) {
  //  separate window
  gint bheight=100;  //approx height of preview_box

  if (prefs->video_player==VID_PLAYER_INT&&mainw->playing_file>-1) {
      unhide_cursor(mainw->playarea->window);
  }

  gtk_image_set_from_pixbuf(GTK_IMAGE(mainw->image274),NULL);

  if (!(mainw->play_window==NULL)) {
    // this shouldn't ever happen
    kill_play_window();
  } 
  
  mainw->play_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_add_accel_group (GTK_WINDOW (mainw->play_window), mainw->accel_group);

  if (prefs->sepwin_type==0) {
    gtk_window_set_title(GTK_WINDOW(mainw->play_window),gtk_window_get_title(GTK_WINDOW(mainw->LiVES)));
  }
  else {
    gtk_window_set_title (GTK_WINDOW (mainw->play_window),"LiVES: - Play Window");
  }

  gtk_widget_modify_bg (mainw->play_window, GTK_STATE_NORMAL, &palette->normal_back);

  // show the window (so we can hide its cursor !), and get its xwin
  if (!(mainw->fs==TRUE&&mainw->playing_file>-1&&vid_playback_plugin!=NULL)) {
    gtk_widget_show(mainw->play_window);
  }

  if (mainw->playing_file==-1&&mainw->current_file>0) {
    if (mainw->preview_box==NULL) {
      // create the preview box that shows frames
      make_preview_box();
    }
    if (cfile->is_loaded) {
      //and add it the play window
      gtk_container_add (GTK_CONTAINER (mainw->play_window), mainw->preview_box);
      
      if (mainw->is_processing&&(mainw->prv_link==PRV_START||mainw->prv_link==PRV_END)) {
	// block spinbutton in play window
	gtk_widget_set_sensitive(mainw->preview_spinbutton,FALSE);
      }

      // load whatever frame and show it
      if (mainw->prv_link!=PRV_FREE) mainw->preview_frame=0;
      load_preview_image(FALSE);
      // force a redraw
      gtk_widget_queue_resize(mainw->preview_box);
      gtk_widget_queue_draw(mainw->preview_box);
      // be careful, the user could switch out of sepwin here !
      while (g_main_context_iteration(NULL,FALSE));
      if (mainw->play_window==NULL) return;
    }
  }

  resize_play_window();
  if (mainw->play_window==NULL) return;

  if (mainw->ext_playback) {
    //approximate...we want to move it though, so it is in the right place 4 later
    gtk_window_move (GTK_WINDOW (mainw->play_window), MAX ((mainw->scr_width-cfile->hsize)/2,0), MAX ((mainw->scr_height-cfile->vsize-bheight)/2,0));
  }
  else {
    // be careful, the user could switch out of sepwin here !
    while (g_main_context_iteration(NULL,FALSE));
    if (mainw->play_window==NULL) return;
    
    gtk_window_move (GTK_WINDOW (mainw->play_window), (mainw->scr_width-mainw->play_window->allocation.width)/2, (mainw->scr_height-mainw->play_window->allocation.height)/2);
    gtk_widget_queue_draw(mainw->play_window);
  }

  mainw->xwin=GDK_WINDOW_XWINDOW (mainw->play_window->window);
    
  if (prefs->video_player==VID_PLAYER_VEEJAY) {
    vj_client_send ("WIDx");
  }

  if ((mainw->current_file==-1||(!cfile->is_loaded&&!mainw->preview)||cfile->frames==0)&&mainw->imframe!=NULL) {
    gdk_draw_pixbuf (GDK_DRAWABLE (mainw->play_window->window),mainw->gc,GDK_PIXBUF (mainw->imframe),0,0,0,0,-1,-1,GDK_RGB_DITHER_NONE,0,0);
  }


  mainw->pw_exp_func=g_signal_connect_after (GTK_OBJECT (mainw->play_window), "expose_event",
					     G_CALLBACK (expose_play_window),
					     NULL);

  g_signal_connect (GTK_OBJECT (mainw->play_window), "delete_event",
		    G_CALLBACK (on_stop_activate_by_del),
		    NULL);

  if ((mainw->current_file>-1&&cfile->is_loaded)||(mainw->preview&&cfile->frames)) g_signal_handler_block(mainw->play_window,mainw->pw_exp_func);

}



void 
resize_play_window (void) {
  gint bx,by;
  gint opwx,opwy;

  if (mainw->play_window==NULL) return;
  if (mainw->current_file==-1||!cfile->frames||(!cfile->is_loaded&&!mainw->preview)) {
    if (!(mainw->imframe==NULL)) {
      mainw->pwidth=gdk_pixbuf_get_width (mainw->imframe);
      mainw->pheight=gdk_pixbuf_get_height (mainw->imframe);
    }
    else {
      mainw->pwidth=DEFAULT_FRAME_HSIZE;
      mainw->pheight=DEFAULT_FRAME_VSIZE;
    }
  }
  else {
    mainw->pwidth=cfile->hsize;
    mainw->pheight=cfile->vsize;
  }
  if (mainw->playing_file>-1) {
    if (mainw->double_size==TRUE) {
      mainw->pheight*=2;
      mainw->pwidth*=2;
    }
   
    if (mainw->fs==TRUE) {
      if (!GTK_WIDGET_VISIBLE (mainw->play_window)) {
	gtk_widget_show (mainw->play_window);
	// be careful, the user could switch out of sepwin here !
	while (g_main_context_iteration (NULL,FALSE));
	if (mainw->play_window==NULL||mainw->fs==FALSE||mainw->playing_file<0) return;
	mainw->opwx=mainw->opwy=-1;
      }
      else {
	mainw->opwx=(mainw->scr_width-mainw->play_window->allocation.width)/2;
	mainw->opwy=(mainw->scr_height-mainw->play_window->allocation.height)/2;
      }
      mainw->pheight=mainw->scr_height;
      mainw->pwidth=mainw->scr_width;

      // init the playback plugin, unless there is a possibility of resized frames
      if (vid_playback_plugin!=NULL&&mainw->preview==FALSE) {
	gdk_window_get_pointer (gdk_get_default_root_window (), &mainw->ptr_x, &mainw->ptr_y, NULL);
	if ((*vid_playback_plugin->init_screen)(mainw->pwidth,mainw->pheight,TRUE)==TRUE) {
	  mainw->ext_playback=TRUE;
	  // the play window is still visible (in case it was 'always on top')
	  // start key polling from ext plugin
	  vid_playback_plugin->kb_timer=gtk_timeout_add (KEY_RPT_INTERVAL,&plugin_poll_keyboard,NULL);
	  return;
	}
      }

      if (GTK_WIDGET_VISIBLE (mainw->play_window)) {
	gtk_window_get_position (GTK_WINDOW (mainw->play_window),&opwx,&opwy);
	if (opwx*opwy) {
	  mainw->opwx=opwx;
	  mainw->opwy=opwy;
	}
      }
      // move window border off screen
      get_border_size (mainw->play_window, &bx, &by);
      gtk_decorated_window_move_resize_window (GTK_WINDOW (mainw->play_window), -bx, -by, mainw->pwidth, mainw->pheight);
      gtk_widget_queue_draw (mainw->play_window);
    }
    gtk_window_present (GTK_WINDOW (mainw->play_window));
  }
  else {
    // not playing
    if (mainw->fs==TRUE&&mainw->playing_file==-2&&(vid_playback_plugin==NULL||mainw->preview==TRUE)&&mainw->sep_win==TRUE&&prefs->sepwin_type==1) {
      if (mainw->opwx>=0&&mainw->opwy>=0) {
	// move window back to its old position after play
	gtk_window_move (GTK_WINDOW (mainw->play_window), mainw->opwx, mainw->opwy);
      }
      else {
	gtk_window_move (GTK_WINDOW (mainw->play_window), (mainw->scr_width-cfile->hsize)/2, (mainw->scr_height-cfile->vsize-100)/2);
      }
      gtk_widget_queue_draw (mainw->play_window);
      mainw->opwx=mainw->opwy=-1;
    }
  }

  if (prefs->video_player==VID_PLAYER_VEEJAY) {
    vj_client_send ("RESIZE");
  }
  gtk_window_resize (GTK_WINDOW (mainw->play_window), mainw->pwidth, mainw->pheight);
}



void
kill_play_window (void) {
  // plug our player back into internal window
  if (prefs->video_player==VID_PLAYER_VEEJAY) {
    vj_client_send ("WID0");
  }

  mainw->xwin=0;

  if (mainw->play_window!=NULL) {
    if (mainw->preview_box!=NULL&&mainw->preview_box->parent!=NULL) {
      gtk_container_remove (GTK_CONTAINER (mainw->play_window), mainw->preview_box);
    }
    if (GTK_IS_WINDOW (mainw->play_window )) gtk_widget_destroy(mainw->play_window);
    mainw->play_window=NULL;
  }
}



void 
add_to_playframe (void) {
  // plug the playback image into its frame in the main window
  gtk_widget_show(mainw->playarea);

  if (!mainw->xwin) {
    mainw->xwin = gtk_socket_get_id(GTK_SOCKET(mainw->playarea));
  }

  ///////////////////////////////////////////////////
  //
  // code here is to plug the internal player into our play socket
  // plug source can be changed by changing prefs->video_player
  if ((prefs->video_player==VID_PLAYER_INT)&&mainw->plug1==NULL) {
    if (mainw->foreign==FALSE&&(mainw->sep_win==FALSE||prefs->sepwin_type==0)) {
      // :-( just creating this stops anything else being shown in the socket
      mainw->plug1 = gtk_plug_new (mainw->xwin);
      gtk_widget_modify_bg (mainw->plug1, GTK_STATE_NORMAL, &palette->normal_back);
      gtk_widget_show (mainw->plug1);
      gtk_container_add (GTK_CONTAINER (mainw->plug1), mainw->image274);
    }
  }
}



